var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { describe, expect, beforeEach, test } from "vitest";
import "reflect-metadata";
import { subDocument, isSubDocument, getSubDocumentProperties, getSubDocuments, clearSubDocumentProperties, injectSubDocuments, parsePath, setNestedSubProperty, recordToISubDocument, } from "./firebase.decorator";
class Address {
    constructor() {
        this.street = "street";
    }
}
__decorate([
    subDocument,
    __metadata("design:type", String)
], Address.prototype, "street", void 0);
class Profile {
    constructor() {
        this.addresses = [];
    }
}
__decorate([
    subDocument,
    __metadata("design:type", Array)
], Profile.prototype, "addresses", void 0);
class User {
    constructor() {
        this.profile = new Profile();
        this.name = "name";
        this.email = "email";
    }
}
__decorate([
    subDocument,
    __metadata("design:type", Profile)
], User.prototype, "profile", void 0);
describe("firebase.decorator tests", () => {
    let user;
    beforeEach(() => {
        user = new User();
        user.profile = new Profile();
        user.profile.addresses = [new Address(), new Address()];
        user.profile.addresses[0].street = "123 Main St";
        user.profile.addresses[1].street = "456 Elm St";
        user.name = "John Doe";
        user.email = "john.doe@example.com";
    });
    test("subDocument decorator should mark properties", () => {
        var _a;
        expect(isSubDocument(user, "profile")).toBe(true);
        expect(isSubDocument(user.profile, "addresses")).toBe(true);
        expect(isSubDocument((_a = user.profile) === null || _a === void 0 ? void 0 : _a.addresses[0], "street")).toBe(true);
        expect(isSubDocument(user, "name")).toBe(false);
        expect(isSubDocument(user, "email")).toBe(false);
    });
    describe("getSubDocumentProperties", () => {
        test("should return all sub-document properties", () => {
            const subDocumentProps = getSubDocumentProperties(user);
            expect(subDocumentProps).toEqual([
                "profile",
                "profile.addresses[0]",
                "profile.addresses[1]",
                "profile.addresses[0].street",
                "profile.addresses[1].street",
            ]);
        });
        test("should return key-value pairs of all sub-document properties", () => {
            var _a, _b;
            const subDocuments = getSubDocuments(user);
            expect(subDocuments).toEqual({
                profile: user.profile,
                "profile.addresses[0]": (_a = user.profile) === null || _a === void 0 ? void 0 : _a.addresses[0],
                "profile.addresses[1]": (_b = user.profile) === null || _b === void 0 ? void 0 : _b.addresses[1],
                "profile.addresses[0].street": "123 Main St",
                "profile.addresses[1].street": "456 Elm St",
            });
        });
        test("getSubDocuments should handle empty sub-documents", () => {
            class EmptyClass {
                constructor() {
                    this.emptyProp = null;
                }
            }
            __decorate([
                subDocument,
                __metadata("design:type", Object)
            ], EmptyClass.prototype, "emptyProp", void 0);
            const instance = new EmptyClass();
            const subDocuments = getSubDocuments(instance);
            expect(subDocuments).toEqual({
                emptyProp: null,
            });
        });
        test("getSubDocuments should handle deeply nested sub-documents", () => {
            class LevelThree {
                constructor(levelThreeProp) {
                    this.notDecorated = "notDecorated 3";
                    this.levelThreeProp = levelThreeProp;
                }
            }
            __decorate([
                subDocument,
                __metadata("design:type", String)
            ], LevelThree.prototype, "levelThreeProp", void 0);
            class LevelTwo {
                constructor(levelThree) {
                    this.notDecorated = "notDecorated 2";
                    this.levelThree = levelThree;
                }
            }
            __decorate([
                subDocument,
                __metadata("design:type", LevelThree)
            ], LevelTwo.prototype, "levelThree", void 0);
            class LevelOne {
                constructor(levelTwo) {
                    this.notDecorated = "notDecorated 1";
                    this.levelTwo = levelTwo;
                }
            }
            __decorate([
                subDocument,
                __metadata("design:type", LevelTwo)
            ], LevelOne.prototype, "levelTwo", void 0);
            const instance = new LevelOne(new LevelTwo(new LevelThree("value")));
            const subDocuments = getSubDocuments(instance);
            expect(subDocuments).toEqual({
                levelTwo: instance.levelTwo,
                "levelTwo.levelThree": instance.levelTwo.levelThree,
                "levelTwo.levelThree.levelThreeProp": "value",
            });
        });
    });
    describe("write SubDocument properties", () => {
        describe("clearSubDocumentProperties", () => {
            test("clearSubDocumentProperties should clear all sub-document properties", () => {
                user.profile2 = new Profile();
                clearSubDocumentProperties(user);
                expect(user.profile).toBeUndefined();
                expect(user.profile2.addresses).toStrictEqual([]);
                expect(user.name).toBe("John Doe");
                expect(user.email).toBe("john.doe@example.com");
            });
            test("clearSubDocumentProperties should clear nested sub-document properties", () => {
                var _a;
                user.profile2 = new Profile();
                clearSubDocumentProperties(user);
                expect(user.profile).toBeUndefined();
                expect((_a = user.profile) === null || _a === void 0 ? void 0 : _a.addresses).toBeUndefined();
            });
            test("clearSubDocumentProperties is allready cleared", () => {
                delete user.profile;
                clearSubDocumentProperties(user);
                expect(user.profile).toBeUndefined();
            });
        });
        test("writeBackSubDocuments should restore sub-document properties", () => {
            var _a, _b;
            const subDocuments = getSubDocuments(user);
            subDocuments["profile.addresses[0].street"] = "789 Oak St";
            const iSubDocs = recordToISubDocument(subDocuments);
            clearSubDocumentProperties(user);
            expect(user.profile).toBeUndefined();
            // Write back subDocuments
            injectSubDocuments(user, iSubDocs);
            // Verify that the sub-document properties are restored
            expect(user.profile).toBeDefined();
            expect((_a = user.profile) === null || _a === void 0 ? void 0 : _a.addresses[0].street).toBe("789 Oak St");
            expect((_b = user.profile) === null || _b === void 0 ? void 0 : _b.addresses[1].street).toBe("456 Elm St");
            expect(user.name).toBe("John Doe");
        });
        describe("parsePath", () => {
            test("parses simple path with only strings", () => {
                const path = "a.b.c";
                const result = parsePath(path);
                expect(result).toEqual(["a", "b", "c"]);
            });
            test("parses path with nested array index 0", () => {
                const path = "a.b.c[0]";
                const result = parsePath(path);
                expect(result).toEqual(["a", "b", "c", 0]);
            });
            test("parses path with nested array index 1", () => {
                const path = "a.b.c[1]";
                const result = parsePath(path);
                expect(result).toEqual(["a", "b", "c", 1]);
            });
            test("parses path with strings and numbers", () => {
                const path = "a[0].b[1]";
                const result = parsePath(path);
                expect(result).toEqual(["a", 0, "b", 1]);
            });
            test("parses path with nested arrays", () => {
                const path = "a[0].b[1].c[2]";
                const result = parsePath(path);
                expect(result).toEqual(["a", 0, "b", 1, "c", 2]);
            });
            test("parses empty string", () => {
                const path = "";
                const result = parsePath(path);
                expect(result).toEqual([]);
            });
        });
        describe("setNestedProperty", () => {
            test("sets value at a simple nested path", () => {
                const obj = {};
                setNestedSubProperty(obj, ["a", "b", "c"], "value");
                expect(obj).toEqual({ a: { b: { c: "value" } } });
            });
            test("sets value at a nested path with arrays", () => {
                const obj = {};
                setNestedSubProperty(obj, ["a", "b", "c", 0], "value");
                expect(obj).toEqual({ a: { b: { c: ["value"] } } });
            });
            test("sets multiple values in nested array", () => {
                const obj = {};
                setNestedSubProperty(obj, ["a", "b", "c", 0], "value1");
                setNestedSubProperty(obj, ["a", "b", "c", 1], "value2");
                setNestedSubProperty(obj, ["a", "b", "c", 2], "value3");
                expect(obj).toEqual({
                    a: {
                        b: {
                            c: ["value1", "value2", "value3"],
                        },
                    },
                });
            });
            test("sets value for editSpecification.clips", () => {
                const obj = {};
                setNestedSubProperty(obj, ["editSpecification", "clips"], [{ id: 1 }, { id: 2 }]);
                expect(obj).toEqual({
                    editSpecification: {
                        clips: [{ id: 1 }, { id: 2 }],
                    },
                });
            });
            test("sets multiple values in editSpecification.clips array", () => {
                const obj = {};
                setNestedSubProperty(obj, ["editSpecification", "clips", 0], { id: 1 });
                setNestedSubProperty(obj, ["editSpecification", "clips", 1], { id: 2 });
                setNestedSubProperty(obj, ["editSpecification", "clips", 2], { id: 3 });
                expect(obj).toEqual({
                    editSpecification: {
                        clips: [{ id: 1 }, { id: 2 }, { id: 3 }],
                    },
                });
            });
            test("sets value at a deeply nested path", () => {
                const obj = {};
                setNestedSubProperty(obj, ["a", "b", "c", "d", "e"], "value");
                expect(obj).toEqual({ a: { b: { c: { d: { e: "value" } } } } });
            });
            test("sets value when intermediate objects/arrays need to be created", () => {
                const obj = { a: { b: {} } };
                setNestedSubProperty(obj, ["a", "b", "c", 1], "value");
                expect(obj).toEqual({ a: { b: { c: [undefined, "value"] } } });
            });
            test("sets value on an empty object", () => {
                const obj = {};
                setNestedSubProperty(obj, [], "value");
                expect(obj).toEqual({});
            });
            test("overwrites existing value", () => {
                const obj = { a: { b: { c: "oldValue" } } };
                setNestedSubProperty(obj, ["a", "b", "c"], "newValue");
                expect(obj).toEqual({ a: { b: { c: "newValue" } } });
            });
            test("handles null and undefined intermediate values", () => {
                const obj = { a: { b: null } };
                setNestedSubProperty(obj, ["a", "b", "c"], "value");
                expect(obj).toEqual({ a: { b: { c: "value" } } });
                const obj2 = { a: { b: undefined } };
                setNestedSubProperty(obj2, ["a", "b", "c"], "value");
                expect(obj2).toEqual({ a: { b: { c: "value" } } });
            });
        });
    });
});
//# sourceMappingURL=firebase.decorator.spec.js.map