import { z } from "zod";
export var Anchor;
(function (Anchor) {
    Anchor["Start"] = "start";
    Anchor["Center"] = "center";
    Anchor["End"] = "end";
})(Anchor || (Anchor = {}));
export const AnchorSchema = z.enum([Anchor.Start, Anchor.Center, Anchor.End]);
export var VideoOrientation;
(function (VideoOrientation) {
    VideoOrientation[VideoOrientation["Portrait"] = 1] = "Portrait";
    VideoOrientation[VideoOrientation["PortraitUpsideDown"] = 2] = "PortraitUpsideDown";
    VideoOrientation[VideoOrientation["LandscapeRight"] = 3] = "LandscapeRight";
    VideoOrientation[VideoOrientation["LandscapeLeft"] = 4] = "LandscapeLeft";
})(VideoOrientation || (VideoOrientation = {}));
export const VideoOrientationSchema = z.union([
    z.literal(VideoOrientation.Portrait),
    z.literal(VideoOrientation.PortraitUpsideDown),
    z.literal(VideoOrientation.LandscapeRight),
    z.literal(VideoOrientation.LandscapeLeft),
]);
export var VideoResolution;
(function (VideoResolution) {
    VideoResolution["_8K"] = "8K";
    VideoResolution["UHD"] = "4K";
    VideoResolution["HD"] = "HD";
    VideoResolution["QHD"] = "QHD";
    VideoResolution["SD"] = "SD";
})(VideoResolution || (VideoResolution = {}));
export const VideoResolutionSchema = z.enum([
    VideoResolution._8K, // "8K"
    VideoResolution.UHD, // "4K"
    VideoResolution.HD, // "HD"
    VideoResolution.QHD, // "QHD"
    VideoResolution.SD, // "SD"
]);
// "h264" | "vp8" | "vp9" | "mp3" | "aac" | "wav" | "gif" | "prores"
// "h264" | "h265" | "vp8" | "vp9" | "mp3" | "aac" | "wav" | "prores" | "h264-mkv" | "gif" |
export var VideoCodecType;
(function (VideoCodecType) {
    VideoCodecType["H264"] = "h264";
    VideoCodecType["H264MKV"] = "h264-mkv";
    VideoCodecType["H265"] = "h265";
    VideoCodecType["ProRes"] = "prores";
    VideoCodecType["VP8"] = "vp8";
    VideoCodecType["VP9"] = "vp9";
    VideoCodecType["GIF"] = "gif";
})(VideoCodecType || (VideoCodecType = {}));
export const VideoCodecTypeSchema = z.enum([
    VideoCodecType.H264, // "h264"
    VideoCodecType.H264MKV, // "h264-mkv"
    VideoCodecType.H265, // "h265"
    VideoCodecType.ProRes, // "prores"
    VideoCodecType.VP8, // "vp8"
    VideoCodecType.VP9, // "vp9"
    VideoCodecType.GIF, // "gif"
]);
export var AudioCodecType;
(function (AudioCodecType) {
    AudioCodecType["AAC"] = "aac";
    AudioCodecType["WAV"] = "wav";
    AudioCodecType["MP3"] = "mp3";
})(AudioCodecType || (AudioCodecType = {}));
export const AudioCodecTypeSchema = z.enum([
    AudioCodecType.AAC, // "aac"
    AudioCodecType.WAV, // "wav"
    AudioCodecType.MP3, // "mp3"
]);
export var VideoImageFormatType;
(function (VideoImageFormatType) {
    VideoImageFormatType["JPEG"] = "jpeg";
    VideoImageFormatType["PNG"] = "png";
    VideoImageFormatType["NONE"] = "none";
})(VideoImageFormatType || (VideoImageFormatType = {}));
export const VideoImageFormatTypeSchema = z.enum([
    VideoImageFormatType.JPEG, // "jpeg"
    VideoImageFormatType.PNG, // "png"
    VideoImageFormatType.NONE, // "none"
]);
export var StillImageFormatType;
(function (StillImageFormatType) {
    StillImageFormatType["PNG"] = "png";
    StillImageFormatType["JPEG"] = "jpeg";
    StillImageFormatType["PDF"] = "pdf";
    StillImageFormatType["WEBP"] = "webp";
})(StillImageFormatType || (StillImageFormatType = {}));
export const StillImageFormatTypeSchema = z.enum([
    StillImageFormatType.PNG, // "png"
    StillImageFormatType.JPEG, // "jpeg"
    StillImageFormatType.PDF, // "pdf"
    StillImageFormatType.WEBP, // "webp"
]);
export var PixelFormat;
(function (PixelFormat) {
    PixelFormat["yuv420p"] = "yuv420p";
    PixelFormat["yuva420p"] = "yuva420p";
    PixelFormat["yuv422p"] = "yuv422p";
    PixelFormat["yuv444p"] = "yuv444p";
    PixelFormat["yuv420p10le"] = "yuv420p10le";
    PixelFormat["yuv422p10le"] = "yuv422p10le";
    PixelFormat["yuv444p10le"] = "yuv444p10le";
    PixelFormat["yuva444p10le"] = "yuva444p10le";
})(PixelFormat || (PixelFormat = {}));
export const PixelFormatSchema = z.enum([
    PixelFormat.yuv420p,
    PixelFormat.yuva420p,
    PixelFormat.yuv422p,
    PixelFormat.yuv444p,
    PixelFormat.yuv420p10le,
    PixelFormat.yuv422p10le,
    PixelFormat.yuv444p10le,
    PixelFormat.yuva444p10le,
]);
export var ProResProfile;
(function (ProResProfile) {
    ProResProfile["ProRes444"] = "4444";
    ProResProfile["ProRes444XQ"] = "4444-xq";
    ProResProfile["HQ"] = "hq";
    ProResProfile["Standard"] = "standard";
    ProResProfile["Light"] = "light";
    ProResProfile["Proxy"] = "proxy";
})(ProResProfile || (ProResProfile = {}));
export const ProResProfileSchema = z.enum([
    ProResProfile.ProRes444, // "4444"
    ProResProfile.ProRes444XQ, // "4444-xq"
    ProResProfile.HQ, // "hq"
    ProResProfile.Standard, // "standard"
    ProResProfile.Light, // "light"
    ProResProfile.Proxy, // "proxy"
]);
export var VideoFileType;
(function (VideoFileType) {
    VideoFileType["mp4"] = "mp4";
    VideoFileType["mov"] = "mov";
    VideoFileType["mkv"] = "mkv";
    VideoFileType["webm"] = "webm";
    VideoFileType["hevc"] = "hevc";
})(VideoFileType || (VideoFileType = {}));
export const VideoFileTypeSchema = z.enum([
    VideoFileType.mp4, // "mp4"
    VideoFileType.mov, // "mov"
    VideoFileType.mkv, // "mkv"
    VideoFileType.webm, // "webm"
    VideoFileType.hevc, // "hevc"
]);
export var VideoRatioType;
(function (VideoRatioType) {
    VideoRatioType[VideoRatioType["R16To9"] = 1.7777777777777777] = "R16To9";
    VideoRatioType[VideoRatioType["R9To16"] = 0.5625] = "R9To16";
    VideoRatioType[VideoRatioType["R4To3"] = 1.3333333333333333] = "R4To3";
    VideoRatioType[VideoRatioType["R3To4"] = 0.75] = "R3To4";
    VideoRatioType[VideoRatioType["R5To4"] = 1.25] = "R5To4";
    VideoRatioType[VideoRatioType["R1To1"] = 1] = "R1To1";
    VideoRatioType[VideoRatioType["RCustom"] = 0] = "RCustom";
})(VideoRatioType || (VideoRatioType = {}));
export const VideoRatioTypeSchema = z.union([
    z.literal(VideoRatioType.R16To9),
    z.literal(VideoRatioType.R9To16),
    z.literal(VideoRatioType.R4To3),
    z.literal(VideoRatioType.R3To4),
    z.literal(VideoRatioType.R5To4),
    z.literal(VideoRatioType.R1To1),
    z.literal(VideoRatioType.RCustom),
]);
export var VideoFrameRate;
(function (VideoFrameRate) {
    VideoFrameRate[VideoFrameRate["Fps24"] = 24] = "Fps24";
    VideoFrameRate[VideoFrameRate["Fps25"] = 25] = "Fps25";
    VideoFrameRate[VideoFrameRate["Fps30"] = 30] = "Fps30";
    VideoFrameRate[VideoFrameRate["Fps50"] = 50] = "Fps50";
    VideoFrameRate[VideoFrameRate["Fps60"] = 60] = "Fps60";
    VideoFrameRate[VideoFrameRate["Custom"] = 0] = "Custom";
})(VideoFrameRate || (VideoFrameRate = {}));
export const VideoFrameRateSchema = z.union([
    z.literal(VideoFrameRate.Fps24),
    z.literal(VideoFrameRate.Fps25),
    z.literal(VideoFrameRate.Fps30),
    z.literal(VideoFrameRate.Fps50),
    z.literal(VideoFrameRate.Fps60),
    z.literal(VideoFrameRate.Custom),
]);
export var LayerType;
(function (LayerType) {
    LayerType["Audio"] = "audio";
    LayerType["Video"] = "video";
    LayerType["Image"] = "image";
    LayerType["ImageOverlay"] = "image-overlay";
    LayerType["Title"] = "title";
    LayerType["Subtitle"] = "subtitle";
    LayerType["NewsTitle"] = "news-title";
    LayerType["SlideInText"] = "slide-in-text";
    LayerType["LowerThird"] = "lower-third";
    LayerType["TitleBackground"] = "title-background";
    LayerType["FillColor"] = "fill-color";
    LayerType["Pause"] = "pause";
    LayerType["RadialGradient"] = "radial-gradient";
    LayerType["LinearGradient"] = "linear-gradient";
    LayerType["RainbowColors"] = "rainbow-colors";
    LayerType["Canvas"] = "canvas";
    LayerType["Fabric"] = "fabric";
    LayerType["Gl"] = "gl";
    LayerType["Lottie"] = "lottie";
})(LayerType || (LayerType = {}));
export const LayerTypeSchema = z.enum([
    LayerType.Audio,
    LayerType.Video,
    LayerType.Image,
    LayerType.ImageOverlay,
    LayerType.Title,
    LayerType.Subtitle,
    LayerType.NewsTitle,
    LayerType.SlideInText,
    LayerType.LowerThird,
    LayerType.TitleBackground,
    LayerType.FillColor,
    LayerType.Pause,
    LayerType.RadialGradient,
    LayerType.LinearGradient,
    LayerType.RainbowColors,
    LayerType.Canvas,
    LayerType.Fabric,
    LayerType.Gl,
    LayerType.Lottie,
]);
export var FadeType;
(function (FadeType) {
    FadeType["None"] = "none";
    FadeType["Cut"] = "cut";
    FadeType["Linear"] = "linear";
    FadeType["Exponential"] = "exponential";
    FadeType["Cubic"] = "cubic";
})(FadeType || (FadeType = {}));
export const FadeTypeSchema = z.enum([
    FadeType.None, // "none"
    FadeType.Cut, // "cut"
    FadeType.Linear, // "linear"
    FadeType.Exponential, // "exponential"
    FadeType.Cubic, // "cubic"
]);
export var SelectorType;
(function (SelectorType) {
    SelectorType["Language"] = "language";
    SelectorType["Languages"] = "languages";
    SelectorType["IsPlayer"] = "isPlayer";
    SelectorType["IsPreview"] = "isPreview";
    SelectorType["LogLevel"] = "logLevel";
    SelectorType["CalculateClips"] = "calculateClips";
    SelectorType["CalculateFromJson"] = "calculateFromJson";
})(SelectorType || (SelectorType = {}));
export const SelectorTypeSchema = z.enum([
    SelectorType.Language, // "language"
    SelectorType.IsPlayer, // "isPlayer"
    SelectorType.IsPreview, // "isPlayer"
    SelectorType.LogLevel, // "logLevel"
    SelectorType.CalculateClips, // "calculateClips"
    SelectorType.CalculateFromJson, // "calculateFromJson"
]);
export var GroupType;
(function (GroupType) {
    GroupType["CurrentUser"] = "1CurrentUser";
    GroupType["Groups"] = "2Groups";
    GroupType["ActiveProject"] = "3ActiveProject";
    GroupType["RenderedVideoFromActiveProject"] = "4RenderedVideoFromActiveProject";
})(GroupType || (GroupType = {}));
export var AnchorType;
(function (AnchorType) {
    AnchorType["Null"] = "null";
    AnchorType["Top"] = "top";
    AnchorType["Bottom"] = "bottom";
    AnchorType["Center"] = "center";
    AnchorType["Right"] = "right";
    AnchorType["Left"] = "left";
})(AnchorType || (AnchorType = {}));
export const AncorTypeSchema = z.enum([
    AnchorType.Null, // "null"
    AnchorType.Top, // "top"
    AnchorType.Bottom, // "bottom"
    AnchorType.Center, // "center"
    AnchorType.Right, // "right"
    AnchorType.Left, // "left"
]);
export var PositionType;
(function (PositionType) {
    PositionType["Static"] = "static";
    PositionType["Relative"] = "relative";
    PositionType["Fixed"] = "fixed";
    PositionType["Absolute"] = "absolute";
    PositionType["Sticky"] = "sticky";
})(PositionType || (PositionType = {}));
export const PositionTypeSchema = z.enum([
    PositionType.Static, // "static"
    PositionType.Relative, // "relative"
    PositionType.Fixed, // "fixed"
    PositionType.Absolute, // "absolute"
    PositionType.Sticky, // "sticky"
]);
export var AspectType;
(function (AspectType) {
    AspectType["Fill"] = "fill";
    AspectType["Fit"] = "fit";
})(AspectType || (AspectType = {}));
export const AspectTypeSchema = z.enum([
    AspectType.Fill, // "fill"
    AspectType.Fit, // "fit"
]);
export var TransitionCurve;
(function (TransitionCurve) {
    TransitionCurve["Cut"] = "cut";
    TransitionCurve["Linear"] = "linear";
    TransitionCurve["Cubic"] = "cubic";
    TransitionCurve["Quad"] = "quad";
})(TransitionCurve || (TransitionCurve = {}));
export const TransitionCurveSchema = z.enum([
    TransitionCurve.Cut, // "cut"
    TransitionCurve.Linear, // "linear"
    TransitionCurve.Cubic, // "cubic"
    TransitionCurve.Quad, // "quad"
]);
export var TimingAction;
(function (TimingAction) {
    TimingAction["Trim"] = "trim";
    TimingAction["Expand"] = "expand";
    TimingAction["Shift"] = "shift";
})(TimingAction || (TimingAction = {}));
export const TimingActionSchema = z.enum([
    TimingAction.Trim, // "trim"
    TimingAction.Expand, // "expand"
    TimingAction.Shift, // "shift"
]);
//# sourceMappingURL=general.js.map