/* eslint-disable capitalized-comments */
/* eslint-disable @remotion/deterministic-randomness */
// declare global {
//   interface StringConstructor {
//     makeId(length: number): string;
//     // ToBase64(str: string): string;
//     // fromBase64(str: string): string;
//   }
//   interface String {
//     makeId(length: number): string;
//     isValidUrl(): boolean;
//     isBucketLink(): boolean;
//     trimBucketName(bucketName: string): string;
//     cleanUp(): string | undefined;
//     fileExtention(): string | undefined;
//     fileName(): string | undefined;
//     // ReplaceExtention(extention: string): string;
//     // toFilename(): string;
//     // toFilename(suffix: string | null): string;
//     // filenameFromUrl(): string | undefined;
//     // filenameFromUrl(suffix: string | null): string;
//     // toUniqueFilename(): string;
//   }
// }
// String.prototype.makeId = (length: number): string => {
//   let result = "";
//   const characters =
//     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
//   const charactersLength = characters.length;
//   for (let i = 0; i < length; i++) {
//     result += characters.charAt(Math.floor(Math.random() * charactersLength));
//   }
//   return result;
// };
// String.prototype.isValidUrl = function (): boolean {
//   if (!this) return false;
//   const pattern = new RegExp(
//     "^(https?:\\/\\/)?" + // Protocol
//       "((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|" + // Domain name
//       "((\\d{1,3}\\.){3}\\d{1,3}))" + // OR ip (v4) address
//       "(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*" + // Port and path
//       "(\\?[;&a-z\\d%_.~+=-]*)?" + // Query string
//       "(\\#[-a-z\\d_]*)?$",
//     "i"
//   ); // Fragment locator
//   return Boolean(pattern.test(this.toString()));
// };
// String.prototype.isBucketLink = function (): boolean {
//   if (!this) return false;
//   return this.startsWith("gs://");
// };
// String.prototype.trimBucketName = function (bucketName): string {
//   if (!this) return this;
//   return this.replace(`gs://${bucketName}/`, "");
// };
// String.prototype.cleanUp = function (): string {
//   if (!this) return this;
//   const res = this.replace("&", '"&"');
//   return res;
// };
// String.prototype.fileExtention = function (): string {
//   const res = getExtension(this.toString());
//   return res;
// };
// String.prototype.fileName = function (): string {
//   const res = getFilename(this.toString());
//   return res;
// };
export const makeId = (length) => {
    let result = "";
    const characters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
    const charactersLength = characters.length;
    for (let i = 0; i < length; i++) {
        result += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return result;
};
export const toBase64 = (value) => {
    return Buffer.from(value).toString("base64");
};
export const fromBase64 = (value) => {
    return Buffer.from(value, "base64").toString();
};
const isSeparator = (value) => value === "/" || value === "\\" || value === ":";
export const getExtension = (path) => {
    for (let i = path.length - 1; i > -1; --i) {
        const value = path[i];
        if (value === ".") {
            if (i > 1) {
                if (isSeparator(path[i - 1])) {
                    return "";
                }
                return path.substring(i + 1);
            }
            return "";
        }
        if (isSeparator(value)) {
            return "";
        }
    }
    return "";
};
export const getFilename = (path) => {
    for (let i = path.length - 1; i > -1; --i) {
        const value = path[i];
        if (isSeparator(value)) {
            return path.substring(i + 1);
        }
    }
    return path;
};
export const isValidUrl = function (string) {
    const pattern = new RegExp("^(https?:\\/\\/)?" + // Protocol
        "((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|" + // Domain name
        "((\\d{1,3}\\.){3}\\d{1,3}))" + // OR ip (v4) address
        "(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*" + // Port and path
        "(\\?[;&a-z\\d%_.~+=-]*)?" + // Query string
        "(\\#[-a-z\\d_]*)?$", "i"); // Fragment locator
    return Boolean(pattern.test(string));
};
export const isBucketLink = function (string) {
    return string.startsWith("gs://");
};
export const trimBucketName = function (string, bucketName) {
    return string.replace(`gs://${bucketName}/`, "");
};
export const cleanUp = function (string) {
    const res = string.replace("&", '"&"');
    return res;
};
export const fileExtention = function (string) {
    const res = getExtension(string);
    return res;
};
export const fileName = function (string) {
    const res = getFilename(string);
    return res;
};
export function compareJsonStrings(json1, json2) {
    if (!json1 || !json2)
        return false;
    try {
        const obj1 = JSON.parse(json1);
        const obj2 = JSON.parse(json2);
        return deepEqual(obj1, obj2);
    }
    catch (error) {
        console.error("Error parsing JSON:", error);
        return false;
    }
}
function deepEqual(obj1, obj2) {
    if (obj1 === obj2) {
        return true;
    }
    if (typeof obj1 !== "object" ||
        typeof obj2 !== "object" ||
        obj1 === null ||
        obj2 === null) {
        return false;
    }
    const keys1 = Object.keys(obj1);
    const keys2 = Object.keys(obj2);
    if (keys1.length !== keys2.length) {
        return false;
    }
    for (const key of keys1) {
        if (!keys2.includes(key) || !deepEqual(obj1[key], obj2[key])) {
            return false;
        }
    }
    return true;
}
export function getFirestoreCollectionPath(path) {
    // trim the trailing slash and all char after it
    const pathSegments = path.split("/");
    pathSegments.pop();
    return pathSegments.join("/");
}
export function getFirestoreId(path) {
    var _a;
    // substring after the last slash
    const pathSegments = path.split("/");
    return (_a = pathSegments.pop()) !== null && _a !== void 0 ? _a : undefined;
}
//# sourceMappingURL=string.extensions.js.map