import { ITranscriptionResult } from "../models/webVTT";
export interface IGladiaTranscript {
    id: string;
    created_at: string;
    completed_at: string;
    kind: string;
    request_id: string;
    request_params: any;
    result: IGladiaTranscriptionResult;
    status: string;
}
export interface IGladiaTranscriptionResult {
    metadata: IGladiaMetadata;
    transcription: IGladiaTranscription;
    translation: IGladiaTranslation;
    summarization: IGladiaSummarization;
    moderation: IGladiaModeration;
    audio_to_llm: IGladiaAudioToLlm;
}
export interface IGladiaTranscription {
    full_transcript: string;
    languages: string[];
    subtitles: IGladiaSubtitle[];
    utterances: IGladiaUtterance[];
    sentences: IGladiaSentences[];
}
export interface IGladiaWord {
    word: string;
    start: number;
    end: number;
    confidence: number;
}
export interface IGladiaUtterance {
    text: string;
    language: string;
    start: number;
    end: number;
    confidence: number;
    channel: number;
    speaker: number;
    words: IGladiaWord[];
}
export interface IGladiaSubtitle {
    format: string;
    subtitles: string;
}
export interface IGladiaError {
    status_code: number;
    exception: string;
    message: string;
}
export interface IGladiaTranslation {
    success: boolean;
    is_empty: boolean;
    exec_time: number;
    error: IGladiaError;
    results: IGladiaResults[];
}
export interface IGladiaResults {
    error?: string | null;
    full_transcript: string;
    languages: string[];
    sentences: IGladiaSentences[];
    subtitles: IGladiaSubtitle[];
    utterances: IGladiaUtterance[];
}
export interface IGladiaSummarization {
    success: boolean;
    is_empty: boolean;
    exec_time: number;
    error: IGladiaError;
    results: string;
}
export interface IGladiaModeration {
    success: boolean;
    is_empty: boolean;
    exec_time: number;
    error: IGladiaError;
    results: string;
}
export interface IGladiaAudioToLlm {
    success: boolean;
    is_empty: boolean;
    exec_time: number;
    error: IGladiaError;
    results: IGladiaLLMResults;
}
export interface IGladiaLLMResults {
    prompt: string;
    response: string;
}
export interface IGladiaSentences {
    success: boolean;
    is_empty: boolean;
    exec_time: number;
    error: IGladiaError;
    results: IGladiaSentence[];
}
export interface IGladiaSentence {
    channel: number;
    confidence: number;
    start: number;
    end: number;
    sentence: string;
    language: string;
    speaker: number;
    words: IGladiaWord[];
}
export interface IGladiaMetadata {
    audio_duration: number;
    number_of_distinct_channels: number;
    billing_time: number;
    transcription_time: number;
}
export declare function IGladiaTranscriptionToITranscriptionResult(data: IGladiaTranscription): ITranscriptionResult;
//# sourceMappingURL=IGladiaTranscript.d.ts.map