var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios from "axios";
import { DEFAULT_AUDIO_TRANSCRIPTION_CONFIG, TranscriptionStatus, } from "./interfaces.js";
const TIMEOUT = 10 * 60 * 1000;
export class GladiaClient {
    constructor(apiKey) {
        if (!apiKey)
            throw new Error("API key is required to use Gladia's API");
        this.apiKey = apiKey;
        this.baseUrl = "https://api.gladia.io/v2/";
        this.client = axios.create({
            baseURL: this.baseUrl,
            headers: {
                "x-gladia-key": this.apiKey,
                "Content-Type": "application/json",
            },
        });
    }
    // async uploadLocalMedia(filePath: string): Promise<IAudioUploadResponse> {
    //   fs.access(filePath, fs.constants.F_OK, async (err) => {
    //     if (err) {
    //       console.log("gladia - File does not exist", err);
    //       throw new Error(
    //         `File access denied or dont exist: ${filePath}:  ${err.message}`
    //       );
    //     }
    //   });
    //   const mediaType = getMediaType(filePath);
    //   const buffer = fs.readFileSync(filePath);
    //   const blob = new Blob([buffer], { type: mediaType });
    //   return await this.uploadMedia(blob);
    // }
    uploadMedia(file) {
        return __awaiter(this, void 0, void 0, function* () {
            const formData = new FormData();
            formData.append("file", file);
            try {
                const response = yield this.client.post("/upload", formData, {
                    headers: {
                        "Content-Type": "multipart/form-data",
                    },
                });
                return response.data;
            }
            catch (error) {
                throw new Error(`Error uploading audio: ${error}`);
            }
        });
    }
    transcribeAudioFromUrl(audioUrl, transcriptionConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = Object.assign(Object.assign(Object.assign({}, DEFAULT_AUDIO_TRANSCRIPTION_CONFIG), transcriptionConfig), { audio_url: audioUrl });
            try {
                const response = yield this.client.post("/transcription", config);
                return response.data;
            }
            catch (error) {
                return {
                    message: `Error transcribing audio from URL: ${error}`,
                };
            }
        });
    }
    getTranscriptions(offset = 0, limit = 50, status, kind, date, before_date, after_date) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.client.get("/transcription", {
                    params: {
                        offset,
                        limit,
                        status,
                        kind,
                        date,
                        before_date,
                        after_date,
                    },
                });
                return response.data;
            }
            catch (error) {
                throw new Error(`Error getting list of transcriptions: ${error}`);
            }
        });
    }
    getTranscriptionState(id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.client.get(`/transcription/${id}`);
                return response.data;
            }
            catch (error) {
                return {
                    message: `Error getTranscription: ${error}`,
                };
            }
        });
    }
    pollTranscriptionState(id, interval = 2000, timeout = TIMEOUT) {
        return __awaiter(this, void 0, void 0, function* () {
            let response = yield this.getTranscriptionState(id);
            if (response.statusCode)
                return response;
            let loopResponse = response;
            const startTime = Date.now();
            while (loopResponse.status === TranscriptionStatus.queued ||
                loopResponse.status === TranscriptionStatus.processing) {
                if (Date.now() - startTime > timeout) {
                    throw new Error("Timeout");
                }
                yield new Promise((resolve) => setTimeout(resolve, interval));
                response = yield this.getTranscriptionState(id);
                loopResponse = response;
                console.log(`gladia polling sleep ... ${loopResponse.status}`);
            }
            return response;
        });
    }
    getTranscription(resultUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.client.get(resultUrl);
                return response.data;
            }
            catch (error) {
                return {
                    message: `Error getTranscription: ${error}`,
                };
            }
        });
    }
    deleteTranscription(id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.client.delete(`/transcription/${id}`);
                return response.data;
            }
            catch (error) {
                throw new Error(`Error deleting transcription: ${error}`);
            }
        });
    }
}
export default GladiaClient;
// export async function gladiaTranscribeLocalAudioFn(
//   input: Transient,
//   output: Transient,
//   args?: IArgs
// ): Promise<Transient> {
//   if (typeof input !== "string") throw new Error("Input must be a string");
//   const localCopyPath = input as string;
//   if (!fs.existsSync(localCopyPath))
//     throw new Error("Input file does not exist");
//   const {
//     apiKey,
//     languageCode,
//     prompt,
//     translateToLanguages,
//     customVocabulary,
//   } = args as {
//     apiKey: string;
//     languageCode?: string;
//     prompt?: string;
//     translateToLanguages?: string[];
//     customVocabulary?: string[];
//   };
//   const gladia = new GladiaClient(apiKey);
//   const uploadResult = await gladia.uploadLocalMedia(input);
//   const url = uploadResult.audio_url;
//   console.log("gladia url", url);
//   const transcribeConfig: Partial<IAudioTranscriptionConfig> = {
//     language: languageCode,
//     context_prompt: prompt,
//     custom_vocabulary: customVocabulary,
//     sentences: true,
//     translation: !!translateToLanguages && translateToLanguages.length > 0,
//     translation_config: {
//       target_languages: translateToLanguages ?? [],
//       model: "base",
//     },
//   };
//   const response = await gladia.transcribeAudioFromUrl(url, transcribeConfig);
//   const id = (response as IAudioTranscriptionResponse).id;
//   let transcriptionResult: ITranscriptionResult | undefined = undefined;
//   if (id) {
//     let transcript = await gladia.pollTranscriptionState(id);
//     const res = await gladia.deleteTranscription(id);
//     if (!isIErrorResponse(transcript)) {
//       transcript = transcript as IGetTranscriptionResponse;
//       if (transcript.result?.transcription) {
//         transcriptionResult = IGladiaTranscriptionToITranscriptionResult(
//           transcript.result?.transcription
//         );
//       }
//     } else {
//       return `ERROR:  gladiaTranscribeLocalAudioFn failed ${
//         (transcript as IGladiaErrorResponse).statusCode
//       }  ${(transcript as IGladiaErrorResponse).message}`;
//     }
//     console.log("Gladia Transcript RESULT", transcript);
//   }
//   if (transcriptionResult) return JSON.stringify(transcriptionResult);
//   return "ERROR: gladiaTranscribeLocalAudioFn failed";
// }
const getMediaType = (filePath) => {
    var _a;
    const extension = (_a = filePath.split(".").pop()) === null || _a === void 0 ? void 0 : _a.toLowerCase();
    const mediaTypes = {
        mp3: "audio/mpeg",
        wav: "audio/wav",
        flac: "audio/flac",
        m4a: "audio/mp4",
        aac: "audio/aac",
        mp4: "video/mp4",
        avi: "video/x-msvideo",
        mkv: "video/x-matroska",
        mov: "video/quicktime",
        wmv: "video/x-ms-wmv",
        // Add more file extensions and media types as needed
    };
    return mediaTypes[extension || ""] || "application/octet-stream";
};
//# sourceMappingURL=gladia-service.js.map