export function connectSiblings(itemA, itemB) {
    if (!itemA.documentReference || !itemB.documentReference)
        throw new Error("Document reference is required");
    const a = addSibling(itemA, itemB.documentReference);
    const b = addSibling(itemB, itemA.documentReference);
    return { a, b };
}
export function disconnectSiblings(itemA, itemB) {
    if (!itemA.documentReference || !itemB.documentReference)
        throw new Error("Document reference is required");
    const a = removeSibling(itemA, itemB.documentReference);
    const b = removeSibling(itemB, itemA.documentReference);
    return { a, b };
}
export function addSibling(item, siblingRef) {
    if (!item.linkedSiblings)
        item.linkedSiblings = [];
    if (item.linkedSiblings.find((x) => x === siblingRef.path))
        return item;
    item.linkedSiblings.push(siblingRef.path);
    return item;
}
export function removeSibling(item, siblingRef) {
    if (!item.linkedSiblings)
        return item;
    const index = item.linkedSiblings.findIndex((x) => x === siblingRef.path);
    if (index < 0)
        return item;
    item.linkedSiblings.splice(index, 1);
    return item;
}
export function iHasSiblingsRelationshipToAlgolia(item) {
    var _a;
    return {
        linkedSiblings: (_a = item.linkedSiblings) !== null && _a !== void 0 ? _a : [],
    };
}
export function sanitizeIHasSiblingsRelationship(item) {
    let changed = false;
    if (!item.linkedSiblings) {
        item.linkedSiblings = null;
        changed = true;
    }
    if (!item.linkedSiblings) {
        item.linkedSiblings = [];
        changed = true;
    }
    return changed;
}
//# sourceMappingURL=IHasSiblingsRelationship.js.map