import type { LayerType } from "../enums/index.js";
import type { LottieData } from "../types/index.js";
import type { IPosition } from "./IPosition.js";
import type { ISize } from "./ISize.js";
import type { IBackground } from "./IBackground.js";
import type { IProperty } from "./IProperty.js";
import type { IFrameRange } from "./IFrameRange.js";
import type { IMediaAsset } from "./IMediaAsset.js";
import type { ITransition } from "./ITransition.js";
import type { IWebVTT } from "./IWebVTT.js";
import type { IVariant, LanguageKey } from "./ILocalCloudFile.js";
import { BoxData } from "../models/index.js";
export interface ILayer {
    id: string | null;
    libId: string | null;
    assetId: string | null;
    name: string | null;
    linkedTo: string | null;
    lockedAtParent?: boolean;
    hiddenInTimeline?: boolean;
    language: string | null;
    renderDisabled: boolean | null;
    renderDisabledLocal: boolean | null;
    alphaChannel?: boolean | null;
    background: IBackground | null;
    backgroundColor: string | null;
    charSpacing: number | null;
    color: string | null;
    colors: string[] | null;
    cutFrom: number | null;
    cutTo: number | null;
    start: number | null;
    stop: number | null;
    duration: number | null;
    innerBoundaryRange: IFrameRange | null;
    outerBoundaryRange: IFrameRange | null;
    absoluteRange: IFrameRange | null;
    relativeRange: IFrameRange | null;
    cutRange: IFrameRange | null;
    transitions: ITransition[] | null;
    fontPath: string | null;
    fontSize: number | null;
    fontWeight: string | null;
    fragmentPath: string;
    mixVolume: number | null;
    speed: number | null;
    type: LayerType;
    text: string | null;
    text2: string | null;
    textColor: string | null;
    path: string | null;
    proxyPath: string | null;
    spacialBox: BoxData | null;
    zindex: number | null;
    position: IPosition | null;
    size: ISize | null;
    resizeMode: string | null;
    sourceSize: ISize | null;
    sourceRatioFraction: [number, number] | null;
    zoomAmount: number | null;
    zoomDirection: string | null;
    lottieData?: LottieData | string | null;
    properties: IProperty[] | null;
    propertiesLocale?: IVariant<LanguageKey, IProperty[]>[] | null;
    subtitlesLocale?: IVariant<LanguageKey, IWebVTT>[] | null;
    styles?: {
        [key: string]: string;
    } | null;
}
export interface IAudioClip extends ILayer {
    asset: IMediaAsset;
}
export interface ITitle extends ILayer {
    active: boolean;
}
export interface IVideo extends ILayer {
}
export interface ISlideInText extends ILayer {
}
export declare function compareableLayer(layer: ILayer): ILayer;
//# sourceMappingURL=ILayer.d.ts.map