import type { IHasUsers } from "./IHasUsers.js";
import { type ILocalCloudFile, type ILocalCloudFileAlgolia, type IVariant, LanguageVersionKey, TypeParamIndexKey } from "./ILocalCloudFile.js";
import type { ISelectors } from "./ISelectors.js";
import type { ITimerangeInformation } from "./ITimerangeInformation.js";
import type { IVideoMetadata } from "./IVideoMetadata.js";
import type { IVideoSettings } from "./IVideoSettings.js";
export interface ITranscription {
    transcript?: string;
    srtFile?: ILocalCloudFile;
    jsonFile?: ILocalCloudFile;
    variants?: IVariant<LanguageVersionKey, ITranscription>[];
}
export interface ITranscriptionAlgolia {
    transcript?: string;
    srtFile?: ILocalCloudFileAlgolia;
    jsonFile?: ILocalCloudFileAlgolia;
    variants?: ITranscriptionAlgolia[];
}
export declare function iTranscriptionToAlgolia(item: ITranscription): ITranscriptionAlgolia;
export declare enum DerivedFileTypes {
    proxy = "proxy",
    extractedAudio = "extractedAudio",
    normalizedAudio = "normalizedAudio",
    thumbnail = "thumbnail",
    previewStripeImage = "previewStripeImage"
}
export declare enum MediaType {
    null = -1,
    Text = 0,
    Picture = 1,
    Audio = 2,
    Video = 3,
    Pdf = 4,
    PictureOverlay = 5,
    Lottie = 6,
    Subtitle = 7,
    Font = 8
}
export declare function mediaTypeToString(type: MediaType): string;
export interface IMediaAsset extends IHasUsers {
    sourceFile: ILocalCloudFile;
    derivedFiles?: IVariant<TypeParamIndexKey, ILocalCloudFile>[];
    proxyFile?: ILocalCloudFile;
    proxyFileLow?: ILocalCloudFile;
    audioFile?: ILocalCloudFile;
    normalizedAudioFile?: ILocalCloudFile;
    thumbnailFile?: ILocalCloudFile;
    previewStripeImageFile?: ILocalCloudFile;
    transcription?: ITranscription;
    isUploaded: boolean;
    version?: number | null;
    mediaType: MediaType;
    value: string;
    newRecord?: boolean | null;
    size: number;
    length: number;
    durationInSeconds: number;
    durationTicks?: number;
    durationInFrames?: number;
    inPoint?: number;
    outPoint?: number;
    videoSettings?: IVideoSettings | null;
    videoMetadata?: IVideoMetadata | null;
    analysis?: ITimerangeInformation<any>[] | null;
    languageCode?: string | null;
    selectors?: ISelectors;
}
export interface IMediaAssetAlgolia {
    sourceFile?: ILocalCloudFileAlgolia | null;
    derivedFiles?: (ILocalCloudFileAlgolia | null)[];
    transcription?: ITranscriptionAlgolia;
    analysis?: string[] | null;
    value?: string;
    videoSettings?: IVideoSettings | null;
    languageCode?: string | null;
}
export declare function iMediaAssetToAlgolia(item: IMediaAsset): IMediaAssetAlgolia;
//# sourceMappingURL=IMediaAsset.d.ts.map