/* eslint-disable capitalized-comments */
import { isSmaller } from "../extensions/index.js";
import { iLocalCloudFileToAlgolia, } from "./ILocalCloudFile.js";
export function iTranscriptionToAlgolia(item) {
    const variants = [];
    if (item.variants && Array.isArray(item.variants)) {
        item.variants.forEach((variant) => {
            const x = iTranscriptionToAlgolia(variant.value);
            if (isSmaller(variants, 0.7)) {
                variants.push(x);
            }
        });
    }
    return {
        transcript: item.transcript,
        srtFile: item.srtFile ? item.srtFile : undefined,
        jsonFile: item.jsonFile ? item.jsonFile : undefined,
        variants,
    };
}
export var DerivedFileTypes;
(function (DerivedFileTypes) {
    DerivedFileTypes["proxy"] = "proxy";
    DerivedFileTypes["extractedAudio"] = "extractedAudio";
    DerivedFileTypes["normalizedAudio"] = "normalizedAudio";
    DerivedFileTypes["thumbnail"] = "thumbnail";
    DerivedFileTypes["previewStripeImage"] = "previewStripeImage";
})(DerivedFileTypes || (DerivedFileTypes = {}));
export var MediaType;
(function (MediaType) {
    MediaType[MediaType["null"] = -1] = "null";
    MediaType[MediaType["Text"] = 0] = "Text";
    MediaType[MediaType["Picture"] = 1] = "Picture";
    MediaType[MediaType["Audio"] = 2] = "Audio";
    MediaType[MediaType["Video"] = 3] = "Video";
    MediaType[MediaType["Pdf"] = 4] = "Pdf";
    MediaType[MediaType["PictureOverlay"] = 5] = "PictureOverlay";
    MediaType[MediaType["Lottie"] = 6] = "Lottie";
    MediaType[MediaType["Subtitle"] = 7] = "Subtitle";
    MediaType[MediaType["Font"] = 8] = "Font";
})(MediaType || (MediaType = {}));
export function mediaTypeToString(type) {
    switch (type) {
        case MediaType.Video:
            return "Video";
        case MediaType.Picture:
            return "Picture";
        case MediaType.Audio:
            return "Audio";
        case MediaType.Text:
            return "Text";
        case MediaType.Lottie:
            return "Lottie";
        case MediaType.Subtitle:
            return "Subtitle";
        case MediaType.Font:
            return "Font";
        default:
            return "Unknown";
    }
}
export function iMediaAssetToAlgolia(item) {
    const derivedFiles = [];
    if (item.derivedFiles && Array.isArray(item.derivedFiles)) {
        item.derivedFiles.forEach((variant) => {
            const x = iLocalCloudFileToAlgolia(variant.value, item.mediaType);
            if (x)
                derivedFiles.push(x);
        });
    }
    const transcription = item.transcription
        ? iTranscriptionToAlgolia(item.transcription)
        : undefined;
    const analysis = [];
    if (item.analysis && Array.isArray(item.analysis)) {
        item.analysis.forEach((item) => {
            if (isSmaller(analysis, 0.7))
                analysis.push(`${item.information}`);
        });
    }
    return {
        sourceFile: iLocalCloudFileToAlgolia(item.sourceFile, item.mediaType),
        derivedFiles,
        // transcription: ifSmaller(transcription, 0.7),
        // value: item.value,
        // videoSettings: item.videoSettings,
        // languageCode: item.languageCode,
        analysis,
    };
}
//# sourceMappingURL=IMediaAsset.js.map