export interface IRangeDto<T> {
    start: T;
    end: T;
}
export interface IRange<T> extends IRangeDto<T> {
    compare(a: T, b: T): number;
    add?(a: T, b: T): T;
    subtract?(a: T, b: T): T;
    multiply?(a: T, factor: number): T;
    divide?(a: T, divisor: number): T;
    isValid(): boolean;
    isEmpty(): boolean;
    isEqual(other: IRange<T>): boolean;
    clone(): IRange<T>;
    isBefore(other: IRange<T>): boolean;
    isBehind(other: IRange<T>): boolean;
    isIntersecting(other: IRange<T>): boolean;
    isInside(other: IRange<T>): boolean;
    startIsEqual(other: IRange<T>): boolean;
    endIsEqual(other: IRange<T>): boolean;
    containsPoint(point: T): boolean;
    duration?(): T;
    deltaStart?(other: IRange<T>): T;
    deltaEnd?(other: IRange<T>): T;
    getIntersection(other: IRange<T>): IRange<T> | null;
    union(other: IRange<T> | IRange<T>[]): IRange<T>[];
    difference(other: IRange<T>): IRange<T>[];
    shift(amount: T): IRange<T>;
    scale(factor: number): IRange<T>;
}
//# sourceMappingURL=IRange.d.ts.map