import type { IDocumentReference } from "./IDocumentReference.js";
export type Transient = string | number | boolean | object | Error | null;
export interface ITransient {
    value?: Transient;
    args: IArgs;
    outputBacklog: IOutput[];
    argsBacklog: IArgs;
    state: "pending" | "success" | "cancelled" | "failed";
    message?: string;
    trace?: ITrace;
    traceReference?: string;
    depth: number;
}
export declare enum PipeNames {
    audioMetadataUpdatedPipe = "audioMetadataUpdatedPipe",
    extractAudioPipe = "extractAudioPipe",
    translateProjectFactory = "translateProjectFactory",
    translateSubtitleLibraryItemFactory = "translateSubtitleLibraryItemFactory",
    previewPipe = "previewPipe",
    translateProjectPipe = "translateProjectPipe",
    translateSubtitleLibraryItemPipe = "translateSubtitleLibraryItemPipe",
    videoUploadedPipe = "videoUploadedPipe",
    videoMetadataUpdatedPipe = "videoMetadataUpdatedPipe"
}
export interface ITaskInfo {
    docRef?: IDocumentReference;
    filePath?: string;
    userId?: string;
    workGroupId?: string;
    info?: any;
}
export interface ITrace extends ITaskInfo {
    start: Date | null | undefined;
    end: Date | null | undefined;
    state: "pending" | "success" | "cancelled" | "failed";
    name: string;
    taskFunction: TaskFunction;
    invoker?: string;
    input: Transient;
    output: Transient;
    args: IArgs;
    depth: number;
    error?: string;
    subsequent: ITrace[];
}
export interface ITask {
    name: string;
    taskFunction: TaskFunction;
    active?: boolean;
    input?: Transient;
    output?: Transient;
    args?: IArgs;
    subsequent?: ITask[];
}
export interface IArgs {
    [key: string]: boolean | number | string | IArgs | object | number[] | string[] | IArgs[] | object[] | null;
}
export declare function argStrings(args: IArgs | undefined, key: string, defaultValue?: string[]): string[];
export declare function argObjects(args: IArgs | undefined, key: string, defaultValue?: object[]): object[];
export declare function argObject(args: IArgs | undefined, key: string, defaultValue: object): object;
export declare function argNumber(args: IArgs | undefined, key: string, defaultValue: number): number;
export declare function argString(args: IArgs | undefined, key: string, defaultValue: string): string;
export interface ITaskFunction {
    (input: Transient, output: Transient, args?: IArgs): Promise<Transient | undefined>;
}
export declare enum TaskFunction {
    map = "map",
    forEach = "forEach",
    none = "none",
    updateDocument = "updateDocument",
    fetchText = "fetchText",
    downloadFile = "downloadFile",
    downloadFromUrl = "downloadFromUrl",
    uploadFile = "uploadFile",
    createTextFile = "createTextFile",
    setMetadata = "setMetadata",
    getMetadata = "getMetadata",
    renderThumbnail = "renderThumbnail",
    renderPreview = "renderPreview",
    extractAudio = "extractAudio",
    extractVideoMetadata = "extractVideoMetadata",
    doublePassNormalization = "doublePassNormalization",
    convertVideoToH264 = "convertVideoToH264",
    startTranscoderApiJob = "startTranscoderApiJob",
    transcribeAudioToBucket = "transcribeAudioToBucket",
    ITranscriptionResultToSrt = "ITranscriptionResultToSrt",
    transcribeAudio = "transcribeAudio",
    gladiaTranscribeUrl = "gladiaTranscribeUrl",// IArgs with { languageCode, prompt, translateToLanguages, customVocabulary } -> IGladiaResult
    gladiaTranscribeLocalAudio = "gladiaTranscribeLocalAudio",// IArgs with { languageCode, prompt, translateToLanguages, customVocabulary }
    IGladiaResultToITranscriptionResult = "IGladiaResultToITranscriptionResult",
    ITranscriptionResultContainerToSubtitleAsset = "IGladiaTranscriptionResultToSubtitleAsset",
    ITranscriptionResultContainerUpdateSubtitleAsset = "ITranscriptionResultContainerUpdateSubtitleAsset",
    srtToSubtitleAsset = "srtToSubtitleAsset",
    jsonToVTT = "jsonToVTT",
    jsonToSRT = "jsonToSRT",
    labelVideo = "labelVideo",
    translateText = "translateText",
    translateIEditSpecification = "translateIEditSpecification",// IArgs with [sourceLanguageCode, prompt?]
    insertTranslationStoreIntoIEditSpecification = "insertTranslationStoreIntoIEditSpecification",// IArgs with [sourceLanguageCode, prompt?]
    translateVTT = "translateVTT",// IArgs with [sourceLanguageCode, targetLanguageCode, prompt?]
    translateJson = "translateJson",// IArgs with [sourceLanguageCode, targetLanguageCode, fields: string[], prompt?: string[]]
    renderVideo = "renderVideo",
    setFFmpegMetadata = "setFFmpegMetadata"
}
export interface IOutput {
    name: string;
    function: string;
    depth: number;
    value: Transient;
}
//# sourceMappingURL=ITask.d.ts.map