import type { GroupType } from "../enums/index.js";
import type { MediaType } from "./IMediaAsset.js";
import type { ICollectionDocument } from "./ICollectionDocument.js";
export interface IUserSettings extends ICollectionDocument {
    selectedWorkgroup: string | null;
    selectedProject?: string | null;
    uiSettings?: IUISettings;
    generalSettings?: IGeneralSettings;
    languageSettings?: ILanguageSettings;
    playerSettings?: IPlayerSettings;
    timelineSettings?: ITimelineSettings;
    librarySettings?: ILibrarySettings[];
    workerSettings?: IWorkerSettings | null;
}
export interface IUISettings {
    leftExpanded: boolean;
    leftSize: number;
    rightExpanded: boolean;
    rightSize: number;
    bottomExpanded: boolean;
    bottomSize: number;
}
export declare const DEFAULT_UI_SETTINGS: IUISettings;
export interface IGeneralSettings {
    loadProjectOnStartup: boolean;
}
export declare const DEFAULT_GENERAL_SETTINGS: IGeneralSettings;
export interface ILibrarySettings {
    name: string;
    type: MediaType;
    layout: "masonry" | "list";
    orderOfGroups?: GroupType[];
    sortBy: "name" | "createdAt";
    sortOrder: "asc" | "desc";
    filterAspect: "16:9" | "1:1" | "9:16" | undefined;
}
export declare const DEFAULT_LIBRARY_SETTINGS: ILibrarySettings[];
export interface IPlayerSettings {
    playOnClick: boolean;
    autoplay: boolean;
    loop: boolean;
    muted: boolean;
    volume: number;
    playbackRate: number;
    showControls?: boolean;
}
export declare const DEFAULT_PLAYER_SETTINGS: Readonly<IPlayerSettings>;
export interface ITimelineSettings {
    skimming: boolean;
    snapping: boolean;
}
export declare const DEFAULT_TIMELINE_SETTINGS: ITimelineSettings;
export interface ILanguageSettings {
    defaultMainLanguage: string;
    defaultTranslationLanguages: string[];
    dictionary: string[];
}
export declare const DEFAULT_LANGUAGE_SETTINGS: ILanguageSettings;
export interface IWorkerSettings {
    onAssetUploaded: {
        video: TaskActivation;
        image: TaskActivation;
        audio: TaskActivation;
        subtitle: TaskActivation;
    };
}
export declare const DEFAULT_WORKER_SETTINGS: IWorkerSettings;
export interface TaskActivation {
    name: string;
    active: boolean;
    icon: string;
    subtasks?: TaskActivation[];
}
export declare const DEFAULT_USER_SETTINGS: IUserSettings;
//# sourceMappingURL=IUserSettings.d.ts.map