var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var _a;
import _ from "lodash";
import "reflect-metadata";
import { immerable } from "immer";
import { Type } from "class-transformer";
import { Aspect, CssUnit } from "../enums/index.js";
import { Position } from "./position.js";
import { CssValue } from "./cssValue.js";
import { NormalizedSize } from "./normalizedSize.js";
import { PercentSize } from "./percentSize.js";
export class BoxModel {
    get position() {
        return this._position;
    }
    set position(value) {
        if (!value)
            throw new Error("BoxModel position cannot be null");
        this._position = value;
        this.setBox();
    }
    get percentPosition() {
        return this.position.toPercent();
    }
    get size() {
        return this._size;
    }
    set size(value) {
        if (!value)
            throw new Error("BoxModel size cannot be null");
        this._size = value;
        this.setBox();
    }
    get percentSize() {
        return NormalizedSizeToPercentSize(this.size);
    }
    get topLeftWidthHeight() {
        const style = {
            position: this.position.positionType,
            top: this.top.toString(),
            left: this.left.toString(),
            width: this.width.toString(),
            height: this.height.toString(),
        };
        return style;
    }
    constructor(position, size, name = "") {
        this.name = name;
        this[_a] = true; // Option 1
        this._position = Position.leftTopZero();
        this._size = NormalizedSize.fill();
        this.aspect = Aspect.None;
        this.top = new CssValue();
        this.bottom = new CssValue();
        this.left = new CssValue();
        this.right = new CssValue();
        this.width = new CssValue();
        this.height = new CssValue();
        this.position = position !== null && position !== void 0 ? position : BoxModel.default().position;
        this.size = size !== null && size !== void 0 ? size : BoxModel.default().size;
    }
    setBox() {
        var _b, _c, _d, _e, _f, _g;
        const isWidth = (_b = this.size) === null || _b === void 0 ? void 0 : _b.width;
        const isHeight = (_c = this.size) === null || _c === void 0 ? void 0 : _c.height;
        const isX = ((_d = this.position) === null || _d === void 0 ? void 0 : _d.x) || ((_e = this.position) === null || _e === void 0 ? void 0 : _e.x) === 0;
        const isY = ((_f = this.position) === null || _f === void 0 ? void 0 : _f.y) || ((_g = this.position) === null || _g === void 0 ? void 0 : _g.y) === 0;
        this.width = CssValue.fromPercent(this.percentSize.width);
        this.height = CssValue.fromPercent(this.percentSize.height);
        const offsetX = this.position.getOffsetX(this.width);
        const offsetY = this.position.getOffsetY(this.height);
        if (isX && this.position.xUnit === CssUnit.percent)
            this.left = CssValue.fromPercent(this.percentPosition.x + offsetX);
        if (isX && this.position.xUnit === CssUnit.pixel)
            this.left = CssValue.fromPixel(this.position.x + offsetX);
        if (isY && this.position.xUnit === CssUnit.percent)
            this.top = CssValue.fromPercent(this.percentPosition.y + offsetY);
        if (isY && this.position.xUnit === CssUnit.pixel)
            this.top = CssValue.fromPixel(this.position.y + offsetY);
        if (isX && isWidth)
            this.right = CssValue.fromPercent(100 - (this.left.value + this.percentSize.width));
        if (isY && isHeight)
            this.bottom = CssValue.fromPercent(100 - (this.top.value + this.percentSize.height));
    }
    getBoxInPercent() {
        const box = {
            left: this.left,
            top: this.top,
            right: this.right,
            bottom: this.bottom,
        };
        box.top.value =
            this.percentPosition.y + this.position.getOffsetY(this.height);
        box.bottom.value = 100 - (box.top.value + this.percentSize.height);
        box.left.value =
            this.percentPosition.x + this.position.getOffsetX(this.width);
        box.right.value = 100 - (box.left.value + this.percentSize.width);
        return box;
    }
    withPosition(position) {
        const clone = _.cloneDeep(this);
        clone.position = position;
        return clone;
    }
    withSize(size) {
        const clone = _.cloneDeep(this);
        clone.size = size;
        return clone;
    }
    clone() {
        const clone = _.cloneDeep(this);
        return clone;
    }
    update(fn) {
        fn(this);
        this.setBox();
    }
    static fromTopLeftWidthHeight(leftPercent, topPercent, width, height) {
        const box = new BoxModel(Position.leftTopZero(), NormalizedSize.fill());
        box.top = CssValue.fromPercent(leftPercent !== null && leftPercent !== void 0 ? leftPercent : 0);
        box.left = CssValue.fromPercent(topPercent !== null && topPercent !== void 0 ? topPercent : 0);
        box.width = CssValue.fromPixel(width !== null && width !== void 0 ? width : 400);
        box.height = CssValue.fromPixel(height !== null && height !== void 0 ? height : 400);
        return box;
    }
    static default(name = "leftTopZero") {
        return new BoxModel(Position.leftTopZero(), NormalizedSize.fill(), name);
    }
    static defaultCenter(name = "centerCenter") {
        return new BoxModel(Position.centerCenter(), NormalizedSize.fill(), name);
    }
}
_a = immerable;
__decorate([
    Type(() => Position),
    __metadata("design:type", Object)
], BoxModel.prototype, "_position", void 0);
__decorate([
    Type(() => NormalizedSize),
    __metadata("design:type", Object)
], BoxModel.prototype, "_size", void 0);
__decorate([
    Type(() => CssValue),
    __metadata("design:type", Object)
], BoxModel.prototype, "top", void 0);
__decorate([
    Type(() => CssValue),
    __metadata("design:type", Object)
], BoxModel.prototype, "bottom", void 0);
__decorate([
    Type(() => CssValue),
    __metadata("design:type", Object)
], BoxModel.prototype, "left", void 0);
__decorate([
    Type(() => CssValue),
    __metadata("design:type", Object)
], BoxModel.prototype, "right", void 0);
__decorate([
    Type(() => CssValue),
    __metadata("design:type", Object)
], BoxModel.prototype, "width", void 0);
__decorate([
    Type(() => CssValue),
    __metadata("design:type", Object)
], BoxModel.prototype, "height", void 0);
function NormalizedSizeToPercentSize(normalized) {
    return new PercentSize(normalized.width * 100, normalized.height * 100);
}
function PercentSizeToNormalizedSize(percent) {
    return new NormalizedSize(percent.width * 0.01, percent.height * 0.01);
}
//# sourceMappingURL=boxModel.js.map