import { describe, it, expect, beforeEach } from "vitest";
import { instanceToPlain } from "class-transformer";
import { spec, specWithSub } from "../data";
import { EditSpecification } from "./editSpecification.js";
import { Clip, editSpecificationToClips, clipsFromEditSpecification, setClipTiming, } from "./clip.js";
import { SubtitleLayer } from "./layer/subtitle.layer.js";
import { VideoLayer } from "./layer/video.layer.js";
import { Transition } from "./transition.js";
describe("Clip", () => {
    let jsonString = "";
    let editSpecification;
    let videoSettings;
    beforeEach(() => {
        const transition = Transition.default();
        jsonString = JSON.stringify(spec);
        editSpecification = EditSpecification.fromJSON(jsonString);
        videoSettings = editSpecification.videoSettings;
    });
    describe("jsonToClips", () => {
        it("should return clips from editSpecification", () => {
            expect(1).toBe(1);
        });
    });
    describe("fromEditSpecification", () => {
        it("should return clips from editSpecification", () => {
            const clips = clipsFromEditSpecification(editSpecification, editSpecification.videoSettings);
            expect(clips.length).toBe(1);
        });
    });
    describe("calculateClips", () => {
        it("should return 1 clip from editSpecification instance", () => {
            const clips = editSpecificationToClips(editSpecification, editSpecification.videoSettings);
            expect(clips.length).toBe(1);
            expect(clips[0].getRange).toBeDefined();
            expect(clips[0]).toBeInstanceOf(Clip);
            expect(clips[0].layers.length).toBe(1);
            expect(clips[0].layers[0]).toBeInstanceOf(VideoLayer);
        });
        it("should return 1 clip from editSpecification option", () => {
            const data = instanceToPlain(editSpecification);
            const clips = editSpecificationToClips(data, editSpecification.videoSettings);
            expect(clips.length).toBe(1);
            expect(clips[0].getRange).toBeDefined();
            expect(clips[0]).toBeInstanceOf(Clip);
            expect(clips[0].layers.length).toBe(1);
            expect(clips[0].layers[0]).toBeInstanceOf(VideoLayer);
        });
        it("should return 1 clip from editSpecification string", () => {
            const data = jsonString;
            const clips = editSpecificationToClips(data, editSpecification.videoSettings);
            expect(clips.length).toBe(1);
            expect(clips[0].getRange).toBeDefined();
            expect(clips[0]).toBeInstanceOf(Clip);
            expect(clips[0].layers.length).toBe(1);
            expect(clips[0].layers[0]).toBeInstanceOf(VideoLayer);
        });
        it("WITH SUB should return 1 clip from editSpecification string", () => {
            const data = JSON.stringify(specWithSub);
            const clips = editSpecificationToClips(data, editSpecification.videoSettings);
            expect(clips.length).toBe(1);
            expect(clips[0].getRange).toBeDefined();
            expect(clips[0]).toBeInstanceOf(Clip);
            expect(clips[0].layers.length).toBe(2);
            expect(clips[0].layers[0]).toBeInstanceOf(VideoLayer);
            expect(clips[0].layers[1]).toBeInstanceOf(SubtitleLayer);
        });
    });
    describe("setClipTiming", () => {
        it("should modify timing", () => {
            const clips = editSpecificationToClips(editSpecification, editSpecification.videoSettings);
            setClipTiming(clips);
            expect(clips.length).toBe(1);
        });
    });
});
//# sourceMappingURL=clip.spec.js.map