var _a;
import { immerable } from "immer";
import { Timestamp } from "@firebase/firestore";
import { workerStateInProgress, } from "../interfaces/index.js";
import { makeId } from "../extensions/index.js";
import { HasUsers } from "./hasUsers.js";
export class CollectionDocument extends HasUsers {
    constructor(data) {
        const mutable = Object.assign({}, data);
        super(mutable);
        this[_a] = true; // Option 1
        this.id = "";
        if (data) {
            Object.assign(this, mutable);
        }
        else {
            this.id = makeId(20);
        }
    }
    get createdDate() {
        if (!this.createdAt)
            return null;
        if (this.createdAt instanceof Date)
            return this.createdAt;
        if (this.createdAt instanceof Timestamp)
            return this.createdAt.toDate();
        if (Boolean(this.createdAt) &&
            typeof this.createdAt === "object" &&
            this.createdAt.seconds &&
            this.createdAt.nanoseconds)
            return new Timestamp(this.createdAt.seconds, this.createdAt.nanoseconds).toDate();
        return this.createdAt;
    }
    get updatedDate() {
        if (!this.updatedAt)
            return null;
        if (this.updatedAt instanceof Date)
            return this.updatedAt;
        if (this.updatedAt instanceof Timestamp)
            return this.updatedAt.toDate();
        if (Boolean(this.updatedAt) &&
            typeof this.updatedAt === "object" &&
            this.updatedAt.seconds &&
            this.updatedAt.nanoseconds)
            return new Timestamp(this.updatedAt.seconds, this.updatedAt.nanoseconds).toDate();
        return this.updatedAt;
    }
    get tasksInProgress() {
        if (!this.backgroundTasks)
            return [];
        return this.backgroundTasks.filter((task) => workerStateInProgress(task.state));
    }
    get hasTaskInProgress() {
        if (!this.backgroundTasks)
            return false;
        return this.backgroundTasks.some((task) => workerStateInProgress(task.state));
    }
    toICollectionDocument() {
        return Object.assign(Object.assign({}, super.toIHasUsers()), { id: this.id, createdAt: this.createdAt, createdBy: this.createdBy, updatedAt: this.updatedAt, updatedBy: this.updatedBy, linkedParent: this.linkedParent, linkedChildren: this.linkedChildren, workerLock: this.workerLock, notificationToken: this.notificationToken, collectionPath: this.collectionPath, documentReference: this.documentReference, tags: this.tags, metadata: this.metadata, backgroundTasks: this.backgroundTasks });
    }
    toDTO() {
        return this.toICollectionDocument();
    }
    static fromDTO(data) {
        return new CollectionDocument(data);
    }
    static sortByCreatedAt(a, b) {
        const timestampA = a === null || a === void 0 ? void 0 : a.createdAt;
        const timestampB = b === null || b === void 0 ? void 0 : b.createdAt;
        if (!timestampA && !timestampB)
            return 0;
        if (!timestampA && timestampB)
            return 1;
        if (timestampA && !timestampB)
            return -1;
        const dateA = a.createdDate;
        const dateB = b.createdDate;
        if (dateA < dateB) {
            return 1;
        }
        if (dateA > dateB) {
            return -1;
        }
        return 0;
    }
    static sortByUpdatedAt(a, b) {
        const timestampA = a === null || a === void 0 ? void 0 : a.updatedAt;
        const timestampB = b === null || b === void 0 ? void 0 : b.updatedAt;
        if (!timestampA && !timestampB)
            return 0;
        if (!timestampA && timestampB)
            return 1;
        if (timestampA && !timestampB)
            return -1;
        const dateA = a.updatedDate;
        const dateB = b.updatedDate;
        if (dateA < dateB) {
            return 1;
        }
        if (dateA > dateB) {
            return -1;
        }
        return 0;
    }
    static sortByProperty(property, a, b) {
        const aPropertyExist = Object.prototype.hasOwnProperty.call(a, property);
        const bPropertyExist = Object.prototype.hasOwnProperty.call(b, property);
        if (!aPropertyExist && !bPropertyExist)
            return 0;
        if (!aPropertyExist && bPropertyExist)
            return 1;
        if (aPropertyExist && !bPropertyExist)
            return -1;
        const valueA = a[property];
        const valueB = b[property];
        if (valueA < valueB) {
            return 1;
        }
        if (valueA > valueB) {
            return -1;
        }
        return 0;
    }
    static trackingFn(index, item) {
        return item.id;
    }
}
_a = immerable;
//# sourceMappingURL=collectionDocument.js.map