import { jsx as _jsx } from "react/jsx-runtime";
import "reflect-metadata";
import LayerSequence from "./layer-sequence.js";
import { disableContext } from "../layer/devModeContext.js";
import { Clip } from "../clip.js";
import { verbose } from "../../constants.js";
const ClipSequence = ({ clip, clipIndex, previousClip, selectors, }) => {
    let _clip = clip;
    if (verbose)
        console.log("ClipSequence: clip", clipIndex, _clip, selectors);
    if (!_clip)
        throw new Error(`ClipSequence: clip is undefined  ${selectors}`);
    if (!(_clip instanceof Clip)) {
        _clip = Clip.fromClip(clip);
    }
    if (!(_clip instanceof Clip))
        throw new Error(`ClipSequence: clip is not a Clip: ${typeof _clip}, ${JSON.stringify(_clip, null, 2)}`);
    if (!_clip.layers)
        throw new Error(`ClipSequence: clip.layers is undefined ${selectors}`);
    if (!Array.isArray(_clip.layers))
        throw new Error(`ClipSequence: clip.layers is not an array ${selectors}`);
    const sequenceRange = _clip.getRange(previousClip);
    _clip.absoluteRange = sequenceRange.clone();
    const layerSequences = _clip.layers.map((layer, index) => {
        const props = {
            sequenceRange,
            index: clipIndex * 100 + index,
            layer,
            selectors,
        };
        return _jsx(LayerSequence, Object.assign({}, props), `Layer-${index}`);
    });
    return (_jsx("div", { style: { backgroundColor: undefined }, children: _jsx(disableContext.Consumer, { children: (disable) => {
                if (disable.includes("layerSequences"))
                    return null;
                return layerSequences;
            } }) }, "ClipSequence"));
};
export default ClipSequence;
//# sourceMappingURL=clip-sequence.js.map