import { jsx as _jsx } from "react/jsx-runtime";
import { useMemo } from "react";
import { Sequence } from "remotion";
export const DnDLayer = ({ item, config }) => {
    const style = useMemo(() => {
        return Object.assign({ backgroundColor: item.color, position: "absolute" }, spacialStyle(item));
    }, [item]);
    return (_jsx(Sequence, { from: item.from, durationInFrames: item.durationInFrames, layout: "none", children: _jsx("div", { style: style }) }, item.id));
};
/**
 * Generates a spatial style object for a drag-and-drop item.
 *
 * @param item - The drag-and-drop item containing spatial properties.
 * @returns An object containing the CSS properties: left, top, width, height, and zIndex.
 */
export function spacialStyle(item) {
    return {
        left: inUnit(item.left, item),
        top: inUnit(item.top, item),
        width: inUnit(item.width, item),
        height: inUnit(item.height, item),
        zIndex: item.zIndex,
    };
}
function inUnit(value, item) {
    return item.unit === "%" ? `${value}%` : value;
}
//# sourceMappingURL=dnd-layer.js.map