import { jsx as _jsx } from "react/jsx-runtime";
import { useCallback, useMemo } from "react";
import { useCurrentScale, useVideoConfig } from "remotion";
const HANDLE_SIZE = 12;
const SNAP_THRESHOLD = 5;
export const ResizeHandle = ({ type, setItem, item, config }) => {
    var _a, _b;
    const { width, height } = useVideoConfig();
    const scale = useCurrentScale();
    const size = Math.round((_a = config.resizeHandleSize) !== null && _a !== void 0 ? _a : HANDLE_SIZE / scale);
    const borderSize = (_b = config.resizeHandleBorderSize) !== null && _b !== void 0 ? _b : 1 / scale;
    const sizeStyle = useMemo(() => {
        var _a, _b;
        return {
            position: "absolute",
            height: size,
            width: size,
            backgroundColor: (_a = config.resizeHandleBackgroundColor) !== null && _a !== void 0 ? _a : "white",
            border: `${borderSize}px solid ${(_b = config.resizeHandleBorderColor) !== null && _b !== void 0 ? _b : "#0B84F3"}`,
        };
    }, [borderSize, config, size]);
    const margin = -size / 2 - borderSize;
    const style = useMemo(() => {
        if (type === "top-left") {
            return Object.assign(Object.assign({}, sizeStyle), { marginLeft: margin, marginTop: margin, cursor: "nwse-resize" });
        }
        if (type === "top-right") {
            return Object.assign(Object.assign({}, sizeStyle), { marginTop: margin, marginRight: margin, right: 0, cursor: "nesw-resize" });
        }
        if (type === "bottom-left") {
            return Object.assign(Object.assign({}, sizeStyle), { marginBottom: margin, marginLeft: margin, bottom: 0, cursor: "nesw-resize" });
        }
        if (type === "bottom-right") {
            return Object.assign(Object.assign({}, sizeStyle), { marginBottom: margin, marginRight: margin, right: 0, bottom: 0, cursor: "nwse-resize" });
        }
        throw new Error("Unknown type: " + JSON.stringify(type));
    }, [margin, sizeStyle, type]);
    const onPointerDown = useCallback((e) => {
        e.stopPropagation();
        if (e.button !== 0) {
            return;
        }
        const initialX = e.clientX;
        const initialY = e.clientY;
        const onPointerMove = (pointerMoveEvent) => {
            const unitScaleX = item.unit === "%" ? 100 / width : 1;
            const unitScaleY = item.unit === "%" ? 100 / height : 1;
            const offsetX = ((pointerMoveEvent.clientX - initialX) / scale) * unitScaleX;
            const offsetY = ((pointerMoveEvent.clientY - initialY) / scale) * unitScaleY;
            const isLeft = type === "top-left" || type === "bottom-left";
            const isTop = type === "top-left" || type === "top-right";
            setItem(item.id, (i) => {
                let newWidth = item.width + (isLeft ? -offsetX : offsetX);
                let newHeight = item.height + (isTop ? -offsetY : offsetY);
                let newLeft = item.left + (isLeft ? offsetX : 0);
                let newTop = item.top + (isTop ? offsetY : 0);
                // #region Snap to positions
                const snapPositionsX = config.snapPositionsX
                    ? [
                        ...config.snapPositionsX,
                        ...config.snapPositionsX.map((x) => x + item.width),
                        ...config.snapPositionsX.map((x) => x + item.width / 2),
                        ...config.snapPositionsX.map((x) => x - item.width),
                        ...config.snapPositionsX.map((x) => x - item.width / 2),
                    ]
                    : undefined;
                const snapPositionsY = config.snapPositionsY
                    ? [
                        ...config.snapPositionsY,
                        ...config.snapPositionsY.map((y) => y + item.height),
                        ...config.snapPositionsY.map((y) => y + item.height / 2),
                        ...config.snapPositionsY.map((y) => y - item.height),
                        ...config.snapPositionsY.map((y) => y - item.height / 2),
                    ]
                    : undefined;
                // Function to snap to the closest position if within threshold
                const snapToPositions = (value, positions) => {
                    if (!positions) {
                        return value;
                    }
                    let snappedValue = value;
                    positions.forEach((pos) => {
                        var _a;
                        if (Math.abs(value - pos) <=
                            ((_a = config.sanppingThreshold) !== null && _a !== void 0 ? _a : SNAP_THRESHOLD) / scale) {
                            snappedValue = pos;
                        }
                    });
                    return snappedValue;
                };
                // Snap dimensions and positions if close to any value in SNAP_POSITIONS
                newWidth = snapToPositions(newWidth, snapPositionsX);
                newHeight = snapToPositions(newHeight, snapPositionsY);
                newLeft = snapToPositions(newLeft, snapPositionsX);
                newTop = snapToPositions(newTop, snapPositionsY);
                // #endregion
                // #region Enforce aspect ratio if set
                if (item.enforceRatio) {
                    const [ratioWidth, ratioHeight] = item.enforceRatio;
                    const aspectRatio = ratioWidth / ratioHeight;
                    if (newWidth / newHeight > aspectRatio) {
                        newWidth = newHeight * aspectRatio;
                    }
                    else {
                        newHeight = newWidth / aspectRatio;
                    }
                    // Adjust position if resizing from left or top
                    if (isLeft) {
                        newLeft = item.left + (item.width - newWidth);
                    }
                    if (isTop) {
                        newTop = item.top + (item.height - newHeight);
                    }
                }
                // #endregion
                return Object.assign(Object.assign({}, i), { width: Math.max(1, Math.round(newWidth)), height: Math.max(1, Math.round(newHeight)), left: Math.min(item.left + item.width - 1, Math.round(newLeft)), top: Math.min(item.top + item.height - 1, Math.round(newTop)), isDragging: true });
            });
        };
        const onPointerUp = () => {
            setItem(item.id, (i) => {
                return Object.assign(Object.assign({}, i), { isDragging: false });
            });
            window.removeEventListener("pointermove", onPointerMove);
        };
        window.addEventListener("pointermove", onPointerMove, { passive: true });
        window.addEventListener("pointerup", onPointerUp, {
            once: true,
        });
    }, [
        config.sanppingThreshold,
        config.snapPositionsX,
        config.snapPositionsY,
        height,
        item.enforceRatio,
        item.height,
        item.id,
        item.left,
        item.top,
        item.unit,
        item.width,
        scale,
        setItem,
        type,
        width,
    ]);
    return _jsx("div", { style: style, onPointerDown: onPointerDown });
};
//# sourceMappingURL=resize-handle.js.map