import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useCallback, useMemo } from "react";
import { useCurrentScale, useVideoConfig } from "remotion";
import { ResizeHandle } from "./resize-handle.js";
import { spacialStyle } from "./dnd-layer.js";
const SNAP_THRESHOLD = 5;
export const SelectionOutline = ({ item, changeItem, setSelectedItem, selectedItem, isDragging, config, }) => {
    var _a;
    const { width, height } = useVideoConfig();
    const scale = useCurrentScale();
    const scaledBorder = Math.ceil(((_a = config.selectionBorderSize) !== null && _a !== void 0 ? _a : 2) / scale);
    const [hovered, setHovered] = React.useState(false);
    const onMouseEnter = useCallback(() => {
        setHovered(true);
    }, []);
    const onMouseLeave = useCallback(() => {
        setHovered(false);
    }, []);
    const isSelected = item.id === selectedItem;
    const style = useMemo(() => {
        var _a;
        return Object.assign(Object.assign({ position: "absolute" }, spacialStyle(item)), { outline: (hovered && !isDragging) || isSelected
                ? `${scaledBorder}px solid ${(_a = config.selectionBorderColor) !== null && _a !== void 0 ? _a : "#0B84F3"}`
                : undefined, userSelect: "none", touchAction: "none" });
    }, [item, hovered, isDragging, isSelected, scaledBorder, config]);
    const startDragging = useCallback((e) => {
        const initialX = e.clientX;
        const initialY = e.clientY;
        const onPointerMove = (pointerMoveEvent) => {
            const unitScaleX = item.unit === "%" ? 100 / width : 1;
            const unitScaleY = item.unit === "%" ? 100 / height : 1;
            const offsetX = ((pointerMoveEvent.clientX - initialX) / scale) * unitScaleX;
            const offsetY = ((pointerMoveEvent.clientY - initialY) / scale) * unitScaleY;
            let newLeft = Math.round(item.left + offsetX);
            let newTop = Math.round(item.top + offsetY);
            // #region Snap to positions
            const snapPositionsX = config.snapPositionsX
                ? [
                    ...config.snapPositionsX,
                    ...config.snapPositionsX.map((x) => x + item.width),
                    ...config.snapPositionsX.map((x) => x + item.width / 2),
                    ...config.snapPositionsX.map((x) => x - item.width),
                    ...config.snapPositionsX.map((x) => x - item.width / 2),
                ]
                : undefined;
            const snapPositionsY = config.snapPositionsY
                ? [
                    ...config.snapPositionsY,
                    ...config.snapPositionsY.map((y) => y + item.height),
                    ...config.snapPositionsY.map((y) => y + item.height / 2),
                    ...config.snapPositionsY.map((y) => y - item.height),
                    ...config.snapPositionsY.map((y) => y - item.height / 2),
                ]
                : undefined;
            // Function to snap to the closest position if within threshold
            const snapToPositions = (value, positions) => {
                if (!positions) {
                    return value;
                }
                let snappedValue = value;
                positions.forEach((pos) => {
                    var _a;
                    if (Math.abs(value - pos) <=
                        ((_a = config.sanppingThreshold) !== null && _a !== void 0 ? _a : SNAP_THRESHOLD) / scale) {
                        snappedValue = pos;
                    }
                });
                return snappedValue;
            };
            // Snap dimensions and positions if close to any value in SNAP_POSITIONS
            newLeft = snapToPositions(newLeft, snapPositionsX);
            newTop = snapToPositions(newTop, snapPositionsY);
            // #endregion
            changeItem(item.id, (befor) => {
                return Object.assign(Object.assign({}, befor), { left: newLeft, top: newTop, isDragging: true });
            });
        };
        const onPointerUp = () => {
            changeItem(item.id, (befor) => {
                return Object.assign(Object.assign({}, befor), { isDragging: false });
            });
            window.removeEventListener("pointermove", onPointerMove);
        };
        window.addEventListener("pointermove", onPointerMove, { passive: true });
        window.addEventListener("pointerup", onPointerUp, {
            once: true,
        });
    }, [
        item.unit,
        item.left,
        item.top,
        item.id,
        item.width,
        item.height,
        width,
        height,
        scale,
        config.snapPositionsX,
        config.snapPositionsY,
        config.sanppingThreshold,
        changeItem,
    ]);
    const onPointerDown = useCallback((e) => {
        e.stopPropagation();
        if (e.button !== 0) {
            return;
        }
        setSelectedItem(item.id);
        startDragging(e);
    }, [item.id, setSelectedItem, startDragging]);
    return (_jsx("div", { style: style, onPointerDown: onPointerDown, onPointerEnter: onMouseEnter, onPointerLeave: onMouseLeave, children: isSelected ? (_jsxs(_Fragment, { children: [_jsx(ResizeHandle, { item: item, config: config, setItem: changeItem, type: "top-left" }), _jsx(ResizeHandle, { item: item, config: config, setItem: changeItem, type: "top-right" }), _jsx(ResizeHandle, { item: item, config: config, setItem: changeItem, type: "bottom-left" }), _jsx(ResizeHandle, { item: item, config: config, setItem: changeItem, type: "bottom-right" })] })) : null }));
};
//# sourceMappingURL=selection-outline.js.map