/// <reference types="react" />
import { z } from "zod";
export type DnDConfig = {
    readonly selectionBorderColor?: string;
    readonly selectionBorderSize?: number;
    readonly resizeHandleSize?: number;
    readonly resizeHandleBackgroundColor?: string;
    readonly resizeHandleBorderColor?: string;
    readonly resizeHandleBorderSize?: number;
    readonly sanppingThreshold?: number;
    readonly snapPositionsX?: number[];
    readonly snapPositionsY?: number[];
};
export declare const DnDConfigSchema: z.ZodObject<{
    selectionBorderColor: z.ZodOptional<z.ZodString>;
    selectionBorderSize: z.ZodOptional<z.ZodNumber>;
    resizeHandleSize: z.ZodOptional<z.ZodNumber>;
    resizeHandleBorderSize: z.ZodOptional<z.ZodNumber>;
    resizeHandleBorderColor: z.ZodOptional<z.ZodString>;
    resizeHandleBackgroundColor: z.ZodOptional<z.ZodString>;
    sanppingThreshold: z.ZodOptional<z.ZodNumber>;
    snapPositionsX: z.ZodOptional<z.ZodArray<z.ZodNumber, "many">>;
    snapPositionsY: z.ZodOptional<z.ZodArray<z.ZodNumber, "many">>;
}, "strip", z.ZodTypeAny, {
    selectionBorderColor?: string | undefined;
    selectionBorderSize?: number | undefined;
    resizeHandleSize?: number | undefined;
    resizeHandleBorderSize?: number | undefined;
    resizeHandleBorderColor?: string | undefined;
    resizeHandleBackgroundColor?: string | undefined;
    sanppingThreshold?: number | undefined;
    snapPositionsX?: number[] | undefined;
    snapPositionsY?: number[] | undefined;
}, {
    selectionBorderColor?: string | undefined;
    selectionBorderSize?: number | undefined;
    resizeHandleSize?: number | undefined;
    resizeHandleBorderSize?: number | undefined;
    resizeHandleBorderColor?: string | undefined;
    resizeHandleBackgroundColor?: string | undefined;
    sanppingThreshold?: number | undefined;
    snapPositionsX?: number[] | undefined;
    snapPositionsY?: number[] | undefined;
}>;
export type DragAndDropItem = {
    id: number | string;
    from: number;
    durationInFrames: number;
    unit: "px" | "%";
    top: number;
    left: number;
    width: number;
    height: number;
    enforceRatio?: [number, number];
    zIndex?: number;
    color: string;
    isDragging: boolean;
};
export declare const DragAndDropItemSchema: z.ZodObject<{
    id: z.ZodUnion<[z.ZodNumber, z.ZodString]>;
    from: z.ZodNumber;
    durationInFrames: z.ZodNumber;
    unit: z.ZodEnum<["px", "%"]>;
    top: z.ZodNumber;
    left: z.ZodNumber;
    width: z.ZodNumber;
    height: z.ZodNumber;
    enforceRatio: z.ZodOptional<z.ZodTuple<[z.ZodNumber, z.ZodNumber], null>>;
    zIndex: z.ZodOptional<z.ZodNumber>;
    color: z.ZodString;
    isDragging: z.ZodBoolean;
}, "strip", z.ZodTypeAny, {
    left: number;
    top: number;
    color: string;
    id: string | number;
    width: number;
    height: number;
    durationInFrames: number;
    from: number;
    unit: "px" | "%";
    isDragging: boolean;
    enforceRatio?: [number, number] | undefined;
    zIndex?: number | undefined;
}, {
    left: number;
    top: number;
    color: string;
    id: string | number;
    width: number;
    height: number;
    durationInFrames: number;
    from: number;
    unit: "px" | "%";
    isDragging: boolean;
    enforceRatio?: [number, number] | undefined;
    zIndex?: number | undefined;
}>;
export type DnDProps = {
    readonly config: DnDConfig;
    readonly items: DragAndDropItem[];
    readonly setSelectedItem: React.Dispatch<React.SetStateAction<number | string | null>>;
    readonly selectedItem: number | string | null;
    readonly changeItem: (itemId: number | string, updater: (item: DragAndDropItem) => DragAndDropItem) => void;
};
export declare const DnDPropsSchema: z.ZodObject<{
    config: z.ZodObject<{
        selectionBorderColor: z.ZodOptional<z.ZodString>;
        selectionBorderSize: z.ZodOptional<z.ZodNumber>;
        resizeHandleSize: z.ZodOptional<z.ZodNumber>;
        resizeHandleBorderSize: z.ZodOptional<z.ZodNumber>;
        resizeHandleBorderColor: z.ZodOptional<z.ZodString>;
        resizeHandleBackgroundColor: z.ZodOptional<z.ZodString>;
        sanppingThreshold: z.ZodOptional<z.ZodNumber>;
        snapPositionsX: z.ZodOptional<z.ZodArray<z.ZodNumber, "many">>;
        snapPositionsY: z.ZodOptional<z.ZodArray<z.ZodNumber, "many">>;
    }, "strip", z.ZodTypeAny, {
        selectionBorderColor?: string | undefined;
        selectionBorderSize?: number | undefined;
        resizeHandleSize?: number | undefined;
        resizeHandleBorderSize?: number | undefined;
        resizeHandleBorderColor?: string | undefined;
        resizeHandleBackgroundColor?: string | undefined;
        sanppingThreshold?: number | undefined;
        snapPositionsX?: number[] | undefined;
        snapPositionsY?: number[] | undefined;
    }, {
        selectionBorderColor?: string | undefined;
        selectionBorderSize?: number | undefined;
        resizeHandleSize?: number | undefined;
        resizeHandleBorderSize?: number | undefined;
        resizeHandleBorderColor?: string | undefined;
        resizeHandleBackgroundColor?: string | undefined;
        sanppingThreshold?: number | undefined;
        snapPositionsX?: number[] | undefined;
        snapPositionsY?: number[] | undefined;
    }>;
    items: z.ZodArray<z.ZodObject<{
        id: z.ZodUnion<[z.ZodNumber, z.ZodString]>;
        from: z.ZodNumber;
        durationInFrames: z.ZodNumber;
        unit: z.ZodEnum<["px", "%"]>;
        top: z.ZodNumber;
        left: z.ZodNumber;
        width: z.ZodNumber;
        height: z.ZodNumber;
        enforceRatio: z.ZodOptional<z.ZodTuple<[z.ZodNumber, z.ZodNumber], null>>;
        zIndex: z.ZodOptional<z.ZodNumber>;
        color: z.ZodString;
        isDragging: z.ZodBoolean;
    }, "strip", z.ZodTypeAny, {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }, {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }>, "many">;
    setSelectedItem: z.ZodFunction<z.ZodTuple<[z.ZodUnion<[z.ZodNullable<z.ZodUnion<[z.ZodNumber, z.ZodString]>>, z.ZodFunction<z.ZodTuple<[z.ZodNullable<z.ZodUnion<[z.ZodNumber, z.ZodString]>>], z.ZodUnknown>, z.ZodNullable<z.ZodUnion<[z.ZodNumber, z.ZodString]>>>]>], z.ZodUnknown>, z.ZodVoid>;
    selectedItem: z.ZodNullable<z.ZodUnion<[z.ZodNumber, z.ZodString]>>;
    changeItem: z.ZodFunction<z.ZodTuple<[z.ZodUnion<[z.ZodNumber, z.ZodString]>, z.ZodFunction<z.ZodTuple<[z.ZodObject<{
        id: z.ZodUnion<[z.ZodNumber, z.ZodString]>;
        from: z.ZodNumber;
        durationInFrames: z.ZodNumber;
        unit: z.ZodEnum<["px", "%"]>;
        top: z.ZodNumber;
        left: z.ZodNumber;
        width: z.ZodNumber;
        height: z.ZodNumber;
        enforceRatio: z.ZodOptional<z.ZodTuple<[z.ZodNumber, z.ZodNumber], null>>;
        zIndex: z.ZodOptional<z.ZodNumber>;
        color: z.ZodString;
        isDragging: z.ZodBoolean;
    }, "strip", z.ZodTypeAny, {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }, {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }>], z.ZodUnknown>, z.ZodObject<{
        id: z.ZodUnion<[z.ZodNumber, z.ZodString]>;
        from: z.ZodNumber;
        durationInFrames: z.ZodNumber;
        unit: z.ZodEnum<["px", "%"]>;
        top: z.ZodNumber;
        left: z.ZodNumber;
        width: z.ZodNumber;
        height: z.ZodNumber;
        enforceRatio: z.ZodOptional<z.ZodTuple<[z.ZodNumber, z.ZodNumber], null>>;
        zIndex: z.ZodOptional<z.ZodNumber>;
        color: z.ZodString;
        isDragging: z.ZodBoolean;
    }, "strip", z.ZodTypeAny, {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }, {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }>>], z.ZodUnknown>, z.ZodVoid>;
}, "strip", z.ZodTypeAny, {
    config: {
        selectionBorderColor?: string | undefined;
        selectionBorderSize?: number | undefined;
        resizeHandleSize?: number | undefined;
        resizeHandleBorderSize?: number | undefined;
        resizeHandleBorderColor?: string | undefined;
        resizeHandleBackgroundColor?: string | undefined;
        sanppingThreshold?: number | undefined;
        snapPositionsX?: number[] | undefined;
        snapPositionsY?: number[] | undefined;
    };
    items: {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }[];
    setSelectedItem: (args_0: string | number | ((args_0: string | number | null, ...args_1: unknown[]) => string | number | null) | null, ...args_1: unknown[]) => void;
    selectedItem: string | number | null;
    changeItem: (args_0: string | number, args_1: (args_0: {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }, ...args_1: unknown[]) => {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }, ...args_2: unknown[]) => void;
}, {
    config: {
        selectionBorderColor?: string | undefined;
        selectionBorderSize?: number | undefined;
        resizeHandleSize?: number | undefined;
        resizeHandleBorderSize?: number | undefined;
        resizeHandleBorderColor?: string | undefined;
        resizeHandleBackgroundColor?: string | undefined;
        sanppingThreshold?: number | undefined;
        snapPositionsX?: number[] | undefined;
        snapPositionsY?: number[] | undefined;
    };
    items: {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }[];
    setSelectedItem: (args_0: string | number | ((args_0: string | number | null, ...args_1: unknown[]) => string | number | null) | null, ...args_1: unknown[]) => void;
    selectedItem: string | number | null;
    changeItem: (args_0: string | number, args_1: (args_0: {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }, ...args_1: unknown[]) => {
        left: number;
        top: number;
        color: string;
        id: string | number;
        width: number;
        height: number;
        durationInFrames: number;
        from: number;
        unit: "px" | "%";
        isDragging: boolean;
        enforceRatio?: [number, number] | undefined;
        zIndex?: number | undefined;
    }, ...args_2: unknown[]) => void;
}>;
//# sourceMappingURL=type.d.ts.map