var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
/* eslint-disable capitalized-comments */
import "reflect-metadata";
import { useMemo } from "react";
import { AbsoluteFill } from "remotion";
import { instanceToPlain, plainToClass } from "class-transformer";
import ClipSequence from "./clip-sequence.js";
import { disableContext, devModeContext } from "../layer/devModeContext.js";
import { Clip, editSpecificationToClips } from "../clip.js";
import { EditSpecification } from "../editSpecification.js";
import { EditlyService } from "../editlyService.js";
import { Layer } from "../layer/layer.js";
import { verbose } from "../../constants.js";
import { DnDComponent } from "./drag-and-drop/index.js";
export function getVideoStripLength(editSpecification, videoSettings) {
    return EditlyService.getTotalLength(editSpecification, videoSettings);
}
export function calculateClips(props) {
    var _a;
    const _props = props; // _.cloneDeep(props);
    const selectors = toSelector((_a = props === null || props === void 0 ? void 0 : props.editSpecification) === null || _a === void 0 ? void 0 : _a.selectors);
    if (verbose)
        console.log("RemotionVideo calculateClipsAndSequence", selectors);
    const data = selectors.calculateFromJson
        ? _props.editSpecificationJson
        : _props.editSpecification;
    const clips = editSpecificationToClips(data, _props.videoSettings, "RemotionVideo calculateClips");
    if (verbose)
        console.log("RemotionVideo calculateClipsAndSequence clips", clips === null || clips === void 0 ? void 0 : clips.length);
    return clips;
}
export const calculateMetadata = ({ defaultProps, props, }) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    if (verbose) {
        console.log("RemotionVideo calcVideoStripMetadata DEFAULT PROPS", JSON.stringify(defaultProps, null, 2));
        console.log("RemotionVideo calcVideoStripMetadata PROPS", JSON.stringify(props, null, 2));
    }
    let combinedProps = Object.assign(Object.assign({}, defaultProps), props);
    const selector = toSelector((_a = combinedProps === null || combinedProps === void 0 ? void 0 : combinedProps.editSpecification) === null || _a === void 0 ? void 0 : _a.selectors);
    if (selector.calculateFromJson) {
        console.log(" calculateMetadata: calculateFromJson");
        const editSpecification = EditSpecification.fromJSON(combinedProps.editSpecificationJson);
        combinedProps = Object.assign(Object.assign({}, combinedProps), { editSpecification });
        const clips = calculateClips(combinedProps);
        combinedProps = Object.assign(Object.assign({}, combinedProps), { clips });
    }
    else if (selector.calculateClips) {
        console.log(" calculateMetadata: calculateClips");
        const clips = calculateClips(combinedProps);
        combinedProps = Object.assign(Object.assign({}, combinedProps), { clips });
    }
    if (verbose)
        console.log("calcVideoStripMetadata clips", JSON.stringify(combinedProps.clips, null, 2));
    if (verbose)
        console.log("calcVideoStripMetadata props", combinedProps);
    return {
        props: combinedProps,
        durationInFrames: combinedProps.videoSettings.durationInFrames,
        width: combinedProps.videoSettings.width,
        height: combinedProps.videoSettings.height,
        fps: combinedProps.videoSettings.frameRate,
        // DefaultCodec: "h264"
    };
});
export function calculateClipSequence(clips, selectors) {
    if (!clips)
        return [_jsx("div", { children: "\"clips[] is null\"" })];
    if (!Array.isArray(clips))
        return [_jsx("div", { children: "\"clips[] is not an Array\"" })];
    const clipSequences = clips.map((clip, clipIndex) => {
        if (verbose) {
            console.log(`calculateClipSequence clip #${clipIndex} :`);
        }
        const _clip = plainToClass(Clip, instanceToPlain(clip)); // Clone needed to avoid changing original clip (Render issue with Remotion, all clips start at 0)
        // const _clip = clip;
        const previousClip = clipIndex - 1 >= 0 ? clips[clipIndex - 1] : undefined;
        const _previousClip = previousClip // Clone needed to avoid changing original clip (Render issue with Remotion, all clips start at 0)
            ? plainToClass(Clip, instanceToPlain(previousClip))
            : undefined;
        // const _previousClip = previousClip;
        const clipSequenceProps = {
            clip: _clip,
            clipIndex,
            previousClip: _previousClip,
            selectors,
        };
        const key = `ClipSequence-${clipIndex}`;
        const clipSequence = _jsx(ClipSequence, Object.assign({}, clipSequenceProps), key);
        return clipSequence;
    });
    if (verbose) {
        console.log("RemotionVideo calculateClipSequence CLIP_SEQUENCES:", JSON.stringify(clipSequences, null, 2));
    }
    return clipSequences;
}
export const VideoStrip = ({ editSpecificationJson, editSpecification, videoSettings, clips, dndProps, }) => {
    const _selectors = toSelectorMap(editSpecification.selectors);
    const selector = toSelector(editSpecification.selectors);
    // const clipSequence = calculateClipSequence(
    //   _clips,
    //   editSpecification.selectors
    // );
    const clipSequence = useMemo(() => calculateClipSequence(clips, editSpecification.selectors), [clips, editSpecification.selectors]);
    const chessPattern = "linear-gradient(45deg, #808080 25%, transparent 25%), linear-gradient(-45deg, #808080 25%, transparent 25%), linear-gradient(45deg, transparent 75%, #808080 75%), linear-gradient(-45deg, transparent 75%, #808080 75%)";
    const backgroundStyle = {
        backgroundImage: chessPattern,
        backgroundSize: "240px 240px",
        backgroundPosition: "0 0, 0 120px, 120px -120px, -120px 0px",
    };
    return (_jsxs(AbsoluteFill, { id: "VideoStrip", style: Object.assign({}, (selector.isPlayer ? backgroundStyle : null)), children: [_jsx(disableContext.Consumer, { children: (disable) => {
                    if (disable.includes("ClipSequence"))
                        return _jsx("p", { children: "ClipSequence disabled" });
                    return clipSequence;
                } }), _jsx(disableContext.Consumer, { children: (disable) => {
                    if (disable.includes("clipInfos"))
                        return null;
                    return clipInfos(clips, _selectors);
                } }), dndProps && (_jsx(DnDComponent, { config: dndProps.config, items: dndProps.items, selectedItem: dndProps.selectedItem, changeItem: dndProps.changeItem, setSelectedItem: dndProps.setSelectedItem }))] }));
};
// #region clipInfos
const clipInfos = (clips, selectors) => {
    if (!clips)
        return _jsx("div", { children: "\"clips[] is null\"" });
    const clipSequence = clips === null || clips === void 0 ? void 0 : clips.map((clip, clipIndex) => {
        return clipInfo(clip, clipIndex);
    });
    const selectorsInfo = Array.from(selectors).map((selector, index) => {
        return (_jsxs("p", { style: {
                fontFamily: "arial",
                color: "white",
                fontSize: 26,
            }, children: ["`", selector[0], ": ", selector[1], "`"] }, `selectorsInfo-${index}`));
    });
    return (_jsx(devModeContext.Consumer, { children: (devMode) => {
            if (!devMode)
                return null;
            return (_jsxs("div", { style: {
                    zIndex: 1000,
                    color: "white",
                    position: "absolute",
                    top: "50%",
                    left: "50%",
                    transform: "translate(-50%, -50%)",
                    fontSize: 64,
                    fontWeight: "semibold",
                    letterSpacing: "1px",
                    width: "80%",
                    overflow: "hidden",
                    display: "block",
                    backgroundColor: "rgba(0,0,0,0.8)",
                }, children: [selectorsInfo, _jsxs("div", { style: {
                            color: "white",
                            fontSize: 32,
                            fontWeight: "bold",
                        }, children: ["clip count: ", clips === null || clips === void 0 ? void 0 : clips.length] }), _jsx("div", { style: {
                            marginLeft: "80px",
                        }, children: clipSequence })] }));
        } }));
};
const clipInfo = (clip, index) => {
    var _a, _b, _c;
    if (!clip)
        return _jsx("div", { children: "clip is null" }, `clipInfo-${index}`);
    const layerSequence = (_a = clip.layers) === null || _a === void 0 ? void 0 : _a.map((layer, layerIndex) => {
        return layerInfo(layer, layerIndex);
    });
    return (_jsxs("div", { style: {
            color: "white",
            fontSize: 24,
            fontWeight: "semibold",
        }, children: ["CLIP: ", (_b = clip.absoluteRange) === null || _b === void 0 ? void 0 : _b.toString(), " ", clip.name, _jsxs("div", { style: {
                    marginLeft: "80px",
                }, children: [_jsxs("div", { children: ["layer count: ", (_c = clip.layers) === null || _c === void 0 ? void 0 : _c.length] }), layerSequence] })] }, `clipInfo-${index}`));
};
const layerInfo = (layer, index) => {
    if (!layer)
        return _jsx("div", { children: "layer is null" }, `layerInfo-${index}`);
    return (_jsx("div", { style: {
            color: "white",
            fontSize: 20,
        }, children: Layer.toInfoString(layer) }, `layerInfo-${index}`));
};
// #endregion
function isIterable(obj) {
    return obj && typeof obj[Symbol.iterator] === "function";
}
function toSelectorMap(selectors) {
    const _selectors = new Map();
    if (!selectors) {
        if (verbose)
            console.log("toSelectorMap selectors is null");
        return _selectors;
    }
    // For every key in selectors add to map
    // eslint-disable-next-line guard-for-in
    for (const key in selectors) {
        const value = selectors[key];
        if (verbose)
            console.log(`toSelectorMap key: ${key} value: ${value}`);
        _selectors.set(key, value);
    }
    return _selectors;
}
function toSelector(selectors) {
    if (!selectors)
        return {};
    return {
        language: selectors.language,
        logLevel: selectors.logLevel,
        isPlayer: selectors.isPlayer === true,
        calculateClips: selectors.calculateClips === true,
        calculateFromJson: selectors.calculateFromJson === true,
    };
}
//# sourceMappingURL=video-strip.js.map