import "reflect-metadata";
import { Observable } from "rxjs";
import { immerable } from "immer";
import { LayerType } from "../enums/index.js";
import { LottieData } from "../types/index.js";
import { IClip, IEditSpecification, IVideoSettings, ISelectors, ILibraryItem, MediaType, AssetKey } from "../interfaces/index.js";
import { MediaAssetVariant } from "./mediaAsset.js";
import { LibraryItem } from "./library-item.js";
import { Layer } from "./layer/layer.js";
export declare class EditSpecification implements IEditSpecification {
    [immerable]: boolean;
    invokerId?: string | null;
    myProjectId: string | null;
    myVideoId: string | null;
    outPath: string | null;
    selectors: ISelectors;
    audioFilePath: string;
    keepSourceAudio: boolean;
    assets: MediaAssetVariant<AssetKey>[];
    clips: IClip[];
    defaults: IClip | null;
    renderedVideoId?: string | undefined;
    private _videoSettings;
    get language(): string;
    set language(language: string);
    get videoSettings(): IVideoSettings;
    set videoSettings(videoSettings: IVideoSettings);
    get fps(): number;
    set fps(fps: number);
    get width(): number;
    set width(width: number);
    get height(): number;
    set height(height: number);
    get durationInFrames(): number;
    set durationInFrames(durationInFrames: number);
    get libraryItems(): string[];
    get comparableClips(): IClip[];
    constructor(specification?: IEditSpecification | null, videoSettings?: IVideoSettings);
    getMissingTranslations(languageCodes: string[]): string[];
    setOutPath(fileName: string, extention: string, parameter?: string): void;
    setLanguage(language: string): void;
    setAlphaChannel(alphaChannel: boolean): void;
    activateSubtitles(): void;
    deactivateSubtitles(): void;
    setVideoSettings(videoSettings: IVideoSettings): void;
    updateLottieAnimationDataFromPath$(skip?: boolean): Observable<EditSpecification>;
    static setLottieAnimationData(spec: EditSpecification, data: {
        url: string;
        data: any;
    }[]): EditSpecification;
    private getLottiePaths;
    setLottieDataObj(lottieArray: {
        url: string;
        data: any;
    }[]): void;
    setLottieData(lottieMap: Map<string, LottieData>): void;
    static compare(a?: EditSpecification, b?: EditSpecification): boolean;
    equals(other: EditSpecification | undefined): boolean;
    static compareIEditSpecification(objA: Partial<IEditSpecification> | undefined, objB: Partial<IEditSpecification> | undefined): boolean;
    toIEditSpecification(projectId?: string): IEditSpecification;
    toJSON(): string;
    getMd5(): string;
    clone(): EditSpecification;
    withDefaults(defaults: IClip | null): EditSpecification;
    setDefaults(defaults: IClip | null | undefined): void;
    static fromILibraryItem(libItem: ILibraryItem, subtitleItem?: ILibraryItem, languageCode?: string): Promise<EditSpecification>;
    static fromLibraryItem(libItem: LibraryItem, subtitleItem?: LibraryItem, languageCode?: string): Promise<EditSpecification>;
    static fromIEditSpecification(data?: IEditSpecification): EditSpecification;
    static fromJSON(json: string): EditSpecification;
    static libraryItemToSubtitleLayer(libItem: LibraryItem, videoSettings: IVideoSettings, styles?: object): Promise<Layer | null>;
    static setIds(myProjectId: string, editSpecification: Partial<IEditSpecification>): Partial<IEditSpecification>;
    static default(): EditSpecification;
    static empty(): EditSpecification;
}
export declare function mediaTypeToLayerType(mediaType: MediaType): LayerType;
export declare function layerTypeToMediaType(layerType: LayerType): MediaType;
//# sourceMappingURL=editSpecification.d.ts.map