import { jsxs as _jsxs, jsx as _jsx } from "react/jsx-runtime";
import "reflect-metadata";
import { useMemo, useState } from "react";
import { Sequence, Audio, staticFile, useCurrentFrame, Img, useVideoConfig, } from "remotion";
import { LayerType, SelectorType } from "../../enums/index.js";
import { Frame } from "../frame.js";
import { Layer } from "./layer.js";
import { Transition } from "../transition.js";
import { PlotComponent } from "./video.layer.js";
import { devModeContext, disableContext } from "./devModeContext.js";
import RenderGuard from "../components/renderGuard.component.js";
import { FrameRange } from "../frameRange.js";
import { clamp } from "../../extensions/number.extentions.js";
import { BoxStyle } from "../boxModel/boxStyle.js";
import { Box } from "../boxModel/box.js";
export class AudioLayer extends Layer {
    get minDuration() {
        var _a, _b;
        return (_b = (_a = this.innerBoundaryRange) === null || _a === void 0 ? void 0 : _a.duration) !== null && _b !== void 0 ? _b : 25;
    }
    get maxDuration() {
        var _a;
        return (_a = this.outerBoundaryRange.duration) !== null && _a !== void 0 ? _a : this.duration;
    }
    get volume() {
        return this._volume;
    }
    set volume(value) {
        this._volume = clamp(value, 0, 1);
    }
    constructor(layer, asset, path, duration, volume = 1) {
        super(LayerType.Audio, layer, asset, null, null, "");
        this.path = path;
        this.duration = duration;
        this.__LAYER__ = "AudioLayer";
        this._volume = 1;
        // This.relativeRange = FrameRange.fromStart(0, duration);
        this.volume = volume;
        if (duration <= 0)
            throw new Error(`AudioClip duration must be bigger 0! ${duration}`);
    }
    static fromILayer(iLayer, assets, videoSettings, duration) {
        var _a;
        const asset = assets.find((a) => a.key.id === iLayer.assetId);
        const path = (_a = asset === null || asset === void 0 ? void 0 : asset.value.sourceFile.cloudDownloadPath) !== null && _a !== void 0 ? _a : iLayer.path;
        if (!path)
            throw new Error("AudioClip needs a path!");
        if (videoSettings.frameRate <= 0)
            throw new Error(`AudioClip fps:${videoSettings.frameRate} must be bigger than 0`);
        const frameDuration = iLayer.duration && iLayer.duration > 0
            ? Frame.framesFromSeconds(iLayer.duration, videoSettings.frameRate)
            : Frame.framesFromSeconds(duration, videoSettings.frameRate);
        const audioLayer = new AudioLayer(iLayer, asset, path, frameDuration, 1);
        audioLayer.assetId = asset === null || asset === void 0 ? void 0 : asset.key.id;
        if (iLayer.mixVolume || iLayer.mixVolume === 0)
            audioLayer.volume = iLayer.mixVolume;
        return audioLayer;
    }
    getILayer() {
        var _a, _b;
        let durationInSec = this.durationInSeconds;
        if (durationInSec && durationInSec <= 0)
            durationInSec = null;
        const result = Object.assign(Object.assign({}, this.toBaseILayer()), { type: LayerType.Audio, path: (_a = this.path) !== null && _a !== void 0 ? _a : null, mixVolume: (_b = this.volume) !== null && _b !== void 0 ? _b : 0 });
        return result;
    }
    getTitle() {
        if (!this.path)
            return "audio";
        const url = new URL(this.path);
        const filename = url.pathname.substring(url.pathname.lastIndexOf("/") + 1);
        return `${filename}`;
    }
    toInfoString() {
        return `AUDIO LAYER: ${this.absoluteRange.toString()} ${this.zIndex} ${this.name} ${this.path}`;
    }
    getProperties() {
        const result = [];
        return result;
    }
    setProperty(name, value) { }
    updatePropertyValue(name, value) { }
}
export const AudioComponent = ({ sequenceRange, index, layer, path, selectors, }) => {
    const frame = useCurrentFrame();
    const { fps, width, height, durationInFrames } = useVideoConfig();
    const isPreview = selectors && selectors[SelectorType.IsPreview] === true;
    const audioLayer = useMemo(() => {
        const self = layer;
        self.spacialBox.parent = new Box({
            size: [width, height],
        });
        return self;
    }, [height, layer, width]);
    const spacialBoxStyle = useMemo(() => new BoxStyle(audioLayer.spacialBox).getStyleInPercent(), [audioLayer.spacialBox]);
    const layerDisabled = useMemo(() => audioLayer.renderDisabled || audioLayer.renderDisabledLocal, [audioLayer.renderDisabled, audioLayer.renderDisabledLocal]);
    const source = useMemo(() => path
        ? (path === null || path === void 0 ? void 0 : path.startsWith("http"))
            ? path
            : staticFile(`/${path}`)
        : undefined, [path]);
    const iconSource = selectors && selectors.audioIcon
        ? selectors.audioIcon
        : "https://firebasestorage.googleapis.com/v0/b/user-generated-content-app.appspot.com/o/ICONS%2FIconAudio16_9.jpg?alt=media&token=f5911066-5172-4a60-9b55-6701efc81fe8";
    const absoluteRange = useMemo(() => sequenceRange.round().pushInside(FrameRange.positive), [sequenceRange]);
    const cutRange = useMemo(() => { var _a; return (_a = layer.cut) === null || _a === void 0 ? void 0 : _a.round(); }, [layer.cut]);
    const volumeTransitions = useMemo(() => {
        return audioLayer.getTransitions("volume", absoluteRange);
    }, [audioLayer, absoluteRange]);
    const transVolumeFn = (frame) => {
        return Transition.calculateNumberFromTransitions(volumeTransitions, frame);
    };
    const volumeFn = (frame) => {
        return audioLayer.volume * transVolumeFn(frame);
    };
    const [imageError, setImageError] = useState(false);
    const handleError = (event) => {
        setImageError(true);
        console.error("Image failed to load:", event.currentTarget.src);
        // Optionally, you can set a fallback image here
    };
    const valid = source && absoluteRange.isValid();
    const message = `absoluteRange.isValid()=${absoluteRange.isValid()} source=${source}`;
    const size = 300;
    const info = (_jsx(devModeContext.Consumer, { children: (devMode) => {
            var _a, _b, _c;
            if (!devMode)
                return null;
            const activeVolumeTransition = Transition.activeTransitionFromTransitions(volumeTransitions, frame);
            const volumeX = (_a = activeVolumeTransition === null || activeVolumeTransition === void 0 ? void 0 : activeVolumeTransition.calculateNormPosition(frame)) !== null && _a !== void 0 ? _a : 0.5;
            return (_jsxs("div", { style: Object.assign(Object.assign({ opacity: activeVolumeTransition ? 1 : 0 }, spacialBoxStyle), { zIndex: 100, top: (((_b = audioLayer.zIndex) !== null && _b !== void 0 ? _b : 1) % 3) * 350, left: ((_c = audioLayer.zIndex) !== null && _c !== void 0 ? _c : 1) * 20 }), children: [_jsxs("p", { style: {
                            fontSize: 30,
                            textShadow: "0px 0px 4px black",
                        }, children: [activeVolumeTransition ? "X" : "0", " AUDIO volume:", " ", volumeFn(frame).toFixed(2), " ", activeVolumeTransition === null || activeVolumeTransition === void 0 ? void 0 : activeVolumeTransition.toString()] }), _jsx(PlotComponent, { x: 40, y: 0, height: size, width: size, labelX: "time", labelY: "AUDIO volume", valueX: volumeX, valueY: volumeFn(frame), color: "#ff00c3" }), _jsx(PlotComponent, { x: 70, y: 0, height: size, width: size, labelX: "time", labelY: "TRANS volume", valueX: volumeX, valueY: volumeFn(frame), color: "#ff00c3" })] }));
        } }));
    return (_jsx(RenderGuard, { disabled: layerDisabled || !valid, message: message, children: _jsx(disableContext.Consumer, { children: (disable) => {
                var _a;
                if (disable === null || disable === void 0 ? void 0 : disable.includes("AudioComponent"))
                    return null;
                const icon = isPreview && !imageError ? (_jsx(Img, { src: iconSource, style: Object.assign({ zIndex: (_a = audioLayer.zIndex) !== null && _a !== void 0 ? _a : 1 }, spacialBoxStyle), onError: handleError }, "Layer Icon" + index)) : null;
                return (_jsxs(Sequence, { name: "video-sequence", from: absoluteRange.start, durationInFrames: absoluteRange.duration, children: [icon, _jsx(Audio, { src: source, startFrom: cutRange === null || cutRange === void 0 ? void 0 : cutRange.start, endAt: cutRange === null || cutRange === void 0 ? void 0 : cutRange.end, volume: (f) => volumeFn(f) }, "Layer" + index), info] }, "videoSequence" + index));
            } }) }));
};
//# sourceMappingURL=audio.layer.js.map