import { jsx as _jsx } from "react/jsx-runtime";
import "reflect-metadata";
import { useMemo, useState } from "react";
import { Sequence, Img, staticFile, useCurrentFrame, useVideoConfig, } from "remotion";
import { LayerType } from "../../enums/index.js";
import { Frame } from "../frame.js";
import { Layer } from "./layer.js";
import { disableContext } from "./devModeContext.js";
import RenderGuard from "../components/renderGuard.component.js";
import { FrameRange } from "../frameRange.js";
import { Transition } from "../transition.js";
import { Box, BoxStyle } from "../boxModel/index.js";
export class ImageLayer extends Layer {
    get minDuration() {
        var _a, _b;
        return (_b = (_a = this.innerBoundaryRange) === null || _a === void 0 ? void 0 : _a.duration) !== null && _b !== void 0 ? _b : 1;
    }
    get maxDuration() {
        var _a;
        return (_a = this.outerBoundaryRange.duration) !== null && _a !== void 0 ? _a : this.duration;
    }
    constructor(layer, asset, path, duration, position = null, size = null, name = "") {
        super(LayerType.Image, layer, asset, position, size, name);
        this.path = path;
        this.duration = duration;
        this.__LAYER__ = "ImageLayer";
        if (duration <= 0)
            throw new Error(`ImageLayer duration must be bigger 0! ${duration}`);
    }
    getILayer() {
        var _a, _b;
        let durationInSec = this.durationInSeconds;
        if (durationInSec && durationInSec <= 0)
            durationInSec = null;
        const result = Object.assign(Object.assign({}, this.toBaseILayer()), { type: LayerType.Image, path: (_a = this.path) !== null && _a !== void 0 ? _a : null, size: (_b = this.imageSize) !== null && _b !== void 0 ? _b : null });
        return result;
    }
    getTitle() {
        if (!this.path)
            return "image";
        const url = new URL(this.path);
        const filename = url.pathname.substring(url.pathname.lastIndexOf("/") + 1);
        return `${filename}`;
    }
    toInfoString() {
        return `IMAGE LAYER: ${this.absoluteRange.toString()} ${this.zIndex} ${this.name} ${this.path}`;
    }
    getProperties() {
        const result = [];
        return result;
    }
    setProperty(name, value) { }
    updatePropertyValue(name, value) { }
    static fromILayer(iLayer, assets, videoSettings, duration) {
        var _a;
        const asset = assets.find((a) => a.key.id === iLayer.assetId);
        const path = (_a = asset === null || asset === void 0 ? void 0 : asset.value.sourceFile.cloudDownloadPath) !== null && _a !== void 0 ? _a : iLayer.path;
        if (!path)
            throw new Error("ImageClip needs a path!");
        if (videoSettings.frameRate <= 0)
            throw new Error(`ImageClip fps:${videoSettings.frameRate} must be bigger than 0`);
        const frameDuration = iLayer.duration && iLayer.duration > 0
            ? Frame.framesFromSeconds(iLayer.duration, videoSettings.frameRate)
            : Frame.framesFromSeconds(duration, videoSettings.frameRate);
        const normalizedSize = Layer.aspectLayerSize(iLayer, videoSettings);
        const imageLayer = new ImageLayer(iLayer, asset, path, Math.max(frameDuration, 25), null, normalizedSize);
        imageLayer.imageSize = iLayer.size;
        return imageLayer;
    }
}
export const ImageComponent = ({ sequenceRange, index, layer, path, selectors, }) => {
    const frame = useCurrentFrame();
    const { fps, width, height, durationInFrames } = useVideoConfig();
    const imageLayer = useMemo(() => {
        const self = layer;
        self.spacialBox.parent = new Box({
            size: [width, height],
        });
        return self;
    }, [height, layer, width]);
    const spacialBoxStyle = useMemo(() => new BoxStyle(imageLayer.spacialBox).getStyleInPercent(), [imageLayer.spacialBox]);
    const layerDisabled = useMemo(() => imageLayer.renderDisabled || imageLayer.renderDisabledLocal, [imageLayer.renderDisabled, imageLayer.renderDisabledLocal]);
    const source = useMemo(() => path
        ? (path === null || path === void 0 ? void 0 : path.startsWith("http"))
            ? path
            : staticFile(`/${path}`)
        : undefined, [path]);
    const absoluteRange = useMemo(() => sequenceRange.round().pushInside(FrameRange.positive), [sequenceRange]);
    const opacityTransitions = useMemo(() => {
        return imageLayer.getTransitions("opacity", absoluteRange);
    }, [imageLayer, absoluteRange]);
    const opacityFn = (frame) => {
        return Transition.calculateNumberFromTransitions(opacityTransitions, frame);
    };
    const [imageError, setImageError] = useState(false);
    const handleError = (event) => {
        setImageError(true);
        console.error("Image failed to load:", event.currentTarget.src);
        // Optionally, you can set a fallback image here
    };
    const valid = source && absoluteRange.isValid() && !imageError;
    const message = `absoluteRange.isValid()=${absoluteRange.isValid()} source=${source}`;
    return (_jsx(RenderGuard, { disabled: layerDisabled || !valid, message: message, children: _jsx(disableContext.Consumer, { children: (disable) => {
                var _a;
                if (disable === null || disable === void 0 ? void 0 : disable.includes("ImageComponent"))
                    return null;
                return (_jsx(Sequence, { name: "image-sequence", from: absoluteRange.start, durationInFrames: absoluteRange.duration, children: _jsx(Img, { src: source, style: Object.assign(Object.assign({}, spacialBoxStyle), { pointerEvents: "none", opacity: opacityFn(frame), zIndex: (_a = imageLayer.zIndex) !== null && _a !== void 0 ? _a : 1 }), onError: handleError }, "Layer" + index) }, "imageSequence" + index));
            } }) }));
};
//# sourceMappingURL=image.layer.js.map