import "reflect-metadata";
import { immerable } from "immer";
import { LayerType, TimingAction } from "../../enums/index.js";
import { AssetKey, ILayer, IProperty, ISize, IVideoSettings } from "../../interfaces/index.js";
import { NormalizedSize } from "../normalizedSize.js";
import { TimeLineElement } from "../timeLineElement.js";
import { BoxModel } from "../boxModel.js";
import { FrameRange } from "../frameRange.js";
import { Position } from "../position.js";
import type { MediaAssetVariant } from "../mediaAsset.js";
import { Box } from "../boxModel/box.js";
export declare abstract class Layer extends TimeLineElement {
    __LAYER__: string;
    [immerable]: boolean;
    assetId?: string;
    alphaChannel: boolean;
    private _language;
    get language(): string;
    set language(value: string);
    get languages(): string[];
    styles: Map<string, string>;
    type: LayerType;
    duration: number;
    lockedAtParent: boolean;
    hiddenInTimeline: boolean;
    justifyContent: string;
    alignItems: string;
    children: Layer[];
    box: BoxModel;
    spacialBox: Box;
    get cutOrRange(): FrameRange;
    private _position;
    get position(): Position;
    set position(value: Position);
    sourceSize: ISize | null;
    sourceRatioFraction: [number, number] | null;
    private _size;
    get size(): NormalizedSize;
    set size(value: NormalizedSize);
    private _name;
    get name(): string;
    set name(value: string);
    protected constructor(type: LayerType, layer: ILayer, asset: MediaAssetVariant<AssetKey> | undefined, position?: Position | null, size?: NormalizedSize | null, name?: string);
    protected languageChanged(): void;
    protected toBaseILayer(): ILayer;
    moveToLeft(delta: number): void;
    moveToRight(delta: number): void;
    trimmLeft(delta: number, moveToRight?: boolean, absMoveToRight?: boolean): void;
    trimmRight(delta: number): void;
    snapCutLeft(): void;
    snapCutRight(): void;
    resetTrimm(): void;
    abstract getILayer(): ILayer;
    abstract getTitle(): string;
    abstract getProperties(): IProperty[];
    abstract setProperty(name: string, value: IProperty): void;
    abstract updatePropertyValue(name: string, value: string): void;
    abstract toInfoString(): string;
    static toInfoString(layer: Layer): string;
    abstract get minDuration(): number | undefined;
    abstract get maxDuration(): number | undefined;
    get positionType(): "static" | "relative" | "fixed" | "absolute" | "sticky";
    get textAlignment(): "left" | "center" | "start" | "end" | "right";
    setRelativeTiming(parentInsideRange: FrameRange, startOffset?: number, endOffset?: number, timingAction?: TimingAction): void;
    setAbsoluteTimingFromParent(parent: TimeLineElement | undefined): void;
    fixCutRange(): void;
    static getlastFrameFromLayers(layers: Layer[], range: FrameRange): number;
    setAspectLayerSize(viewPort: ISize): void;
    setAspectLayerSize(viewPort: ISize, layerSize?: ISize): void;
    static aspectLayerSize(layer: ILayer, videoSettings: IVideoSettings): NormalizedSize;
    private static getAspectSize;
}
//# sourceMappingURL=layer.d.ts.map