import "reflect-metadata";
import { LayerType, AnchorType } from "../../enums/index.js";
import { NormalizedSize } from "../normalizedSize.js";
import { Transition } from "../transition.js";
import { VideoSettings } from "../videoSettings.js";
import { CssValue } from "../cssValue.js";
import { Frame } from "../frame.js";
import { FrameRange } from "../frameRange.js";
import { Position } from "../position.js";
import { Layer } from "./layer.js";
export class SlideInTextLayer extends Layer {
    get minDuration() {
        return undefined;
    }
    get maxDuration() {
        return undefined;
    }
    get textWidth() {
        if (this.textSettings)
            return CssValue.getTextWidthFromITextSettings(this.text, this.textSettings);
        return 0;
    }
    // eslint-disable-next-line max-params
    constructor(layer, asset, text, duration, animationDuration = 25, position, size, textSettings, name = "") {
        super(LayerType.SlideInText, layer, asset, position, size, name);
        this.text = text;
        this.duration = duration;
        this.animationDuration = animationDuration;
        this.textSettings = textSettings;
        this.__LAYER__ = "SlideInTextLayer";
        this.relativeRange = FrameRange.fromStart(0, duration);
    }
    static fromILayer(iLayer, assets, videoSettings, duration) {
        var _a, _b, _c, _d, _e, _f;
        const asset = assets.find((a) => a.key.id === iLayer.assetId);
        const path = (_a = asset === null || asset === void 0 ? void 0 : asset.value.sourceFile.cloudDownloadPath) !== null && _a !== void 0 ? _a : iLayer.path;
        if (!path && !iLayer.subtitlesLocale)
            throw new Error("Subtitle needs a path or subtitle!");
        const viewPort = VideoSettings.sizeFromVideoSettings(videoSettings);
        if (videoSettings.frameRate <= 0)
            throw new Error(`SlideInText fps:${videoSettings.frameRate} must be bigger than 0`);
        const frameDuration = iLayer.duration
            ? Frame.framesFromSeconds(iLayer.duration, videoSettings.frameRate)
            : Frame.framesFromSeconds(duration, videoSettings.frameRate);
        let fontSize = "5rem";
        if (iLayer.fontSize)
            fontSize = `${iLayer.fontSize * 100}vh`;
        const textSettings = {
            fontSize,
            textColor: (_b = iLayer.textColor) !== null && _b !== void 0 ? _b : "#000",
            fontWeight: (_c = iLayer.fontWeight) !== null && _c !== void 0 ? _c : "normal",
            fontFamily: "DINPro-Light",
            charSpacing: null,
        };
        const textWidth = CssValue.getTextWidthFromITextSettings((_d = iLayer.text) !== null && _d !== void 0 ? _d : "Placeholder!", textSettings);
        const textHeight = CssValue.getTextHeightFromITextSettings((_e = iLayer.text) !== null && _e !== void 0 ? _e : "Placeholder!", textSettings);
        const size = NormalizedSize.fromViewPort({ width: textWidth, height: textHeight }, viewPort);
        const position = Position.fromIPosition(iLayer.position);
        position.anchorX = AnchorType.Center;
        position.anchorY = AnchorType.Center;
        const result = new SlideInTextLayer(iLayer, asset, (_f = iLayer.text) !== null && _f !== void 0 ? _f : "Placeholder!", frameDuration, 20, position, size, textSettings, "SlideIn fromILayer");
        result.durationInSeconds = iLayer.duration;
        if (iLayer.relativeRange)
            result.relativeRange = FrameRange.fromIFrameRange(iLayer.relativeRange);
        if (iLayer.id)
            result.id = iLayer.id;
        result.libId = iLayer.libId;
        result.transitions = Transition.fromITransitions(iLayer.transitions);
        return result;
    }
    getILayer() {
        var _a;
        return Object.assign(Object.assign({}, this.toBaseILayer()), { type: LayerType.SlideInText, text: (_a = this.text) !== null && _a !== void 0 ? _a : null });
    }
    getTitle() {
        return `${this.text.slice(-20)}`;
    }
    toInfoString() {
        return `SLIDE IN TEXT LAYER: ${this.absoluteRange.toString()} ${this.zIndex} ${this.name} ${this.getTitle()}`;
    }
    getProperties() {
        const result = [];
        return result;
    }
    setProperty(name, value) { }
    updatePropertyValue(name, value) { }
}
//# sourceMappingURL=slideInText.layer.js.map