var _a;
import { jsx as _jsx } from "react/jsx-runtime";
/* eslint-disable complexity */
/* eslint-disable capitalized-comments */
import "reflect-metadata";
import _ from "lodash";
import { Sequence, useCurrentFrame, useVideoConfig } from "remotion";
import Balancer from "react-wrap-balancer";
import { useMemo } from "react";
import { immerable, produce } from "immer";
import { LayerType } from "../../enums/index.js";
import { Layer } from "./layer.js";
import { Transition } from "../transition.js";
import { disableContext } from "./devModeContext.js";
import { Aspect } from "../../enums/aspect.enum.js";
import { Frame } from "../frame.js";
import { FrameRange } from "../frameRange.js";
import { FrameRangeCollection } from "../frameRangeCollection.js";
import { WebVTT, VTTCue } from "../webVTT.js";
import { Position } from "../position.js";
import RenderGuard from "../components/renderGuard.component.js";
import { Box, BoxStyle } from "../boxModel/index.js";
export class SubtitleLayer extends Layer {
    subtitleByLanguage(language) {
        var _b;
        if (!this.subtitlesLocale)
            return null;
        const subtitleByLanguage = (_b = this.subtitlesLocale.find((x) => x.key.languageCode === language)) === null || _b === void 0 ? void 0 : _b.value;
        return subtitleByLanguage !== null && subtitleByLanguage !== void 0 ? subtitleByLanguage : null;
    }
    get subtitle() {
        var _b;
        if (!this.subtitlesLocale)
            return null;
        const subtitleByLanguage = (_b = this.subtitlesLocale.find((x) => x.key.languageCode === this.language)) === null || _b === void 0 ? void 0 : _b.value;
        return subtitleByLanguage !== null && subtitleByLanguage !== void 0 ? subtitleByLanguage : null;
    }
    set subtitle(value) {
        if (!value)
            return;
        if (!this.subtitlesLocale) {
            this.subtitlesLocale = [
                {
                    key: { languageCode: this.language },
                    value,
                },
            ];
            return;
        }
        const sub = this.subtitlesLocale.find((x) => x.key.languageCode === this.language);
        if (sub) {
            sub.value = value;
        }
        else {
            this.subtitlesLocale.push({
                key: { languageCode: this.language },
                value,
            });
        }
    }
    get allSubtitles() {
        if (!this.subtitlesLocale)
            return [];
        return this.subtitlesLocale.map((x) => x.value);
    }
    get languages() {
        if (!this.subtitlesLocale)
            return [];
        return this.subtitlesLocale.map((x) => x.key.languageCode);
    }
    get minDuration() {
        var _b, _c;
        return (_c = (_b = this.innerBoundaryRange) === null || _b === void 0 ? void 0 : _b.duration) !== null && _c !== void 0 ? _c : 25;
    }
    get maxDuration() {
        return this.outerBoundaryRange.duration;
    }
    // eslint-disable-next-line max-params
    constructor(layer, asset, path, duration, position = null, size = null, name = "") {
        super(LayerType.Subtitle, layer, asset, position, size, name);
        this.path = path;
        this.duration = duration;
        this.__LAYER__ = "SubtitleLayer";
        this[_a] = true;
        this.volume = 1;
        if (name)
            this.name = name;
        this.zPreference = "high";
        this.box.position = Position.centerCenter();
    }
    static fromILayer(iLayer, assets, videoSettings, duration) {
        var _b, _c, _d, _e;
        const asset = assets.find((a) => a.key.id === iLayer.assetId);
        const path = (_b = asset === null || asset === void 0 ? void 0 : asset.value.sourceFile.cloudDownloadPath) !== null && _b !== void 0 ? _b : iLayer.path;
        if (!path && !iLayer.subtitlesLocale)
            throw new Error("Subtitle needs a path or subtitle!");
        if (videoSettings.frameRate <= 0)
            throw new Error(`Subtitle fps:${videoSettings.frameRate} must be bigger than 0`);
        const durationInSeconds = (_c = iLayer.duration) !== null && _c !== void 0 ? _c : duration;
        const durationInFrames = Frame.framesFromSeconds(durationInSeconds, videoSettings.frameRate);
        const normalizedSize = Layer.aspectLayerSize(iLayer, videoSettings);
        const subtitleLayer = new SubtitleLayer(iLayer, asset, (_d = iLayer.path) !== null && _d !== void 0 ? _d : "", durationInFrames, null, normalizedSize);
        subtitleLayer.videoSize = iLayer.size;
        subtitleLayer.box.aspect = (_e = iLayer.resizeMode) !== null && _e !== void 0 ? _e : Aspect.Fit;
        subtitleLayer.durationInSeconds = durationInSeconds;
        if (iLayer.subtitlesLocale) {
            subtitleLayer.subtitlesLocale = _.cloneDeep(iLayer.subtitlesLocale);
        }
        return subtitleLayer;
    }
    getILayer() {
        var _b, _c, _d;
        return Object.assign(Object.assign({}, this.toBaseILayer()), { type: LayerType.Subtitle, path: (_b = this.path) !== null && _b !== void 0 ? _b : null, size: (_c = this.videoSize) !== null && _c !== void 0 ? _c : null, mixVolume: this.volume, subtitlesLocale: (_d = this.subtitlesLocale) !== null && _d !== void 0 ? _d : null });
    }
    getDefaultSubtitle() {
        var _b, _c;
        return (_c = (_b = this.subtitlesLocale) === null || _b === void 0 ? void 0 : _b.at(0)) === null || _c === void 0 ? void 0 : _c.value;
    }
    SubtitlesLanguageExist(language) {
        if (!this.subtitlesLocale)
            return false;
        return (this.subtitlesLocale.findIndex((x) => x.key.languageCode === language) !==
            -1);
    }
    languageChanged() {
        if (!this.subtitlesLocale)
            this.subtitlesLocale = [];
        if (!this.SubtitlesLanguageExist(this.language)) {
            // Const clone = WebVTT.fromIWebVTT(_.cloneDeep(this.getDefaultSubtitle()));
            // clone.clearText();
            // this.subtitlesLocale.push({
            //   key: { languageCode: this.language },
            //   value: clone.toIWebVTT(),
            // } as IVariant<LanguageKey, IWebVTT>);
        }
    }
    getTitle() {
        if (!this.path)
            return "Subtitle";
        const url = new URL(this.path);
        const filename = url.pathname.substring(url.pathname.lastIndexOf("/") + 1);
        return `${filename}`;
    }
    toInfoString() {
        return `SUBTITLE LAYER: ${this.absoluteRange.toString()} ${this.zIndex} ${this.name} ${this.language}`;
    }
    getProperties() {
        const result = [];
        return result;
    }
    setProperty(name, value) { }
    updatePropertyValue(name, value) { }
    cueSequenceScaled(sequenceRange, frameRate) {
        const webVTT = this.subtitle ? WebVTT.fromIWebVTT(this.subtitle) : null;
        const cueRanges = webVTT === null || webVTT === void 0 ? void 0 : webVTT.cues.map((node, i) => {
            return FrameRange.fromMilliseconds(node.data.start, node.data.end, frameRate);
        });
        if (!cueRanges)
            return undefined;
        const clipPauseSequence = new FrameRangeCollection();
        let lastEnd = 0;
        const valueSequence = [];
        cueRanges.forEach((cueRange, index) => {
            if (index === 0) {
                if (cueRange.start > 0) {
                    const gapRange = FrameRange.fromStart(0, cueRange.start);
                    clipPauseSequence.append(gapRange, gapRange);
                    valueSequence.push(0);
                }
                clipPauseSequence.append(cueRange, cueRange);
                valueSequence.push(1);
                lastEnd = cueRange.end;
            }
            else {
                if (cueRange.start > lastEnd) {
                    const gapRange = FrameRange.fromStart(lastEnd, cueRange.start - lastEnd);
                    clipPauseSequence.append(gapRange, gapRange);
                    valueSequence.push(0);
                }
                clipPauseSequence.append(cueRange, cueRange);
                valueSequence.push(1);
                lastEnd = cueRange.end;
            }
        });
        return { collection: clipPauseSequence, valueSequence };
    }
    overwriteSubtitlesLocale(prior) {
        const result = produce(this, (draft) => {
            var _b, _c;
            const clone = draft;
            if (clone.subtitlesLocale)
                for (let i = 0; i < ((_b = clone.subtitlesLocale) === null || _b === void 0 ? void 0 : _b.length); i++) {
                    const subtitle = clone.subtitlesLocale[i];
                    const priorSubtitle = (_c = prior.subtitlesLocale) === null || _c === void 0 ? void 0 : _c.find((x) => x.key.languageCode === subtitle.key.languageCode);
                    if (priorSubtitle) {
                        subtitle.value = priorSubtitle.value;
                    }
                }
        });
        return result;
        // const clone = _.cloneDeep(this);
        // if (clone.subtitlesLocale)
        //   for (let i = 0; i < clone.subtitlesLocale?.length; i++) {
        //     const subtitle = clone.subtitlesLocale[i];
        //     const priorSubtitle = prior.subtitlesLocale?.find(
        //       (x) => x.key.languageCode === subtitle.key.languageCode
        //     );
        //     if (priorSubtitle) {
        //       subtitle.value = priorSubtitle.value;
        //     }
        //   }
        // return clone;
    }
    overwriteSpatialProperties(prior) {
        const result = produce(this, (draft) => {
            var _b, _c;
            const clone = draft;
            if (clone.subtitlesLocale)
                for (let i = 0; i < ((_b = clone.subtitlesLocale) === null || _b === void 0 ? void 0 : _b.length); i++) {
                    const subtitle = clone.subtitlesLocale[i];
                    const priorSubtitle = (_c = prior.subtitlesLocale) === null || _c === void 0 ? void 0 : _c.find((x) => x.key.languageCode === subtitle.key.languageCode);
                    if (priorSubtitle) {
                        const webVtt = WebVTT.fromIWebVTT(subtitle.value);
                        webVtt.overwriteSpatialProperties(priorSubtitle.value);
                        subtitle.value = webVtt.toIWebVTT();
                    }
                }
        });
        return result;
        // const clone = _.cloneDeep(this);
        // if (clone.subtitlesLocale)
        //   for (let i = 0; i < clone.subtitlesLocale?.length; i++) {
        //     const subtitle = clone.subtitlesLocale[i];
        //     const priorSubtitle = prior.subtitlesLocale?.find(
        //       (x) => x.key.languageCode === subtitle.key.languageCode
        //     );
        //     if (priorSubtitle) {
        //       const webVtt = WebVTT.fromIWebVTT(subtitle.value);
        //       webVtt.overwriteSpatialProperties(priorSubtitle.value);
        //       subtitle.value = webVtt.toIWebVTT();
        //     }
        //   }
        // return clone;
    }
    /// <summary>
    /// Merge the spatial properties of the subtitle with the other subtitle
    /// </summary>
    mergeSpatialProperties(other) {
        var _b, _c, _d, _e;
        const existingKeys = (_c = (_b = this.subtitlesLocale) === null || _b === void 0 ? void 0 : _b.map((x) => x.key)) !== null && _c !== void 0 ? _c : [];
        const newKeys = (_e = (_d = other.subtitlesLocale) === null || _d === void 0 ? void 0 : _d.map((x) => x.key)) !== null && _e !== void 0 ? _e : [];
        const missingKeys = newKeys === null || newKeys === void 0 ? void 0 : newKeys.filter((x) => !(existingKeys === null || existingKeys === void 0 ? void 0 : existingKeys.some((y) => y.languageCode === x.languageCode)));
        if (!missingKeys)
            return this;
        const result = produce(this, (draft) => {
            missingKeys.forEach((key) => {
                var _b, _c;
                const subtitle = (_b = other.subtitlesLocale) === null || _b === void 0 ? void 0 : _b.find((x) => x.key === key);
                if (subtitle) {
                    (_c = draft.subtitlesLocale) === null || _c === void 0 ? void 0 : _c.push(subtitle);
                }
            });
        });
        return result;
        // const clone = _.cloneDeep(this);
        // if (missingKeys) {
        //   missingKeys.forEach((key) => {
        //     const subtitle = other.subtitlesLocale?.find((x) => x.key === key);
        //     if (subtitle) {
        //       clone.subtitlesLocale?.push(subtitle);
        //     }
        //   });
        // }
        // return clone;
    }
}
_a = immerable;
export const SubtitleComponent = ({ sequenceRange, index, layer, path, subtitle, selectors, }) => {
    var _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    const frame = useCurrentFrame();
    const { fps, width, height, durationInFrames } = useVideoConfig();
    const pixelHeightFromFraction = (fraction, height) => {
        return `${(height * fraction).toFixed(2)}px`;
    };
    const subtitleLayer = useMemo(() => {
        const self = layer;
        if (self.videoSize)
            self.setAspectLayerSize({ width, height }, self.videoSize);
        self.spacialBox.parent = new Box({
            size: [width, height],
        });
        return self;
    }, [layer, width, height]);
    const spacialBoxStyle = useMemo(() => new BoxStyle(subtitleLayer.spacialBox).getStyleInPercent(), [subtitleLayer.spacialBox]);
    const layerDisabled = useMemo(() => subtitleLayer.renderDisabled || subtitleLayer.renderDisabledLocal, [subtitleLayer.renderDisabled, subtitleLayer.renderDisabledLocal]);
    const absoluteRange = useMemo(() => sequenceRange.round().pushInside(FrameRange.positive), [sequenceRange]);
    // #region OFFSET
    const absoluteOffset = useMemo(() => {
        var _b, _c;
        const startOffset = absoluteRange.start;
        const cutRange = (_b = layer.cut) === null || _b === void 0 ? void 0 : _b.round();
        return startOffset - ((_c = cutRange === null || cutRange === void 0 ? void 0 : cutRange.start) !== null && _c !== void 0 ? _c : 0);
    }, [absoluteRange, layer.cut]);
    // #endregion
    const opacity = useMemo(() => {
        const opacityTransitions = subtitleLayer.getTransitions("opacity", absoluteRange);
        return Transition.calculateNumberFromTransitions(opacityTransitions, frame);
    }, [subtitleLayer, absoluteRange, frame]);
    // #region SUBTITLE
    const webVTTAbsoluteFrames = useMemo(() => {
        const webVTTFromSubtitle = subtitle ? WebVTT.fromIWebVTT(subtitle) : null;
        return webVTTFromSubtitle
            ? webVTTFromSubtitle
                .withFrameTiming(fps, absoluteOffset)
                .intersectRange(absoluteRange, true)
            : null;
    }, [subtitle, fps, absoluteOffset, absoluteRange]);
    // #endregion
    // #region CUE at FRAME in Absolute Frames
    const [nodeCue, vttCue] = useMemo(() => {
        const cue = webVTTAbsoluteFrames === null || webVTTAbsoluteFrames === void 0 ? void 0 : webVTTAbsoluteFrames.getCueAtFrame(frame);
        if (cue) {
            const parsedVttCue = VTTCue.parseCueSettings(cue.data.start, cue.data.end, cue.data.text, cue.data.settings);
            return [cue, parsedVttCue];
        }
        return [undefined, undefined];
    }, [webVTTAbsoluteFrames, frame]);
    // #endregion
    // #region CUE BOX
    const cueBox = useMemo(() => {
        if (vttCue) {
            const box = vttCue.getBox(subtitleLayer.box.clone());
            box.setBox();
            return box;
        }
        return undefined;
    }, [vttCue, subtitleLayer.box]);
    // #endregion
    // #region ACTIVE CUE
    const [activeCue, trimmEnd] = useMemo(() => {
        var _b, _c;
        const cueRange = new FrameRange((_b = nodeCue === null || nodeCue === void 0 ? void 0 : nodeCue.data.start) !== null && _b !== void 0 ? _b : 0, (_c = nodeCue === null || nodeCue === void 0 ? void 0 : nodeCue.data.end) !== null && _c !== void 0 ? _c : 0);
        let trimmEnd = 0;
        if (!cueRange.isInside(absoluteRange)) {
            if (!cueRange.isIntersecting(absoluteRange)) {
                return [undefined, trimmEnd];
            }
            trimmEnd = cueRange.end - absoluteRange.end;
        }
        return [nodeCue, trimmEnd];
    }, [nodeCue, absoluteRange]);
    // #endregion
    const subtitleStyle = useMemo(() => {
        var _b;
        const paddingBottomFraction = 1 / 13;
        const paddingBottom = pixelHeightFromFraction(paddingBottomFraction, height);
        return Object.assign({ display: "flex", justifyContent: vttCue === null || vttCue === void 0 ? void 0 : vttCue.getTextAlign(), alignItems: "flex-end", paddingBottom,
            opacity, zIndex: (_b = subtitleLayer.zIndex) !== null && _b !== void 0 ? _b : 1 }, cueBox === null || cueBox === void 0 ? void 0 : cueBox.topLeftWidthHeight);
    }, [vttCue, opacity, subtitleLayer.zIndex, cueBox, height]);
    const valid = activeCue && vttCue && cueBox && absoluteRange.isValid();
    const message = `layerDisabled=${layerDisabled} absoluteRange.isValid()=${absoluteRange.isValid()} activeCue=${activeCue} vttCue=${vttCue} cueBox=${cueBox}`;
    // let styles = {
    //   fontFamily: "Arial",
    //   fontWeight: "regular",
    //   color: "white",
    //   fontSize: pixelHeightFromFraction(1 / 27, height),
    //   lineHeight: pixelHeightFromFraction(1 / 23, height),
    //   textAlign: "center",
    //   textShadow: "3px 3px 0px rgba(0,0,0,0.85)",
    //   backgroundColor: "rgba(0,0,0,0)",
    // };
    let fontFamily = "Arial";
    if (((_b = layer.styles) === null || _b === void 0 ? void 0 : _b.has) && layer.styles.has("fontFamily")) {
        fontFamily = layer.styles.get("fontFamily");
    }
    let fontWeight = "regular";
    if (((_c = layer.styles) === null || _c === void 0 ? void 0 : _c.has) && layer.styles.has("fontWeight")) {
        fontWeight = layer.styles.get("fontWeight");
    }
    let color = "white";
    if (((_d = layer.styles) === null || _d === void 0 ? void 0 : _d.has) && layer.styles.has("color")) {
        color = layer.styles.get("color");
    }
    let fontSizeFraction = 1 / 27;
    if (((_e = layer.styles) === null || _e === void 0 ? void 0 : _e.has) && layer.styles.has("fontSizeFraction")) {
        fontSizeFraction = parseFloat(layer.styles.get("fontSizeFraction"));
    }
    const fontSize = pixelHeightFromFraction(fontSizeFraction, height);
    let backgroundColor = "rgba(0,0,0,0)";
    if (((_f = layer.styles) === null || _f === void 0 ? void 0 : _f.has) && layer.styles.has("backgroundColor")) {
        backgroundColor = layer.styles.get("backgroundColor");
    }
    let lineHeightFraction = 1 / 23;
    if (((_g = layer.styles) === null || _g === void 0 ? void 0 : _g.has) && layer.styles.has("lineHeightFraction")) {
        lineHeightFraction = parseFloat(layer.styles.get("lineHeightFraction"));
    }
    const lineHeight = pixelHeightFromFraction(lineHeightFraction, height);
    // #region Text Shadow
    let textShadowFractionV = 1 / 360;
    if (((_h = layer.styles) === null || _h === void 0 ? void 0 : _h.has) && layer.styles.has("textShadowFractionV")) {
        textShadowFractionV = parseFloat(layer.styles.get("textShadowFractionV"));
    }
    const textShadowPixelV = pixelHeightFromFraction(textShadowFractionV, height);
    let textShadowFractionH = 1 / 360;
    if (((_j = layer.styles) === null || _j === void 0 ? void 0 : _j.has) && layer.styles.has("textShadowFractionH")) {
        textShadowFractionH = parseFloat(layer.styles.get("textShadowFractionH"));
    }
    const textShadowPixelH = pixelHeightFromFraction(textShadowFractionH, height);
    let textShadowFractionBlur = 0;
    if (((_k = layer.styles) === null || _k === void 0 ? void 0 : _k.has) && layer.styles.has("textShadowFractionBlur")) {
        textShadowFractionBlur = parseFloat(layer.styles.get("textShadowFractionBlur"));
    }
    const textShadowPixelBlur = pixelHeightFromFraction(textShadowFractionBlur, height);
    let textShadowColor = "rgba(0,0,0,0.85)";
    if (((_l = layer.styles) === null || _l === void 0 ? void 0 : _l.has) && layer.styles.has("textShadowColor")) {
        textShadowColor = layer.styles.get("textShadowColor");
    }
    let textShadow = `${textShadowPixelH} ${textShadowPixelV} ${textShadowPixelBlur} ${textShadowColor}`; // "2px 2px 0px rgba(0,0,0,0.85)";
    if (((_m = layer.styles) === null || _m === void 0 ? void 0 : _m.has) && layer.styles.has("textShadow")) {
        textShadow = layer.styles.get("textShadow");
    }
    // #endregion
    return (_jsx(RenderGuard, { disabled: layerDisabled || !valid, message: message, children: _jsx(disableContext.Consumer, { children: (disable) => {
                if (disable === null || disable === void 0 ? void 0 : disable.includes("SubtitleComponent"))
                    return null;
                return (_jsx(Sequence, { from: Math.max(absoluteRange.start, activeCue.data.start), durationInFrames: Math.min(absoluteRange.end, activeCue.data.end - activeCue.data.start - trimmEnd), children: _jsx("div", { style: subtitleStyle, children: _jsx("span", { className: "subtitle", style: {
                                fontFamily,
                                fontWeight,
                                color,
                                fontSize,
                                textAlign: vttCue.getTextAlign(),
                                textShadow,
                                lineHeight,
                                backgroundColor,
                                padding: "0.5em",
                                paddingTop: "0.25em",
                                paddingBottom: "0.25em",
                                borderRadius: "0.5em",
                            }, children: _jsx(Balancer, { dangerouslySetInnerHTML: { __html: vttCue.htmlText } }) }) }) }, "subtitle_" + index + "_CUE"));
            } }) }));
};
//# sourceMappingURL=subtitle.layer.js.map