import { immerable } from "immer";
import "reflect-metadata";
import type { ColorInfo, ImageInfo, LottieData, Marker } from "../types/index.js";
import type { IProperty } from "../interfaces/index.js";
import { FrameRange } from "./frameRange.js";
import { FunctionDescription } from "./function-description.js";
export declare class LottieAnimation {
    [immerable]: boolean;
    path?: string;
    lottieData: LottieData;
    get lottieDataWithoutBase64(): LottieData;
    get name(): string;
    get frameDuration(): number;
    get seconds(): number;
    get frameDurationWithoutPause(): number;
    get frameRate(): number;
    get sumOfBrakes(): number;
    constructor(data?: LottieData, path?: string | undefined);
    getBreaksRanges(startOffset?: number): FrameRange[] | undefined;
    shrink(): void;
    getTextProperties(): IProperty[];
    private getTextPropertiesFromLayers;
    private getTextPropertiesFromAssets;
    setTextProperty(name: string, newValue: string): void;
    private setTextProperyInAssets;
    private setTextProperyInLayers;
    getColorProperties(): IProperty[];
    static findColorsInArray(collection: any[], collectionName?: string, whereFn?: (layer: any) => boolean, path?: string[]): ColorInfo[];
    static findColorsInObject(item: any, objName: string, whereFn?: (layer: any) => boolean, path?: string[]): ColorInfo[];
    setColorProperty(property: IProperty): void;
    getImageProperties(): IProperty[];
    static findImagesInObject(item: any, objName: string, whereFn?: (layer: any) => boolean, path?: string[]): ImageInfo[];
    static findImagesInArray(collection: any[], collectionName?: string, whereFn?: (layer: any) => boolean, path?: string[]): ImageInfo[];
    setImageProperty(name: string, newValue: string): void;
    private setImageProperyInAssets;
    getPauseMarker(): Marker | undefined;
    getBreakMarkers(): Marker[] | undefined;
    static encodeToBase64(inputString: string): string;
    static decodeFromBase64(base64String: string): string;
    toBase64(): string;
    static fromBase64(base64: string): LottieAnimation;
    static removeBase64Data(lottieData: LottieData): LottieData;
    static removeBase64FontData(lottieData: LottieData): LottieData;
    static removeBase64ImageData(lottieData: LottieData): LottieData;
    private static dataHeader;
    static LottieDataFrom(data: string | LottieData | null | undefined): LottieData;
}
export declare function parseFunctionDescription(name: string): FunctionDescription[];
export declare function layerStartWith(layer: any, name: string): boolean;
export declare function layerContainsName(layer: any, name: string): boolean;
export declare enum LottieObjectType {
    Shape = "4|sh",
    Text = "5|t",
    Solid = "1|s",
    Image = "2|i",
    Null = "3|n",
    Camera = "6|c",
    PreComp = "0|p",
    Adjustment = "7|a",
    Audio = "8|au",
    Light = "9|l",
    Rectangle = "rc",
    Ellipse = "el",
    Path = "sh",
    Fill = "fl",
    Stroke = "st",
    Group = "gr",
    TrimPaths = "tm",
    RoundCorners = "rc",
    Repeater = "rp",
    GradientFill = "gf",
    GradientStroke = "gs",
    Star = "sr",
    Polystar = "ps",
    MergePaths = "mm"
}
export declare function normalizedArrayToHex(arr: number[]): string;
export declare function hexToNormalizedArray(hex: string): number[];
export declare function formatHex(hex: string): string;
export declare function getNestedProperty(obj: any, path: string): any | undefined;
export declare function setNestedProperty(obj: any, path: string, value: any): void;
//# sourceMappingURL=lottieAnimation.d.ts.map