var _a;
import "reflect-metadata";
import { immerable } from "immer";
import { clamp } from "../index.js";
import { aspectFill, aspectFit } from "./aspect.js";
export class NormalizedSize {
    get width() {
        return this._width;
    }
    set width(value) {
        this._width = Math.max(value, 0);
    }
    // @limit(0, 1)
    get height() {
        return this._height;
    }
    set height(value) {
        this._height = Math.max(value, 0);
    }
    constructor(width, height) {
        this[_a] = true; // Option 1
        // @limit(0, 1)
        this._width = 0;
        this._height = 0;
        this.width = width;
        this.height = height;
    }
    setWidthInPercent(percent) {
        this.width = clamp(percent, 0, 100) * 0.01;
    }
    setHeightInPercent(percent) {
        this.height = clamp(percent, 0, 100) * 0.01;
    }
    toISize() {
        return {
            width: this.width,
            height: this.height,
        };
    }
    toViewPortProjection(viewPort) {
        return {
            width: Math.round(this.width * viewPort.width),
            height: Math.round(this.height * viewPort.height),
        };
    }
    static fromISize(size) {
        return new NormalizedSize(size.width, size.height);
    }
    static fill() {
        return new NormalizedSize(1, 1);
    }
    static halfWidth() {
        return new NormalizedSize(0.5, 1);
    }
    static halfHeight() {
        return new NormalizedSize(1, 0.5);
    }
    static quarter() {
        return new NormalizedSize(0.5, 0.5);
    }
    static fromViewPort(size, viewPort) {
        return new NormalizedSize(size.width / viewPort.width, size.height / viewPort.height);
    }
    static aspectFill(size, viewPort) {
        const fill = aspectFill(size.width, size.height, viewPort.width, viewPort.height);
        return new NormalizedSize(fill.width, fill.height);
    }
    static aspectFit(size, viewPort) {
        const fit = aspectFit(size.width, size.height, viewPort.width, viewPort.height);
        return new NormalizedSize(fit.width, fit.height);
    }
}
_a = immerable;
// Function NormalizedSizeToPercentSize(normalized: NormalizedSize): PercentSize {
//   return new PercentSize(normalized.width * 100, normalized.height * 100);
// }
// function PercentSizeToNormalizedSize(percent: PercentSize): NormalizedSize {
//   return new NormalizedSize(percent.width * 0.01, percent.height * 0.01);
// }
//# sourceMappingURL=normalizedSize.js.map