import "reflect-metadata";
import { immerable } from "immer";
import type { ISize, IPosition } from "../interfaces/index.js";
import { AnchorType, PositionType, CssUnit } from "../enums/index.js";
import { CssValue } from "./cssValue.js";
import { PositionRelative } from "./rectangle-relative.js";
export declare class Position {
    x: number;
    y: number;
    anchorX: AnchorType;
    anchorY: AnchorType;
    positionType: PositionType;
    [immerable]: boolean;
    xUnit: CssUnit;
    yUnit: CssUnit;
    isNull: boolean;
    zIndex?: number;
    constructor(x: number, y: number, anchorX?: AnchorType, anchorY?: AnchorType, positionType?: PositionType);
    setXInPercent(percent: number): void;
    setYInPercent(percent: number): void;
    getOffsetX(width: CssValue): number;
    getOffsetY(height: CssValue): number;
    toString: () => string;
    textAlign: () => "center" | "left" | "right";
    toPercent(): Position;
    static(): Position;
    relative(): Position;
    fixed(): Position;
    absolute(): Position;
    sticky(): Position;
    left(): Position;
    right(): Position;
    top(): Position;
    bottom(): Position;
    xPixel(): Position;
    xPercent(): Position;
    toIPosition(): IPosition | null;
    static leftTopZero(): Position;
    static centerBottom(): Position;
    static leftTop(x: number, y: number): Position;
    static centerZero(): Position;
    static centerCenter(): Position;
    static centerTop(): Position;
    static center(x: number, y: number): Position;
    static fromIPosition(value: IPosition | null, defaultValue?: Position): Position;
    static fromUnitPositionRelative(value: PositionRelative, anchorPoint: PositionRelative): Position;
    static aspectFit(widthInput: number, heightInput: number, widthBoundary: number, heightBoundary: number): ISize;
    static aspectFill(widthInput: number, heightInput: number, widthBoundary: number, heightBoundary: number): ISize;
}
export declare function positionAnchorToUnitSpace(position: IPosition): {
    x: number;
    y: number;
};
export declare function anchorTypeXToUnitSpace(anchor: AnchorType): number;
export declare function anchorTypeYToUnitSpace(anchor: AnchorType): number;
export declare function anchorTypeToUnitSpace(x: AnchorType, y: AnchorType): {
    x: number;
    y: number;
};
export declare function anchorTypeFromUnitSpace(x: number, y: number): {
    x: AnchorType;
    y: AnchorType;
};
//# sourceMappingURL=position.d.ts.map