var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var _a;
import "reflect-metadata";
import { map } from "rxjs";
import { immerable, produce } from "immer";
import { instanceToPlain, plainToInstance, Type, Expose, plainToClass, } from "class-transformer";
import { IsString, IsBoolean, IsNumber, IsOptional, IsArray, ValidateNested, } from "class-validator";
import { DEFAULT_VIDEO_SETTINGS, } from "../interfaces/index.js";
import { subDocument } from "../index.js";
import { makeId } from "../extensions/index.js";
import { CollectionDocument } from "./collectionDocument.js";
import { EditSpecification } from "./editSpecification.js";
export class Project extends CollectionDocument {
    get missingTranslations() {
        var _b;
        return this.editSpecification.getMissingTranslations([
            this.mainLanguage,
            ...((_b = this.additionalLanguages) !== null && _b !== void 0 ? _b : []),
        ]);
    }
    constructor() {
        super();
        this[_a] = true; // Option 1
        this.subtitleActive = true;
        this.description = "";
        this.keywords = "";
        this.mainLanguage = "en";
        this.progress = 0;
        this.isFinished = false;
        this.myVideoIds = [];
        this.renderedVideos = [];
        this.id = makeId(20);
        this.name = "new Project";
        this.fallbackValues();
        this.editSpecification = new EditSpecification();
        this.editSpecification.videoSettings = Object.assign({}, DEFAULT_VIDEO_SETTINGS);
        this.videoSettings = Object.assign({}, DEFAULT_VIDEO_SETTINGS);
    }
    updateLottieAnimationDataFromPath$() {
        return this.editSpecification.updateLottieAnimationDataFromPath$(true).pipe(map((EditSpecification) => {
            return Object.assign(Object.assign({}, this), { editSpecification: EditSpecification });
        }));
    }
    unpackEditSpecification() {
        if (!this.editSpecificationJson)
            return;
        this.editSpecification = EditSpecification.fromJSON(this.editSpecificationJson);
    }
    static fromIProjectJson(json) {
        return Project.fromIProject(JSON.parse(json));
    }
    static fromIProject(iProject, unpackJson = false) {
        let clone = iProject;
        if (unpackJson) {
            clone = produce(iProject, (draftIProject) => {
                if (draftIProject.editSpecificationJson)
                    draftIProject.editSpecification = EditSpecification.fromJSON(draftIProject.editSpecificationJson);
            });
            if (!clone)
                throw new Error("iProject is undefined");
        }
        const proj = plainToInstance(Project, clone);
        proj.fallbackValues();
        return proj;
    }
    fallbackValues() {
        if (!this.id || this.id === "")
            this.id = makeId(20);
        if (!this.name || this.name === "")
            this.name = "new Project";
        if (!this.editSpecification) {
            this.editSpecification = new EditSpecification();
            this.editSpecification.videoSettings = Object.assign({}, DEFAULT_VIDEO_SETTINGS);
        }
        if (!this.videoSettings) {
            this.videoSettings = Object.assign({}, DEFAULT_VIDEO_SETTINGS);
        }
    }
    static toIProject(obj) {
        return plainToInstance(Project, obj).toIProject();
    }
    toIProject(editSpecToJson = false) {
        const iProject = instanceToPlain(this);
        if (editSpecToJson) {
            iProject.editSpecificationJson = JSON.stringify(iProject.editSpecification);
            delete iProject.editSpecification;
        }
        return iProject;
    }
    toIProjectJson() {
        const iProject = this.toIProject();
        return JSON.stringify(iProject);
    }
    groupByUser() {
        var _b, _c, _d, _e;
        return (_e = (_c = (_b = this.groups) === null || _b === void 0 ? void 0 : _b.at(0)) !== null && _c !== void 0 ? _c : (_d = this.users) === null || _d === void 0 ? void 0 : _d.at(0)) !== null && _e !== void 0 ? _e : "unknown";
    }
    addRenderedVideoRef(docRef) {
        if (this.renderedVideos.find((x) => x.id === docRef.id && x.path === docRef.path))
            return;
        this.renderedVideos.push(docRef);
    }
    setEditSpecificationIds() {
        if (this.editSpecification) {
            this.editSpecification.myProjectId = this.id;
            this.editSpecification.myVideoId = null;
        }
    }
    setEditSpecificationOutPath(path) {
        if (this.editSpecification) {
            this.editSpecification.outPath = path;
        }
    }
    clone(newId = false) {
        const clone = plainToInstance(Project, this);
        if (newId)
            clone.id = makeId(20);
        return clone;
    }
    static setEditSpecificationIds(project) {
        // const clone = _.cloneDeep(project) as Partial<IProject>;
        // if (clone.editSpecification) {
        //   clone.editSpecification.myProjectId = project.id;
        //   clone.editSpecification.myVideoId = null;
        // }
        const clone = produce(project, (draftProject) => {
            if (draftProject.editSpecification) {
                draftProject.editSpecification.myProjectId = project.id;
                draftProject.editSpecification.myVideoId = null;
            }
        });
        return clone;
    }
    static sortBy(propertyName) {
        return (a, b) => {
            if (propertyName &&
                (Object.prototype.hasOwnProperty.call(a, propertyName) ||
                    Object.prototype.hasOwnProperty.call(b, propertyName))) {
                if (propertyName === "createdAt") {
                    return CollectionDocument.sortByCreatedAt(a, b);
                }
                return CollectionDocument.sortByProperty(propertyName, a, b);
            }
            if (a < b) {
                return 1;
                // eslint-disable-next-line max-statements-per-line
            }
            if (a > b) {
                return -1;
            }
            return 0;
        };
    }
    static trackingFn(index, project) {
        return project.id;
    }
    // Convert class instance to DTO
    toDTO() {
        return instanceToPlain(this);
    }
    // Convert DTO to class instance
    static fromDTO(dto) {
        return plainToClass(Project, dto);
    }
}
_a = immerable;
__decorate([
    Expose(),
    IsString(),
    __metadata("design:type", String)
], Project.prototype, "name", void 0);
__decorate([
    Type(() => EditSpecification),
    __metadata("design:type", EditSpecification)
], Project.prototype, "editSpecification", void 0);
__decorate([
    Expose(),
    IsOptional(),
    IsString(),
    subDocument,
    __metadata("design:type", String)
], Project.prototype, "editSpecificationJson", void 0);
__decorate([
    Expose(),
    ValidateNested(),
    __metadata("design:type", Object)
], Project.prototype, "videoSettings", void 0);
__decorate([
    Expose(),
    IsBoolean(),
    __metadata("design:type", Boolean)
], Project.prototype, "subtitleActive", void 0);
__decorate([
    Expose(),
    IsString(),
    __metadata("design:type", String)
], Project.prototype, "description", void 0);
__decorate([
    Expose(),
    IsString(),
    __metadata("design:type", String)
], Project.prototype, "keywords", void 0);
__decorate([
    Expose(),
    IsString(),
    __metadata("design:type", String)
], Project.prototype, "mainLanguage", void 0);
__decorate([
    Expose(),
    IsOptional(),
    IsArray(),
    IsString({ each: true }),
    __metadata("design:type", Array)
], Project.prototype, "additionalLanguages", void 0);
__decorate([
    Expose(),
    IsNumber(),
    __metadata("design:type", Number)
], Project.prototype, "progress", void 0);
__decorate([
    Expose(),
    IsBoolean(),
    __metadata("design:type", Boolean)
], Project.prototype, "isFinished", void 0);
__decorate([
    Expose(),
    IsOptional(),
    ValidateNested(),
    __metadata("design:type", Object)
], Project.prototype, "previewStripeFile", void 0);
__decorate([
    Expose(),
    IsOptional(),
    IsArray(),
    IsString({ each: true }),
    __metadata("design:type", Array)
], Project.prototype, "libraryItems", void 0);
__decorate([
    Expose(),
    IsArray(),
    IsString({ each: true }),
    __metadata("design:type", Array)
], Project.prototype, "myVideoIds", void 0);
__decorate([
    Expose(),
    IsArray(),
    ValidateNested({ each: true }),
    __metadata("design:type", Array)
], Project.prototype, "renderedVideos", void 0);
//# sourceMappingURL=project.js.map