import { Range } from "./range.js";
export class DateRange extends Range {
    compare(a, b) {
        return a.getTime() - b.getTime();
    }
    add(a, b) {
        return new Date(a.getTime() + b.getTime());
    }
    subtract(a, b) {
        return new Date(a.getTime() - b.getTime());
    }
    multiply(a, factor) {
        return new Date(a.getTime() * factor);
    }
    divide(a, divisor) {
        return new Date(a.getTime() / divisor);
    }
    createInstance(start, end) {
        return new DateRange(start, end);
    }
    // Serialization methods
    serializeValue(value) {
        return value.toISOString();
    }
    deserializeValue(value) {
        return new Date(value);
    }
    // Overriding fromDto for DateRange
    static fromDto(dto) {
        const start = new Date(dto.start);
        const end = new Date(dto.end);
        return new DateRange(start, end);
    }
    // Override duration to return number (milliseconds)
    duration() {
        return this.subtract(this.end, this.start);
    }
    // Override shift to accept milliseconds
    shift(amount) {
        const newStart = this.add(this.start, amount);
        const newEnd = this.add(this.end, amount);
        return this.createInstance(newStart, newEnd);
    }
}
//# sourceMappingURL=dateRange.js.map