import type { IRange } from "../../interfaces";
export declare abstract class Range<T> implements IRange<T> {
    start: T;
    end: T;
    constructor(start: T, end: T);
    abstract compare(a: T, b: T): number;
    abstract add(a: T, b: T): T;
    abstract subtract(a: T, b: T): T;
    abstract multiply(a: T, factor: number): T;
    abstract divide(a: T, divisor: number): T;
    protected abstract createInstance(start: T, end: T): Range<T>;
    protected abstract serializeValue(value: T): any;
    protected abstract deserializeValue(value: any): T;
    isValid(): boolean;
    isEmpty(): boolean;
    isEqual(other: Range<T>): boolean;
    clone(): Range<T>;
    isBefore(other: Range<T>): boolean;
    isBehind(other: Range<T>): boolean;
    isIntersecting(other: Range<T>): boolean;
    isInside(other: Range<T>): boolean;
    startIsEqual(other: Range<T>): boolean;
    endIsEqual(other: Range<T>): boolean;
    containsPoint(point: T): boolean;
    duration(): T;
    deltaStart(other: Range<T>): T;
    deltaEnd(other: Range<T>): T;
    getIntersection(other: Range<T>): Range<T> | null;
    union(other: Range<T> | Range<T>[]): Range<T>[];
    difference(other: Range<T>): Range<T>[];
    shift(amount: T): Range<T>;
    scale(factor: number): Range<T>;
    protected isAdjacent(a: Range<T>, b: Range<T>): boolean;
    protected removeDuplicates(ranges: Range<T>[]): Range<T>[];
}
//# sourceMappingURL=range.d.ts.map