/// <reference types="gl-matrix/index.js" />
import "reflect-metadata";
import { vec2 } from "gl-matrix";
import { Position } from "./position.js";
export type Edges = {
    left: number;
    top: number;
    right: number;
    bottom: number;
};
export type SnapActive = {
    x: boolean;
    y: boolean;
    left: boolean;
    top: boolean;
    right: boolean;
    bottom: boolean;
};
export declare class PositionRelative {
    x: number;
    y: number;
    get vector(): vec2;
    static zero(): PositionRelative;
    static fromVector(vector: vec2): PositionRelative;
    static fromPositionAnchor(position: Position): PositionRelative;
    constructor(x: number, y: number);
    add(other: PositionRelative): PositionRelative;
    subtract(other: PositionRelative): PositionRelative;
    difference(other: PositionRelative): PositionRelative;
    clone(): PositionRelative;
}
export declare class SizeRelative {
    width: number;
    height: number;
    get vector(): vec2;
    static fromVector(vector: vec2): SizeRelative;
    constructor(width: number, height: number);
    difference(other: SizeRelative): {
        x: number;
        y: number;
    };
    clone(): SizeRelative;
}
export declare class RectangleRelative {
    private _position;
    private _size;
    private _anchorPoint;
    anchorOffset: PositionRelative;
    anchorOffsetInverted: PositionRelative;
    private _parent;
    private _boundary;
    private _children;
    private _ratioFraction;
    private _aspectRatio;
    private _keepRatio;
    static fromCoordinates(x: number, y: number, width: number, height: number, anchorX?: number, anchorY?: number): RectangleRelative;
    static unit(): RectangleRelative;
    get position(): PositionRelative;
    set position(position: PositionRelative);
    get positionVector(): vec2;
    get center(): PositionRelative;
    get size(): SizeRelative;
    set size(newSize: SizeRelative);
    get anchorPoint(): PositionRelative;
    set anchorPoint(newAnchorPoint: PositionRelative);
    moveAnchorPoint(newAnchorPoint: PositionRelative): void;
    get anchorOffsetVector(): vec2;
    get anchorOffsetInvertedVector(): vec2;
    get parent(): RectangleRelative | null;
    set parent(parent: RectangleRelative | null);
    get boundary(): RectangleRelative | null;
    set boundary(boundary: RectangleRelative | null);
    get left(): number;
    getHorizontal(unit: number): number;
    get horizontalCenter(): number;
    get right(): number;
    get top(): number;
    getVertical(unit: number): number;
    get verticalCenter(): number;
    get bottom(): number;
    get topLeft(): PositionRelative;
    get topRight(): PositionRelative;
    get bottomLeft(): PositionRelative;
    get bottomRight(): PositionRelative;
    get edges(): Edges;
    get ratioFraction(): [number, number];
    set ratioFraction(ratioFraction: [number, number]);
    get aspectRatio(): number;
    get keepRatio(): boolean;
    set keepRatio(keepRatio: boolean);
    constructor(size: SizeRelative, position: PositionRelative, anchorPoint?: PositionRelative, parent?: RectangleRelative | null, ratioFraction?: [number, number], keepRatio?: boolean);
    private setAnchorOffset;
    addChild(child: RectangleRelative): void;
    removeChild(child: RectangleRelative): void;
    getChildren(): RectangleRelative[];
    move(deltaX: number, deltaY: number, snapActive?: SnapActive, snapDistance?: number, snapSiblings?: RectangleRelative[]): void;
    private applyMoveSnapX;
    private applyMoveSnapY;
    private applyMoveSnapLeftEdge;
    private applyMoveSnapRightEdge;
    private applyMoveSnapTopEdge;
    private applyMoveSnapBottomEdge;
    private resizeFromAspectRatio;
    resize(deltaLeft: number, deltaTop: number, deltaRight: number, deltaBottom: number, snapActive?: SnapActive, snapDistance?: number, snapSiblings?: RectangleRelative[]): void;
    private applyResizeSnapLeftEdge;
    private applyResizeSnapRightEdge;
    private applyResizeSnapTopEdge;
    private applyResizeSnapBottomEdge;
    positionDifference(other: RectangleRelative): PositionRelative;
    sizeDifference(other: RectangleRelative): {
        x: number;
        y: number;
    };
    edgeDifference(other: RectangleRelative): Edges;
    edgeDifferenceAbsolute(other: RectangleRelative): Edges;
    isIntersecting(other: RectangleRelative): boolean;
    contains(other: RectangleRelative): boolean;
    isInside(other: RectangleRelative): boolean;
    intersection(other: RectangleRelative): RectangleRelative | null;
    pushInto(other: RectangleRelative): RectangleRelative;
    projectTo(target: RectangleRelative): RectangleRelative;
    calculateSubRectangle(subX: number, subY: number, subWidth: number, subHeight: number): RectangleRelative;
    subRectangleFromUnitSpace(subX: number, subY: number, subWidth: number, subHeight: number, anchorPoint?: PositionRelative): RectangleRelative;
    subRectangleInUnitSpace(input: RectangleRelative): RectangleRelative;
    clone(): RectangleRelative;
    private isDescendantOf;
    private insideSnapDistance;
}
//# sourceMappingURL=rectangle-relative.d.ts.map