import "reflect-metadata";
import { immerable } from "immer";
import { IDocumentReference, IImageSettings, IMediaAsset, IRenderedVideo, IVariant, IVideoSettings, LanguageVersionParamKey, RenderedVideoState } from "../interfaces/index.js";
import { CollectionDocument, EditSpecification } from "./index.js";
import { VariantMap } from "./variantMap.js";
import { MediaAsset } from "./mediaAsset.js";
export declare class RenderedVideo extends CollectionDocument implements IRenderedVideo {
    [immerable]: boolean;
    state: RenderedVideoState;
    projectName: string;
    description?: string;
    projectReference: IDocumentReference | null;
    editSpecification: EditSpecification;
    videoSettings: IVideoSettings | null;
    createdAsset: IMediaAsset | null;
    createdAssets: IVariant<LanguageVersionParamKey, IMediaAsset>[];
    get createdAssetsMap(): VariantMap<LanguageVersionParamKey, MediaAsset>;
    videoCount: number;
    normalizedCount: number;
    updatedByInstance?: string | null | undefined;
    ratios: IImageSettings[] | null;
    languageCodes: string[] | null;
    subtitle: {
        blank: boolean;
        withSubtitle: boolean;
    } | null;
    getAsset(expression: (key: LanguageVersionParamKey, item: IVariant<LanguageVersionParamKey, MediaAsset>) => boolean): MediaAsset | undefined;
    getAssets(): MediaAsset[];
    upsertAsset(key: LanguageVersionParamKey, asset: MediaAsset): void;
    static toIRenderedVideo(item: RenderedVideo | IRenderedVideo): IRenderedVideo;
    toIRenderedVideo(): IRenderedVideo;
    static fromIRenderedVideo(item: IRenderedVideo): RenderedVideo;
    static trackingFn(index: number, item: RenderedVideo): string;
}
//# sourceMappingURL=renderedVideo.d.ts.map