import { formatTimestamp } from "./formatTimestamp.js";
export class Formatter {
    constructor(options) {
        this.options = options;
        this.hasReceivedHeader = false;
        this.isVTT = options.format === "WebVTT";
        this.index = 1;
    }
    format(node) {
        let buffer = "";
        if (node.type === "header" && this.isVTT) {
            this.hasReceivedHeader = true;
            buffer += `${node.data}\n\n`;
        }
        if (node.type === "cue") {
            if (!this.hasReceivedHeader && this.isVTT) {
                this.hasReceivedHeader = true;
                buffer += "WEBVTT\n\n";
            }
            buffer += this.formatCue(node.data, this.index++, this.options);
        }
        return buffer;
    }
    formatCue(cue, index, options) {
        return [
            `${index > 1 ? "\n" : ""}${index}`,
            `${formatTimestamp(cue.start, options)} --> ${formatTimestamp(cue.end, options)}${options.format === "WebVTT" && cue.settings ? " " + cue.settings : ""}`,
            cue.text,
            "",
        ].join("\n");
    }
}
//# sourceMappingURL=formatter.js.map