/* eslint-disable complexity */
/* eslint-disable max-params */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import _ from "lodash";
import { ParallelExecutionStrategy, SequentialExecutionStrategy, } from "./index.js";
import { TaskExecutor } from "./taskExecutor.js";
import { lastValueFrom } from "rxjs";
import { TaskFunction, } from "../../interfaces/ITask.js";
const VERBOSE = false;
export const MOCK_ACTIVE = "mockActive";
export const MOCK_RESULT = "mockResult";
export const CANCLE_ON_NULL = "cancleOnNull";
export const NOT_PASS_OUTPUT = "notPassOutput";
export const SUCCESS_ON_ALL = "successOnAll";
export const ARG_TAG = "@arg:";
export const OUTPUT_TAG = "@output:";
export function combineTransients(mainTransient, subsequent, successOnAllSubSucesses) {
    const combinedTransient = Object.assign({}, mainTransient);
    const outputBacklogMap = new Map();
    mainTransient.outputBacklog.forEach((output) => {
        outputBacklogMap.set(`${output.name}${output.function}${output.depth}`, output);
    });
    for (const transient of subsequent) {
        transient.outputBacklog.forEach((output) => {
            outputBacklogMap.set(`${output.name}${output.function}${output.depth}`, output);
        });
        combinedTransient.argsBacklog = Object.assign(Object.assign({}, combinedTransient.argsBacklog), transient.argsBacklog);
    }
    combinedTransient.outputBacklog = Array.from(outputBacklogMap.values());
    const subsequentTraces = subsequent
        .filter((t) => Boolean(t.trace))
        .map((t) => t.trace);
    if (combinedTransient.trace && subsequentTraces.length > 0) {
        combinedTransient.trace.subsequent.push(...subsequentTraces);
    }
    const messages = subsequent
        .filter((t) => t.message)
        .map((t) => t.message);
    if (messages.length > 0) {
        combinedTransient.message = [combinedTransient.message, ...messages].join("\n");
    }
    const states = subsequent.map((t) => t.state);
    const nonPending = states.every((s) => s !== "pending");
    const allSuccess = states.every((s) => s === "success");
    const anySuccess = states.some((s) => s === "success");
    const allFailed = states.every((s) => s === "failed" || s === "cancelled");
    if (successOnAllSubSucesses) {
        combinedTransient.state = allSuccess ? "success" : "failed";
    }
    else {
        if (anySuccess && nonPending)
            combinedTransient.state = "success";
        if (allFailed)
            combinedTransient.state = "failed";
    }
    return combinedTransient;
}
export class Task {
    static registerWorker(worker) {
        Task.worker = worker;
    }
    static registerLogger(logger) {
        Task.logger = logger;
    }
    static fromITask(task) {
        var _a;
        if (task instanceof Task)
            return task;
        const newTask = new Task(task.name, task.taskFunction, task.active, task.input, task.output, task.args);
        (_a = task.subsequent) === null || _a === void 0 ? void 0 : _a.forEach((t) => newTask.addSubsequent(t));
        return newTask;
    }
    static registerFunctions(functions) {
        Task.functions = functions;
    }
    // eslint-disable-next-line max-params
    constructor(name, taskFunction, active, input, output, args, subsequent) {
        var _a;
        this._lastTask = undefined;
        this.args = {};
        this.name = name;
        this.taskFunction = taskFunction;
        this.active = active !== null && active !== void 0 ? active : true;
        this.input = input !== null && input !== void 0 ? input : null;
        this.output = output !== null && output !== void 0 ? output : null;
        this.inputFromPrevious = !input;
        this.args = args !== null && args !== void 0 ? args : {};
        this.subsequent = (_a = subsequent === null || subsequent === void 0 ? void 0 : subsequent.map((x) => Task.fromITask(x))) !== null && _a !== void 0 ? _a : [];
    }
    setParalellExecution() {
        this.args.execution = "parallel";
    }
    then(task) {
        let next;
        if (task instanceof Task) {
            next = task;
        }
        else {
            next = Task.fromITask(task);
        }
        if (this._lastTask) {
            this._lastTask.addSubsequent(next);
        }
        else {
            this.addSubsequent(next);
        }
        this._lastTask = next;
        return this;
    }
    addSubsequent(task) {
        let next;
        if (task instanceof Task) {
            next = task;
        }
        else {
            next = Task.fromITask(task);
        }
        if (!this.subsequent)
            this.subsequent = [];
        this.subsequent.push(next);
    }
    executeWithTrace(info, input, parentArgs, parentOutputs, depth = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            let rootStackTrace = {
                info,
                start: new Date(),
                end: null,
                state: "pending",
                name: this.name,
                invoker: this.invoker,
                taskFunction: this.taskFunction,
                input: input !== null && input !== void 0 ? input : this.input,
                output: "",
                args: parentArgs,
                depth,
                subsequent: [],
            };
            const startTransient = {
                value: input,
                args: {},
                state: "pending",
                outputBacklog: parentOutputs !== null && parentOutputs !== void 0 ? parentOutputs : [],
                argsBacklog: parentArgs !== null && parentArgs !== void 0 ? parentArgs : {},
                depth,
                trace: rootStackTrace,
            };
            let transientResult = {
                value: "",
                args: {},
                state: "pending",
                outputBacklog: [],
                argsBacklog: {},
                depth: 0,
            };
            try {
                transientResult = yield this.execute(startTransient);
                rootStackTrace = transientResult.trace;
            }
            catch (error) {
                rootStackTrace.state = "failed";
                rootStackTrace.error = error ? JSON.stringify(error) : "ERROR";
                transientResult = Object.assign(Object.assign({}, transientResult), { state: "failed", message: JSON.stringify(error) });
            }
            rootStackTrace.end = new Date();
            this._stackTrace = rootStackTrace;
            const traceReference = yield Task.worker.createTrace(this._stackTrace, info);
            transientResult.traceReference = traceReference;
            return transientResult;
        });
    }
    execute(transient) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const input = (_a = (this.inputFromPrevious ? transient.value : this.input)) !== null && _a !== void 0 ? _a : null;
            if (input instanceof Error) {
                return Object.assign(Object.assign({}, transient), { state: "failed" });
            }
            if (typeof input === "string" && input.toLowerCase().startsWith("error")) {
                return Object.assign(Object.assign({}, transient), { state: "failed" });
            }
            if (!transient.argsBacklog)
                Task.logger.warn("execute argsBacklog is undefined");
            if (!this.args)
                Task.logger.warn("execute this.args is undefined");
            if (!transient.argsBacklog && !this.args) {
                Task.logger.error("Task execute:  argsBacklog and this.args is undefined");
            }
            transient.outputBacklog = (_b = transient.outputBacklog) !== null && _b !== void 0 ? _b : [];
            transient.argsBacklog = Object.assign(Object.assign({}, ((_c = transient.argsBacklog) !== null && _c !== void 0 ? _c : {})), this.args);
            const parsedArgs = Object.assign(Object.assign({}, this.queryArgs(this.args, transient.argsBacklog, transient.outputBacklog)), { depth: transient.depth });
            transient.args = parsedArgs;
            const indent = transient.depth * 2;
            const parsedInput1 = this.fromOutputs(input, transient.outputBacklog);
            const parsedInput2 = this.fromArgs(parsedInput1, transient.argsBacklog);
            Task.logger.log(`${"_".repeat(indent)}TASK EXECUTE: ${this.name}___${this.taskFunction} INPUT ${parsedInput2}`);
            let outputOfFunction;
            const mockActive = transient.argsBacklog[MOCK_ACTIVE];
            const mockResult = transient.args[MOCK_RESULT];
            const fn = mockActive
                ? dummyFnFactory(this.taskFunction, mockResult)
                : (_d = Task.functions.find((f) => f.name === this.taskFunction)) === null || _d === void 0 ? void 0 : _d.fn;
            if (transient.trace) {
                transient.trace.input = parsedInput2;
                transient.trace.args = parsedArgs;
            }
            const transientResult = yield this.executeFunction(fn, `${this.name}____${this.taskFunction}`, parsedInput2, transient.args, transient);
            return transientResult;
        });
    }
    executeFunction(fn, name, input, args, transient) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            JSON.stringify(input);
            const actualTrace = {
                start: new Date(),
                end: null,
                state: "pending",
                name: this.name,
                invoker: this.invoker,
                taskFunction: this.taskFunction.toString(),
                input: JSON.stringify(input),
                output: "",
                args,
                depth: transient.depth,
                subsequent: [],
            };
            const transientResponse = Object.assign(Object.assign({}, transient), { trace: actualTrace });
            const indent = transient.depth * 2;
            const functionFound = Boolean(fn);
            const notPassOutput = args[NOT_PASS_OUTPUT];
            let functionSucceed = false;
            let functionCanceled = false;
            let functionFailed = false;
            let message = "";
            let result;
            if (fn) {
                try {
                    Task.logger.log(`${"_".repeat(indent)}TASK EXECUTE: FN ${name}`, input, args, this.output);
                    //  ######   EXECUTE FUNCTION   ######
                    result = yield fn(input, this.output, args);
                    if (args[CANCLE_ON_NULL] && result === null) {
                        functionCanceled = true;
                        transientResponse.state = "cancelled";
                        Task.logger.warn(`${"_".repeat(indent)}Task function CANCELED ${this.name}`);
                    }
                    else {
                        functionSucceed = true;
                        transientResponse.state = "success";
                    }
                }
                catch (error) {
                    Task.logger.warn(`${"_".repeat(indent)}Task function FAILED ${this.name} ERROR: ${error}`);
                    functionFailed = true;
                    transientResponse.state = "failed";
                    message = `ERROR NAME: ${error.name} \n MESSAGE: ${error.message} \n STACK: ${error.stack}`;
                }
                // #region  ######   TRACE   ######
                if (functionSucceed) {
                    actualTrace.output = (_a = JSON.stringify(result)) !== null && _a !== void 0 ? _a : "";
                    actualTrace.state = "success";
                }
                else if (functionCanceled) {
                    actualTrace.output = (_b = JSON.stringify(result)) !== null && _b !== void 0 ? _b : "";
                    actualTrace.state = "cancelled";
                }
                else if (functionFailed) {
                    actualTrace.error = message;
                    actualTrace.state = "failed";
                }
                actualTrace.end = new Date();
                transientResponse.trace = actualTrace;
                // #endregion
                if (result || result === null) {
                    if (!notPassOutput) {
                        transientResponse.value = result;
                    }
                    transientResponse.outputBacklog.push({
                        name: this.name,
                        function: this.taskFunction,
                        value: result,
                        depth: transient.depth,
                    });
                    if (VERBOSE)
                        Task.logger.log(`${"_".repeat(indent)}Task executed ${this.name}`, result);
                }
            }
            else if (this.taskFunction && this.taskFunction !== TaskFunction.none) {
                Task.logger.warn("Task function not found", this.taskFunction);
            }
            else if (this.taskFunction === TaskFunction.none) {
                transientResponse.state = "success";
                actualTrace.state = "success";
                transientResponse.trace = actualTrace;
            }
            if (functionFound &&
                (transientResponse.state === "failed" ||
                    transientResponse.state === "cancelled")) {
                return transientResponse;
            }
            if (result instanceof Error) {
                return transientResponse;
            }
            const successOnAll = Boolean(args[SUCCESS_ON_ALL]);
            const subsequentResult = yield this.executeSubsequent(Object.assign(Object.assign({}, transientResponse), { depth: transientResponse.depth + 1 }), successOnAll);
            return subsequentResult;
        });
    }
    executeSubsequent(transient, successOnAllSubSucesses) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.subsequent)
                this.subsequent = [];
            const indent = transient.depth * 2;
            const tasks = this.subsequent
                .filter((subTask) => subTask.active)
                .map((subTask) => Task.fromITask(subTask));
            const strategy = ((_a = transient.argsBacklog) === null || _a === void 0 ? void 0 : _a.execution) === "parallel"
                ? new ParallelExecutionStrategy()
                : new SequentialExecutionStrategy();
            const executor = new TaskExecutor(strategy);
            return lastValueFrom(executor.execute(tasks, transient));
        });
    }
    printPipeline(indent = 0) {
        var _a;
        Task.logger.log("# P #", "_".repeat(indent), this.name, this.taskFunction);
        (_a = this.subsequent) === null || _a === void 0 ? void 0 : _a.map((subTask) => Task.fromITask(subTask)).forEach((task) => task.printPipeline(indent + 4));
        return this;
    }
    queryArgs(args, argsBacklog, outputs) {
        const clone = _.cloneDeep(args);
        if (!args)
            return args;
        try {
            // Replace undefined args with null
            const keys = Object.keys(clone);
            keys.forEach((key) => {
                const value = clone[key];
                if (typeof value === "undefined") {
                    clone[key] = null;
                }
            });
            // Query all args values recursively
            keys.forEach((key) => {
                const value = clone[key];
                if (typeof value === "string") {
                    const fromOutput = this.fromOutputs(value, outputs);
                    const fromArgs = this.fromArgs(fromOutput, Object.assign(Object.assign({}, args), argsBacklog));
                    clone[key] = fromArgs;
                }
                else if (typeof value === "object") {
                    clone[key] = this.queryArgs(value, argsBacklog, outputs);
                }
            });
        }
        catch (error) {
            Task.logger.error("ERROR queryArgs", args, error);
        }
        return clone;
    }
    fromArgs(queryString, args) {
        if (typeof queryString !== "string")
            return queryString;
        if (!queryString.startsWith(ARG_TAG))
            return queryString;
        const argName = queryString.replace(ARG_TAG, "");
        const arg = args[argName];
        if (typeof arg === "string") {
            return arg;
        }
        Task.logger.error("argument not found", argName);
        return new Error("argument not found: " + argName);
    }
    fromOutputs(queryString, outputs) {
        if (typeof queryString !== "string")
            return queryString;
        if (!queryString.startsWith(OUTPUT_TAG))
            return queryString;
        // Check if query match = "@output:name:fn"
        const pattern = /^@output:([^:]+):([^:]+)$/;
        const match = queryString.match(pattern);
        if (!match)
            return new Error("output query does not match pattern: " + queryString);
        const split = queryString.split(":");
        const name = split[1];
        const fn = split[2];
        const output = outputs.find((o) => o.name === name && o.function === fn);
        if (output) {
            return output.value;
        }
        Task.logger.error("output not found name:", name + " fn:" + fn);
        return new Error("output not found name:: " + name + " fn:" + fn);
    }
}
Task.worker = {
    createTrace,
};
Task.logger = {
    debug: console.debug,
    info: console.info,
    log: console.log,
    warn: console.warn,
    error: console.error,
};
Task.functions = [];
export function mapFn(input, output, args) {
    return __awaiter(this, void 0, void 0, function* () {
        let innerInput = input;
        if (typeof input === "string") {
            innerInput = JSON.parse(input);
            if (!innerInput)
                return new Error("mapFn: Invalid input, expected object");
        }
        if (!(innerInput instanceof Object))
            return new Error("mapFn: Invalid input, expected object");
        const { propertyPath } = args;
        if (!propertyPath)
            return new Error("mapFn: propertyPath not found in args");
        const propertyKeys = propertyPath.split(".");
        let target = innerInput;
        for (const key in propertyKeys) {
            if (Object.prototype.hasOwnProperty.call(target, key) &&
                !(target instanceof Error)) {
                target = target[key];
                if (!target)
                    return new Error(`Property ${propertyPath} not found in object`);
            }
        }
        if (args === null || args === void 0 ? void 0 : args.stringify)
            return JSON.stringify(target);
        return target;
    });
}
export function createTrace(trace, info) {
    return __awaiter(this, void 0, void 0, function* () {
        throw new Error("No worker registered at Task");
    });
}
function dummyFnFactory(name, result) {
    return (input, output, args) => __awaiter(this, void 0, void 0, function* () {
        // Wait for 1 second
        var _a;
        const maxDelay = 2000;
        const depth = (_a = args === null || args === void 0 ? void 0 : args.depth) !== null && _a !== void 0 ? _a : 0;
        // Exponential decay of maxDelay
        const delay = maxDelay / Math.pow(2, depth);
        yield new Promise((resolve) => {
            setTimeout(resolve, delay);
        });
        Task.logger.log(`INVOKE dummy Fn ${name}`, input, output, args);
        const inputString = input;
        const outputString = `${inputString} OUT:${name};`;
        if (result) {
            if (result === "null")
                return null;
            return result;
        }
        return outputString;
    });
}
//# sourceMappingURL=task.js.map