var _a;
import "reflect-metadata";
import { immerable } from "immer";
export class Time {
    get totalMilliseconds() {
        return (this.hours * 3600000 +
            this.minutes * 60000 +
            this.seconds * 1000 +
            this.milliseconds);
    }
    get totalSeconds() {
        return (this.hours * 3600 +
            this.minutes * 60 +
            this.seconds +
            this.milliseconds / 1000);
    }
    get totalMinutes() {
        return (this.hours * 60 +
            this.minutes +
            this.seconds / 60 +
            this.milliseconds / 60000);
    }
    get totalHours() {
        return (this.hours +
            this.minutes / 60 +
            this.seconds / 3600 +
            this.milliseconds / 3600000);
    }
    constructor(hours = 0, minutes = 0, seconds = 0, milliseconds = 0) {
        this[_a] = true; // Option 1
        if (hours < 0 || hours > 23) {
            throw new Error("Hours must be between 0 and 23");
        }
        if (minutes < 0 || minutes > 59) {
            throw new Error("Minutes must be between 0 and 59");
        }
        if (seconds < 0 || seconds > 59) {
            throw new Error("Seconds must be between 0 and 59");
        }
        if (milliseconds < 0 || milliseconds > 999) {
            throw new Error("Milliseconds must be between 0 and 999");
        }
        this.hours = hours;
        this.minutes = minutes;
        this.seconds = seconds;
        this.milliseconds = milliseconds;
    }
    // #region Arithmetic
    add(time) {
        const totalSeconds = this.totalSeconds + Time.fromITime(time).totalSeconds;
        return Time.fromSeconds(totalSeconds);
    }
    subtract(time) {
        const totalSeconds = this.totalSeconds - Time.fromITime(time).totalSeconds;
        if (totalSeconds < 0) {
            throw new Error("Resulting time cannot be negative");
        }
        return Time.fromSeconds(totalSeconds);
    }
    multiply(factor) {
        const totalSeconds = this.totalSeconds * factor;
        return Time.fromSeconds(totalSeconds);
    }
    divide(divisor) {
        if (divisor === 0) {
            throw new Error("Cannot divide by zero");
        }
        const totalSeconds = this.totalSeconds / divisor;
        return Time.fromSeconds(totalSeconds);
    }
    // #endregion
    clone() {
        return new Time(this.hours, this.minutes, this.seconds, this.milliseconds);
    }
    // #region Comparison
    isEqualTo(time) {
        return this.totalMilliseconds === Time.fromITime(time).totalMilliseconds;
    }
    isLessThan(time) {
        return this.totalMilliseconds < Time.fromITime(time).totalMilliseconds;
    }
    isLessThanOrEqualTo(time) {
        return this.totalMilliseconds <= Time.fromITime(time).totalMilliseconds;
    }
    isGreaterThan(time) {
        return this.totalMilliseconds > Time.fromITime(time).totalMilliseconds;
    }
    isGreaterThanOrEqualTo(time) {
        return this.totalMilliseconds >= Time.fromITime(time).totalMilliseconds;
    }
    // #endregion
    toString() {
        const padStart = (num) => num.toString().padStart(2, "0");
        return `${padStart(this.hours)}:${padStart(this.minutes)}:${padStart(this.seconds)}`;
    }
    static fromString(timeString) {
        if (!timeString)
            return Time.min();
        const parts = timeString.split(":");
        let hours = 0;
        let minutes = 0;
        let seconds = 0;
        switch (parts.length) {
            case 0:
                break;
            case 1: // SS
                seconds = parseInt(parts[0], 10);
                break;
            case 2: // MM:SS
                minutes = parseInt(parts[0], 10);
                seconds = parseInt(parts[1], 10);
                break;
            case 3: // HH:MM:SS
                hours = parseInt(parts[0], 10);
                minutes = parseInt(parts[1], 10);
                seconds = parseInt(parts[2], 10);
                break;
            default:
                throw new Error(`fromString: Invalid time string: ${timeString}`);
        }
        return new Time(hours, minutes, seconds);
    }
    static fromITime(time) {
        var _b, _c, _d, _e;
        return new Time((_b = time.hours) !== null && _b !== void 0 ? _b : 0, (_c = time.minutes) !== null && _c !== void 0 ? _c : 0, (_d = time.seconds) !== null && _d !== void 0 ? _d : 0, (_e = time.milliseconds) !== null && _e !== void 0 ? _e : 0);
    }
    static min() {
        return new Time(0, 0, 0);
    }
    static max() {
        return new Time(23, 59, 59, 999);
    }
    static fromMilliseconds(milliseconds) {
        milliseconds = Math.round(milliseconds);
        const hours = Math.floor(milliseconds / 3600000);
        milliseconds %= 3600000;
        const minutes = Math.floor(milliseconds / 60000);
        milliseconds %= 60000;
        const seconds = Math.floor(milliseconds / 1000);
        milliseconds %= 1000;
        return new Time(hours, minutes, seconds, milliseconds);
    }
    static fromSeconds(seconds) {
        seconds = Math.round(seconds);
        const hours = Math.floor(seconds / 3600);
        seconds %= 3600;
        const minutes = Math.floor(seconds / 60);
        seconds %= 60;
        return new Time(hours, minutes, seconds);
    }
    static fromMinutes(minutes) {
        const hours = Math.floor(minutes / 60);
        minutes %= 60;
        const seconds = 0;
        return new Time(hours, minutes, seconds);
    }
    static fromHours(hours) {
        const minutes = 0;
        const seconds = 0;
        return new Time(hours, minutes, seconds);
    }
    static fromNumberRange(number, numberSpace, duration) {
        if (number < 0) {
            return Time.min();
        }
        if (number > numberSpace) {
            return duration.clone();
        }
        const factor = number / numberSpace;
        return duration.multiply(factor);
    }
}
_a = immerable;
//# sourceMappingURL=time.js.map