var _a;
import _ from "lodash";
import "reflect-metadata";
import { immerable } from "immer";
export const LOCKED = "LOCKED";
export class VariantMapMapper {
    get default() {
        return this._default;
    }
    get keys() {
        return this.collection.map((item) => this.fromDto(item).key);
    }
    get values() {
        return this.collection.map((item) => this.fromDto(item).value);
    }
    get hasValues() {
        return this.collection.length > 0;
    }
    get length() {
        return this.collection.length;
    }
    constructor(dtoMap, fromDto, toDto) {
        this[_a] = true; // Option 1
        this.collection = [];
        this.fromDto = (dto) => {
            return dto;
        };
        this.toDto = (item) => {
            return item;
        };
        this.fromDtoKey = (key) => {
            return this.fromDto({ key, value: {} }).key;
        };
        this.toDtoKey = (key) => {
            return this.toDto({ key, value: {} }).key;
        };
        this._default = undefined;
        if (dtoMap && Array.isArray(dtoMap)) {
            this.initMap(dtoMap);
        }
        if (fromDto) {
            this.fromDto = fromDto;
        }
        if (toDto) {
            this.toDto = toDto;
        }
    }
    setDefault(defaultValue) {
        this._default = defaultValue;
    }
    initMap(map) {
        this.collection = [...map];
    }
    exists(key) {
        return this.collection.some((dto) => _.isEqual(dto.key, this.toDtoKey(key)));
    }
    find(expression) {
        const variant = this.collection.find((item) => expression(this.fromDtoKey(item.key), this.fromDto(item)));
        return variant ? this.fromDto(variant) : undefined;
    }
    get(key, createIfMissing) {
        var _b;
        const variant = this.collection.find((item) => _.isEqual(item.key, this.toDtoKey(key)));
        if (variant) {
            return this.fromDto(variant).value;
        }
        if (createIfMissing) {
            const added = this.add(key, _.cloneDeep((_b = this._default) !== null && _b !== void 0 ? _b : {}));
            return added ? added.value : null;
        }
        return null;
    }
    add(key, value) {
        if (this.exists(key))
            throw new Error(`Key ${key} already exists`);
        this.collection.push(this.toDto({ key, value }));
        const added = this.collection.at(-1);
        return added ? this.fromDto(added) : undefined;
    }
    update(key, value) {
        const dto = this.toDto({ key, value });
        const existing = this.collection.find((item) => _.isEqual(item.key, dto.key));
        if (existing && (existing === null || existing === void 0 ? void 0 : existing.value)[LOCKED] === true)
            return false;
        if (existing) {
            existing.value = dto.value;
            return true;
        }
        return false;
    }
    set(key, value) {
        const dto = this.toDto({ key, value });
        const existing = this.collection.find((item) => _.isEqual(item.key, dto.key));
        if (existing && (existing === null || existing === void 0 ? void 0 : existing.value)[LOCKED] === true)
            return false;
        return this.upsert(key, value);
    }
    upsert(key, value) {
        const dto = this.toDto({ key, value });
        const existing = this.collection.find((item) => _.isEqual(item.key, dto.key));
        if (existing && (existing === null || existing === void 0 ? void 0 : existing.value)[LOCKED] === true)
            return false;
        if (existing) {
            existing.value = dto.value;
            return true;
        }
        return Boolean(this.add(key, value));
    }
    remove(key) {
        const variant = this.collection.find((item) => _.isEqual(item.key, this.toDtoKey(key)));
        if (variant && (variant === null || variant === void 0 ? void 0 : variant.value)[LOCKED] === true)
            return false;
        const index = this.collection.findIndex((item) => _.isEqual(item.key, this.toDtoKey(key)));
        if (index >= 0) {
            this.collection.splice(index, 1);
            return true;
        }
        return false;
    }
    cloneItem(key, newKey) {
        if (this.exists(newKey))
            throw new Error(`Key ${newKey} already exists`);
        const variant = this.collection.find((item) => _.isEqual(item.key, key));
        if (variant) {
            const copy = this.fromDto(variant).value;
            if (copy[LOCKED]) {
                delete copy[LOCKED];
            }
            return Boolean(this.add(newKey, copy));
        }
        return false;
    }
    // #endregion modify
    static fromIVariantArray(variants, fromDto, toDto) {
        const map = new VariantMapMapper(variants);
        map.fromDto = fromDto;
        map.toDto = toDto;
        return map;
    }
}
_a = immerable;
//# sourceMappingURL=variantMapMapper.js.map