import "reflect-metadata";
import { immerable } from "immer";
import { PixelFormat, ProResProfile, VideoCodecType, VideoFileType, VideoImageFormatType, VideoRatioType, VideoResolution, Aspect } from "../enums/index.js";
import { LottieData } from "../types/index.js";
import { type ISize, type IVideoSettings } from "../interfaces/index.js";
export declare class VideoSettings implements IVideoSettings {
    [immerable]: boolean;
    isLandscape: boolean;
    frameRate: number;
    resolution: VideoResolution;
    ratio: VideoRatioType;
    ratioFraction: [number, number];
    width: number;
    height: number;
    durationInFrames: number;
    videoCodec: VideoCodecType;
    videoImageFormat: VideoImageFormatType;
    pixelFormat: PixelFormat;
    proresProfile: ProResProfile | null;
    fileType: VideoFileType;
    alphaChannel: boolean;
    get iSize(): ISize;
    constructor(init?: Partial<VideoSettings>);
    aspectSize(size: ISize, aspect?: Aspect): ISize;
    sizeWithRatio(width: number): ISize;
    calculateSize(): void;
    toIVideoSettings(): IVideoSettings;
    static toSuffix(iSettings: IVideoSettings): string;
    toSuffix(): string;
    ratioToString(): string;
    static compatible(a: IVideoSettings, b: IVideoSettings, checkOrientation?: boolean, checkRatio?: boolean, checkResolution?: boolean): {
        compatible: boolean;
        reason: string;
    };
    static equalOrientation(a: IVideoSettings, b: IVideoSettings): boolean;
    static equalResolution(a: IVideoSettings, b: IVideoSettings): boolean;
    static equalRatio(a: IVideoSettings, b: IVideoSettings): boolean;
    static ratioToString(ratio: [number, number], isLandscape: boolean): string;
    static fromLottieData(data: LottieData): VideoSettings;
    static fromIVideoSettings(data: IVideoSettings): VideoSettings;
    static fromMetaData(width: number, height: number, ratio: VideoRatioType, frameRate: number, durationInSeconds: number): VideoSettings;
    static default(): VideoSettings;
    static sizeFromVideoSettings(videoSettings: IVideoSettings): ISize;
    static resolutionFromSize(size: ISize): VideoResolution;
    static getResolutions(ratio: VideoRatioType): Map<VideoResolution, ISize>;
    static getThumbnailResolutions(ratio: VideoRatioType, isLandscape: boolean): ISize;
    static greatestCommonDivisor(a: number, b: number): number;
    static reduceFraction(numerator?: number, denominator?: number): [number, number];
    static getRatio(width?: number, height?: number): number;
    static smallestDenominator(num: number): number;
    static fractionFromNumber(num: number): [number, number];
    static fractionFromVideoRatioType(ratio: VideoRatioType): [number, number];
    static differenece(a: IVideoSettings, b: IVideoSettings, properties?: (keyof IVideoSettings)[]): Map<keyof IVideoSettings, {
        a: any;
        b: any;
    }>;
    static updateAfromB(a: IVideoSettings, b: IVideoSettings, properties: (keyof IVideoSettings)[]): IVideoSettings;
}
//# sourceMappingURL=videoSettings.d.ts.map