import { jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
import { useState } from "react";
import { Composition } from "remotion";
import { EditSpecification } from "./models/index.js";
import { VideoStrip, calculateMetadata, calculateClips, } from "./models/components/video-strip.js";
import { VideoResolution, VideoRatioType, VideoCodecType, VideoImageFormatType, PixelFormat, VideoFileType, } from "./enums/index.js";
const videoSettings = {
    isLandscape: true,
    frameRate: 24,
    resolution: VideoResolution.HD,
    ratio: VideoRatioType.R16To9,
    ratioFraction: [16, 9],
    width: 1920,
    height: 1080,
    durationInFrames: 830,
    videoCodec: VideoCodecType.H264,
    videoImageFormat: VideoImageFormatType.JPEG,
    pixelFormat: PixelFormat.yuv420p,
    proresProfile: null,
    fileType: VideoFileType.mp4,
    alphaChannel: false,
};
export const RemotionVideo = () => {
    const [editSpecificationJson, setEditSpecificationJson] = useState("");
    const [editSpecification, setEditSpecification] = useState(new EditSpecification());
    const [setting, setSetting] = useState({
        isLandscape: true,
        frameRate: 24,
        resolution: VideoResolution.HD,
        ratio: VideoRatioType.R16To9,
        width: 1920,
        height: 1080,
        durationInFrames: 24,
    });
    const props = {
        editSpecificationJson,
        editSpecification,
        videoSettings,
        clips: [],
    };
    const calculatedClips = calculateClips(props);
    const [clips, setClips] = useState(calculatedClips);
    return (_jsx(_Fragment, { children: _jsx(Composition, { id: "VideoStrip", component: VideoStrip, durationInFrames: setting.durationInFrames, fps: setting.frameRate, width: setting.width, height: setting.height, calculateMetadata: calculateMetadata, 
            // You can override these props for each render:
            // https://www.remotion.dev/docs/parametrized-rendering
            defaultProps: {
                editSpecificationJson,
                editSpecification,
                videoSettings: setting,
                clips,
            } }) }));
};
//# sourceMappingURL=remotionVideo.js.map